var proxmoxOnlineHelpInfo = {
   "getting_help" : {
      "link" : "/pmg-docs/pmg-admin-guide.html#getting_help",
      "title" : "Getting Help"
   },
   "pmg_documentation_index" : {
      "link" : "/pmg-docs/index.html",
      "title" : "Proxmox VE Documentation Index"
   }
};
/*global Proxmox */
Ext.ns('PMG');

console.log("Starting PMG Manager");


Ext.define('PMG.Utils', {
    singleton: true,

    // this singleton contains miscellaneous utilities

    senderText: gettext('Sender'),
    receiverText: gettext('Receiver'),
    scoreText: gettext('Score'),

    user_role_text: {
	root: gettext('Superuser'),
	admin: gettext('Administrator'),
	qmanager: gettext('Quarantine Manager'),
	audit: gettext('Auditor')
    },

    format_user_role: function(role) {
	return PMG.Utils.user_role_text[role] || role;
    },

    oclass_text: {
	who: gettext('Who Objects'),
	what: gettext('What Objects'),
	when: gettext('When Objects'),
	action: gettext('Action Objects'),
	from: gettext('From'),
	to: gettext('To')
    },

    oclass_icon: {
	who: '<span class="fa fa-fw fa-user-circle"></span> ',
	what: '<span class="fa fa-fw fa-cube"></span> ',
	when: '<span class="fa fa-fw fa-clock-o"></span> ',
	action: '<span class="fa fa-fw fa-flag"></span> ',
	from: '<span class="fa fa-fw fa-user-circle"></span> ',
	to: '<span class="fa fa-fw fa-user-circle"></span> '
    },

    mail_status_map: {
	2: 'delivered',
	4: 'deferred',
	5: 'bounced',
	N: 'rejected',
	G: 'greylisted',
	A: 'accepted',
	B: 'blocked',
	Q: 'quarantine'
    },

    icon_status_map_class: {
	2: 'check-circle',
	4: 'clock-o',
	5: 'mail-reply',
	N: 'times-circle',
	G: 'list',
	A: 'check',
	B: 'ban',
	Q: 'cube'
    },

    icon_status_map_color: {
	2: 'green',
	5: 'gray',
	A: 'green',
	B: 'red'
    },

    format_status_icon: function(status) {
	var icon = PMG.Utils.icon_status_map_class[status] || 'question-circle';
	var color = PMG.Utils.icon_status_map_color[status] || '';
	return '<i class="fa fa-' + icon + ' ' + color + '"></i> ';
    },

    format_oclass: function(oclass) {
	var icon = PMG.Utils.oclass_icon[oclass] || '';
	var text = PMG.Utils.oclass_text[oclass] || oclass;
	return icon + text;
    },

    rule_direction_text: {
	0: gettext('In'),
	1: gettext('Out'),
	2: gettext('In & Out')
    },

    rule_direction_icon: {
	0: '<span class="fa fa-fw fa-long-arrow-left"></span> ',
	1: '<span class="fa fa-fw fa-long-arrow-right"></span> ',
	2: '<span class="fa fa-fw fa-exchange"></span> '
    },

    format_rule_direction: function(dir) {
	var icon = PMG.Utils.rule_direction_icon[dir] || '';
	var text = PMG.Utils.rule_direction_text[dir] || dir;
	return icon + text;
    },

    format_otype: function(otype) {
	var editor = PMG.Utils.object_editors[otype];
	var iconCls = 'fa fa-question-circle';
	if (editor) {
	    var icon = '<span class="fa-fw ' + (editor.iconCls || iconCls) + '"></span> ';
	    return icon + editor.subject;
	}

	return '<span class="fa-fw ' + iconCls + '"></span> unknown';
    },

    format_ldap_protocol: function(p) {
	if (p === undefined) { return 'LDAP'; }
	if (p === 'ldap') { return 'LDAP'; }
	if (p === 'ldaps') { return 'LDAPS'; }
	return 'unknown';
    },

    convert_field_to_per_min: function(value, record) {
	return (value/(record.data.timespan/60));
    },

    object_editors: {
	1000: {
	    iconCls: 'fa fa-filter',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'regex',
	    subject: gettext("Regular Expression"),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'regex',
		    labelWidth: 150,
		    reference: 'regex',
		    fieldLabel: gettext("Regular Expression")
		},
		{
		    labelWidth: 150,
		    fieldLabel: gettext('Test String'),
		    xtype: 'pmgRegexTester',
		    regexFieldReference: 'regex'
		}
	    ]
	},
	1005: {
	    iconCls: 'fa fa-users',
	    xtype: 'pmgLDAPGroupEditor',
	    subdir: 'ldap',
	    subject: gettext("LDAP Group")
	},
	1006: {
	    iconCls: 'fa fa-user',
	    xtype: 'pmgLDAPUserEditor',
	    subdir: 'ldapuser',
	    subject: gettext("LDAP User")
	},
	1009: {
	    iconCls: 'fa fa-filter',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'receiver_regex',
	    subject: gettext("Regular Expression"),
	    receivertest: true,
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'regex',
		    labelWidth: 150,
		    fieldLabel: gettext("Regular Expression")
		}
	    ]
	},
	1001: {
	    iconCls: 'fa fa-envelope-o',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'email',
	    subject: gettext("Email"),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'email',
		    fieldLabel: gettext("Email")
		}
	    ]
	},
	1007: {
	    iconCls: 'fa fa-envelope-o',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'receiver',
	    subject: gettext("Email"),
	    receivertest: true,
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'email',
		    fieldLabel: gettext("Email")
		}
	    ]
	},
	1002: {
	    iconCls: 'fa fa-globe',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'domain',
	    subject: gettext("Domain"),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'domain',
		    fieldLabel: gettext("Domain")
		}
	    ]
	},
	1008: {
	    iconCls: 'fa fa-globe',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'receiver_domain',
	    subject: gettext("Domain"),
	    receivertest: true,
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'domain',
		    fieldLabel: gettext("Domain")
		}
	    ]
	},
	1003: {
	    iconCls: 'fa fa-globe',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'ip',
	    subject: gettext("IP Address"),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'ip',
		    fieldLabel: gettext("IP Address")
		}
	    ]
	},
	1004: {
	    iconCls: 'fa fa-globe',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'network',
	    subject: gettext("IP Network"),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'cidr',
		    fieldLabel: gettext("IP Network")
		}
	    ]
	},
	2000: {
	    iconCls: 'fa fa-clock-o',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'timeframe',
	    subject: gettext("TimeFrame"),
	    items: [
		{
		    xtype: 'timefield',
		    name: 'start',
		    format: 'H:i',
		    fieldLabel: gettext("Start Time")
		},
		{
		    xtype: 'timefield',
		    name: 'end',
		    format: 'H:i',
		    fieldLabel: gettext("End Time")
		}
	    ]
	},
	3000: {
	    iconCls: 'fa fa-bullhorn',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'spamfilter',
	    subject: gettext('Spam Filter'),
	    items: [
		{
		    xtype: 'proxmoxintegerfield',
		    name: 'spamlevel',
		    allowBlank: false,
		    minValue: 0,
		    fieldLabel: gettext('Level')
		}
	    ]
	},
	3001: {
	    iconCls: 'fa fa-bug',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'virusfilter',
	    subject: gettext('Virus Filter'),
	    uneditable: true,
	    // there are no parameters to give, so we simply submit it
	    listeners: {
		show: function(win) {
		    win.submit();
		}
	    }
	},
	3002: {
	    iconCls: 'fa fa-code',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'matchfield',
	    subject: gettext('Match Field'),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'field',
		    labelWidth: 150,
		    allowBlank: false,
		    fieldLabel: gettext('Field')
		},
		{
		    xtype: 'textfield',
		    reference: 'value',
		    name: 'value',
		    labelWidth: 150,
		    allowBlank: false,
		    fieldLabel: gettext('Value')
		},
		{
		    labelWidth: 150,
		    fieldLabel: gettext('Test String'),
		    xtype: 'pmgRegexTester',
		    regexFieldReference: 'value'
		}
	    ]
	},
	3003: {
	    iconCls: 'fa fa-file-image-o',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'contenttype',
	    width: 400,
	    subject: gettext('Content Type Filter'),
	    items: [
		{
		    xtype: 'combobox',
		    displayField: 'text',
		    labelWidth: 150,
		    valueField: 'mimetype',
		    name: 'contenttype',
		    editable: true,
		    queryMode: 'local',
		    store: {
			autoLoad: true,
			proxy: {
			    type: 'proxmox',
			    url: '/api2/json/config/mimetypes'
			}
		    },
		    fieldLabel: gettext('Content Type'),
		    anyMatch: true,
		    matchFieldWidth: false,
		    listeners: {
			change: function(cb, value) {
			    var me = this;
			    me.up().down('displayfield').setValue(value);
			}
		    }
		},
		{
		    xtype: 'displayfield',
		    fieldLabel: gettext('Value'),
		    labelWidth: 150,
		    allowBlank: false,
		    reset: Ext.emptyFn
		}
	    ]
	},
	3004: {
	    iconCls: 'fa fa-file-o',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'filenamefilter',
	    width: 400,
	    subject: gettext('Match Filename'),
	    items: [
		{
		    xtype: 'textfield',
		    name: 'filename',
		    reference: 'filename',
		    fieldLabel: gettext('Filename'),
		    labelWidth: 150,
		    allowBlank: false
		},
		{
		    labelWidth: 150,
		    fieldLabel: gettext('Test String'),
		    wholeMatch: true,
		    xtype: 'pmgRegexTester',
		    regexFieldReference: 'filename'
		}
	    ]
	},
	3005: {
	    iconCls: 'fa fa-file-archive-o',
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'archivefilter',
	    width: 400,
	    subject: gettext('Archive Filter'),
	    items: [
		{
		    xtype: 'combobox',
		    displayField: 'text',
		    labelWidth: 150,
		    valueField: 'mimetype',
		    name: 'contenttype',
		    editable: true,
		    queryMode: 'local',
		    store: {
			autoLoad: true,
			proxy: {
			    type: 'proxmox',
			    url: '/api2/json/config/mimetypes'
			}
		    },
		    fieldLabel: gettext('Content Type'),
		    anyMatch: true,
		    matchFieldWidth: false,
		    listeners: {
			change: function(cb, value) {
			    var me = this;
			    me.up().down('displayfield').setValue(value);
			}
		    }
		},
		{
		    xtype: 'displayfield',
		    fieldLabel: gettext('Value'),
		    labelWidth: 150,
		    allowBlank: false,
		    reset: Ext.emptyFn
		}
	    ]
	},
	4002: {
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'notification',
	    subject: gettext('Notification'),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'name',
		    allowBlank: false,
		    fieldLabel: gettext('Name')
		},
		{
		    xtype: 'textareafield',
		    name: 'info',
		    fieldLabel: gettext("Description")
		},
		{
		    xtype: 'textfield',
		    name: 'to',
		    allowBlank: false,
		    value: '__ADMIN__',
		    fieldLabel: gettext('Receiver')
		},
		{
		    xtype: 'textfield',
		    name: 'subject',
		    allowBlank: false,
		    value: 'Notification: __SUBJECT__',
		    fieldLabel: gettext('Subject')
		},
		{
		    xtype: 'textarea',
		    name: 'body',
		    allowBlank: false,
		    grow: true,
		    growMax: 250,
		    value:
			"Proxmox Notifcation:\n\n" +
			"Sender:   __SENDER__\n" +
			"Receiver: __RECEIVERS__\n" +
			"Targets:  __TARGETS__\n\n" +
			"Subject:  __SUBJECT__\n\n" +
			"Matching Rule: __RULE__\n\n" +
			"__RULE_INFO__\n\n" +
			"__VIRUS_INFO__\n" +
			"__SPAM_INFO__\n",
		    fieldLabel: gettext('Body')
		},
		{
		    xtype: 'proxmoxcheckbox',
		    name: 'attach',
		    fieldLabel: gettext("Attach orig. Mail")
		}
	    ]
	},
	4003: {
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'field',
	    subject: gettext('Header Attribute'),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'name',
		    allowBlank: false,
		    fieldLabel: gettext('Name')
		},
		{
		    xtype: 'textareafield',
		    name: 'info',
		    fieldLabel: gettext("Description")
		},
		{
		    xtype: 'textfield',
		    name: 'field',
		    allowBlank: false,
		    fieldLabel: gettext('Field')
		},
		{
		    xtype: 'textfield',
		    reference: 'value',
		    name: 'value',
		    allowBlank: false,
		    fieldLabel: gettext('Value')
		}
	    ]
	},
	4005: {
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'bcc',
	    subject: gettext('BCC'),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'name',
		    allowBlank: false,
		    fieldLabel: gettext('Name')
		},
		{
		    xtype: 'textareafield',
		    name: 'info',
		    fieldLabel: gettext("Description")
		},
		{
		    xtype: 'textfield',
		    name: 'target',
		    allowBlank: false,
		    fieldLabel: gettext("Target")
		},
		{
		    xtype: 'proxmoxcheckbox',
		    checked: true,
		    name: 'original',
		    fieldLabel: gettext("send orig. Mail")
		}
	    ]

	},
	4007: {
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'removeattachments',
	    subject: gettext('Remove Attachments'),
	    width: 500,
	    fieldDefaults: {
		labelWidth: 150
	    },
	    items: [
		{
		    xtype: 'textfield',
		    name: 'name',
		    allowBlank: false,
		    fieldLabel: gettext('Name')
		},
		{
		    xtype: 'textareafield',
		    name: 'info',
		    fieldLabel: gettext("Description")
		},
		{
		    xtype: 'textareafield',
		    name: 'text',
		    grow: true,
		    growMax: 250,
		    fieldLabel: gettext("Text Replacement")
		},
		{
		    xtype: 'proxmoxcheckbox',
		    checked: true,
		    name: 'all',
		    fieldLabel: gettext("Remove all attachments")
		}
	    ]
	},
	4009: {
	    xtype: 'proxmoxWindowEdit',
	    subdir: 'disclaimer',
	    subject: gettext('Disclaimer'),
	    width: 400,
	    items: [
		{
		    xtype: 'textfield',
		    name: 'name',
		    allowBlank: false,
		    fieldLabel: gettext('Name')
		},
		{
		    xtype: 'textareafield',
		    name: 'info',
		    fieldLabel: gettext("Description")
		},
		{
		    xtype: 'textareafield',
		    name: 'disclaimer',
		    grow: true,
		    growMax: 250,
		    fieldLabel: gettext("Disclaimer")
		}
	    ]
	}
    },

    updateLoginData: function(data) {
	Proxmox.CSRFPreventionToken = data.CSRFPreventionToken;
	Proxmox.UserName = data.username;
	Ext.util.Cookies.set('PMGAuthCookie', data.ticket, null, '/', null, true );
    },

    quarantineActionExtracted: false,

    extractQuarantineAction: function() {

	if (PMG.Utils.quarantineActionExtracted) { return; }

	PMG.Utils.quarantineActionExtracted = true;

	var qs = Ext.Object.fromQueryString(location.search);

	var cselect = qs.cselect;
	var action = qs.action;
	var ticket = qs.ticket;
	var dateString = qs.date;

	if (dateString) {
	    var date = new Date(dateString).getTime()/1000;

	    // set from date for QuarantineList
	    /*jslint confusion: true*/
	    /*from is a string above and number here */
	    PMG.QuarantineList.from = date;
	    /*jslint confusion: false*/
	}

	delete qs.cselect;
	delete qs.action;
	delete qs.ticket;
	delete qs.date;

	var newsearch = Ext.Object.toQueryString(qs);

	var newurl = location.protocol + "//" + location.host + location.pathname;
	if (newsearch) { newurl += '?' + newsearch; }
	newurl += location.hash;

	if (window.history) {
	    window.history.pushState({ path:newurl }, '', newurl);
	}

	if (action || cselect) {
	    return { action: action, cselect: cselect };
	}
    },

    doQuarantineAction: function(action, id, callback) {
	Proxmox.Utils.API2Request({
	    url: '/quarantine/content/',
	    params: {
		action: action,
		id: id
	    },
	    method: 'POST',
	    failure: function(response, opts) {
		Ext.Msg.alert(gettext('Error'), response.htmlStatus);
	    },
	    success: function(response, opts) {
		var win = Ext.create('Ext.window.MessageBox',{
		    closeAction: 'destroy'
		}).show({
		    title: gettext('Info'),
		    message: "Action '" + action + ' ' +
		    id + "' successful",
		    buttons: Ext.Msg.OK,
		    icon: Ext.MessageBox.INFO
		});

		if (Ext.isFunction(callback)) {
		    callback();
		}
	    }
	});
    },

    sender_renderer: function(value, metaData, rec) {
	var subject = Ext.htmlEncode(value);
	var from = Ext.htmlEncode(rec.data.from);
	var sender = Ext.htmlEncode(rec.data.sender);
	if (sender) {
	    /*jslint confusion: true*/
	    /*format is a string above*/
	    from = Ext.String.format(gettext("{0} on behalf of {1}"),
				     sender, from);
	    /*jslint confusion: false*/
	}
	return '<small>' + from + '</small><br>' + subject;
    },

    constructor: function() {
	var me = this;

	// do whatever you want here
    }
});
Ext.define('PMG.FilterProxy', {
    extend: 'Proxmox.RestProxy',
    alias : 'proxy.pmgfilterproxy',

    filterId: undefined, // 'x-gridfilter-XXXXX'

    getParams: function(operation) {
	var me = this, i;
	if (!operation.isReadOperation) {
	    return {};
	}
	var params = me.callParent(arguments);

	var filters = operation.getFilters() || [];
	for (i = 0; i < filters.length; i++) {
	    var filter = filters[i];
	    if (filter.config.id === me.filterId) {
		var v = filter.getValue();
		if (v !== undefined && v !== '') {
		    /*jslint confusion: true */
		    params.filter = v;
		    /*jslint confusion: false */
		}
	    }
	}
	return params;
    }
});
Ext.define('PMG.LoginView', {
    extend: 'Ext.container.Container',
    xtype: 'loginview',

    controller: {
	xclass: 'Ext.app.ViewController',

	init: function(view) {
	    var me = this;

	    var realmfield = me.lookup('realmfield');

	    if (view.targetview !== 'quarantineview') {
		return;
	    }

	    realmfield.setValue('quarantine');

	    // try autologin with quarantine ticket from URL

	    var loginForm = this.lookupReference('loginForm');

	    var qs = Ext.Object.fromQueryString(location.search);
	    if (qs.ticket == undefined) { return; }
	    var ticket = decodeURIComponent(qs.ticket);
	    var match = ticket.match(/^PMGQUAR:([^\s\:]+):/);
	    if (!match) { return; }
	    var username = match[1];
	    var loginwin = me.lookup('loginwindow');
	    loginwin.autoShow = false;
	    loginwin.setVisible(false);
	    realmfield.setDisabled(true);

	    me.lookup('usernameField').setValue(username);
	    me.lookup('passwordField').setValue(ticket);

	    me.submitForm();
	},

	submitForm: function() {
	    var me = this;
	    var view = me.getView();
	    var loginForm = me.lookupReference('loginForm');

	    if (loginForm.isValid()) {
		if (loginForm.isVisible()) {
		    loginForm.mask(gettext('Please wait...'), 'x-mask-loading');
		}
		loginForm.submit({
		    success: function(form, action) {
			// save login data and create cookie
			PMG.Utils.updateLoginData(action.result.data);
			PMG.app.changeView(view.targetview);
		    },
		    failure: function(form, action) {
			loginForm.unmask();
			Ext.MessageBox.alert(
			    gettext('Error'),
			    gettext('Login failed. Please try again')
			);
		    }
		});
	    }
	},

	control: {
	    'field[name=lang]': {
		change: function(f, value) {
		    var dt = Ext.Date.add(new Date(), Ext.Date.YEAR, 10);
		    Ext.util.Cookies.set('PMGLangCookie', value, dt);

		    var loginwin = this.lookupReference('loginwindow');
		    loginwin.mask(gettext('Please wait...'), 'x-mask-loading');
		    window.location.reload();
		}
	    },
	    'button[reference=loginButton]': {
		click: 'submitForm'
	    }
	}
    },

    plugins: 'viewport',

    layout: {
	type: 'border'
    },

    items: [
	{
	    region: 'north',
	    xtype: 'container',
	    layout: {
		type: 'hbox',
		align: 'middle'
	    },
	    margin: '2 5 2 5',
	    height: 38,
	    items: [
		{
		    xtype: 'proxmoxlogo'
		},
		{
		    xtype: 'versioninfo',
		    makeApiCall: false
		}
	    ]
	},
	{
	    region: 'center'
	},
	{
	    xtype: 'window',
	    closable: false,
	    resizable: false,
	    reference: 'loginwindow',
	    autoShow: true,
	    modal: true,

	    defaultFocus: 'usernameField',

	    layout: {
		type: 'auto'
	    },

	    title: gettext('Proxmox Mail Gateway Login'),

	    items: [
		{
		    xtype: 'form',
		    layout: {
			type: 'form'
		    },
		    defaultButton: 'loginButton',
		    url: '/api2/extjs/access/ticket',
		    reference: 'loginForm',

		    fieldDefaults: {
			labelAlign: 'right',
			allowBlank: false
		    },

		    items: [
			{
			    xtype: 'textfield',
			    fieldLabel: gettext('User name'),
			    name: 'username',
			    itemId: 'usernameField',
			    reference: 'usernameField'
			},
			{
			    xtype: 'textfield',
			    inputType: 'password',
			    fieldLabel: gettext('Password'),
			    name: 'password',
			    reference: 'passwordField'
			},
			{
			    xtype: 'proxmoxLanguageSelector',
			    fieldLabel: gettext('Language'),
			    value: Ext.util.Cookies.get('PMGLangCookie') || 'en',
			    name: 'lang',
			    submitValue: false
			},
			{
			    xtype: 'hiddenfield',
			    reference: 'realmfield',
			    name: 'realm',
			    value: 'pmg'
                        }
		    ],
		    buttons: [
			{
			    text: gettext('Login'),
			    reference: 'loginButton',
			    formBind: true
			}
		    ]
		}
	    ]
	}
    ]
});
Ext.define('PMG.RoleSelector', {
    extend: 'Proxmox.form.KVComboBox',
    alias: 'widget.pmgRoleSelector',

    comboItems: [
	['admin', PMG.Utils.format_user_role('admin')],
	['qmanager', PMG.Utils.format_user_role('qmanager')],
	['audit', PMG.Utils.format_user_role('audit')]
    ]
});
/*global Proxmox*/
Ext.define('PMG.ServerStatus', {
    extend: 'Ext.panel.Panel',
    alias: 'widget.pmgServerStatus',

    title: gettext('Status'),

    border: false,

    scrollable: true,

    bodyPadding: '10 0 0 0',
    defaults: {
	width: 700,
	padding: '0 0 10 10'
    },

    layout: 'column',

    controller: {
	xclass: 'Ext.app.ViewController',

	openConsole: function() {
	    Proxmox.Utils.openXtermJsViewer('shell', 0, Proxmox.NodeName);
	},

	nodeCommand: function(cmd) {
	    var me = this.getView();
	    Proxmox.Utils.API2Request({
		params: { command: cmd },
		url: '/nodes/' + Proxmox.NodeName + '/status',
		method: 'POST',
		waitMsgTarget: me,
		failure: function(response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	nodeShutdown: function() {
	    this.nodeCommand('shutdown');
	},

	nodeReboot: function() {
	    this.nodeCommand('reboot');
	}
    },

    tbar: [
	{
	    text: gettext("Console"),
	    iconCls: 'fa fa-terminal',
	    handler: 'openConsole'
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Restart'),
	    dangerous: true,
	    confirmMsg: gettext('Node') + " '" + Proxmox.NodeName + "' - " + gettext('Restart'),
	    handler: 'nodeReboot',
	    iconCls: 'fa fa-undo'
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Shutdown'),
	    dangerous: true,
	    confirmMsg: gettext('Node') + " '" + Proxmox.NodeName + "' - " + gettext('Shutdown'),
	    handler: 'nodeShutdown',
	    iconCls: 'fa fa-power-off'
	},
	'->',
	{
	    xtype: 'proxmoxRRDTypeSelector'
	}
    ],

    initComponent: function() {
        var me = this;

	var nodename = Proxmox.NodeName;
	var rrdstore = Ext.create('Proxmox.data.RRDStore', {
	    rrdurl: "/api2/json/nodes/" + nodename + "/rrddata",
	    fields: [
		{ type: 'number', name: 'loadavg' },
		{ type: 'number', name: 'maxcpu' },
		{
		    type: 'number',
		    name: 'cpu',
		    convert: function(val) {
			return val*100;
		    }
		},
		{
		    type: 'number',
		    name: 'iowait',
		    convert: function(val) {
			return val*100;
		    }
		},
		{ type: 'number', name: 'memtotal' },
		{ type: 'number', name: 'memused' },
		{ type: 'number', name: 'swaptotal' },
		{ type: 'number', name: 'swapused' },
		{ type: 'number', name: 'roottotal' },
		{ type: 'number', name: 'rootused' },
		{ type: 'number', name: 'netin' },
		{ type: 'number', name: 'netout' },
		{ type: 'date', dateFormat: 'timestamp', name: 'time' }
	    ]
	});

	Ext.apply(me, {
	    items: [
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('CPU usage'),
		    unit: 'percent',
		    fields: ['cpu','iowait'],
		    fieldTitles: [gettext('CPU usage'), gettext('IO delay')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Server load'),
		    fields: ['loadavg'],
		    fieldTitles: [gettext('Load average')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Memory usage'),
		    unit: 'bytes',
		    fields: ['memtotal','memused'],
		    fieldTitles: [gettext('Total'), gettext('Used')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Swap usage'),
		    unit: 'bytes',
		    fields: ['swaptotal','swapused'],
		    fieldTitles: [gettext('Total'), gettext('Used')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Network traffic'),
		    unit: 'bytespersecond',
		    fields: ['netin','netout'],
		    fieldTitles: [gettext('Ingress'), gettext('Egress')],
		    store: rrdstore
		},
		{
		    xtype: 'proxmoxRRDChart',
		    title: gettext('Disk usage'),
		    unit: 'bytes',
		    fields: ['roottotal','rootused'],
		    fieldTitles: [gettext('Total'), gettext('Used')],
		    store: rrdstore
		}
	    ],
	    listeners: {
		activate: function() {
		    rrdstore.startUpdate();
		},
		destroy: function() {
		    rrdstore.stopUpdate();
		}
	    }
	});
	me.callParent();
   }
});
    
/*global Proxmox*/
Ext.define('PMG.ServerAdministration', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pmgServerAdministration',

    title: gettext('Server Administration'),

    border: false,
    defaults: { border: false },

    controller: {
	xclass: 'Ext.app.ViewController',

        init: function(view) {
	    var upgradeBtn = view.lookupReference('upgradeBtn');
	    upgradeBtn.setDisabled(!(Proxmox.UserName && Proxmox.UserName === 'root@pam'));
	}
    },

    items: [
	{
	    xtype: 'pmgServerStatus',
	    itemId: 'status'
	},
	{
	    xtype: 'proxmoxNodeServiceView',
            title: gettext('Services'),
	    itemId: 'services',
	    startOnlyServices: {
		syslog: true,
		pmgproxy: true,
		pmgdaemon: true
	    },
	    nodename: Proxmox.NodeName
	},
	{
	    xtype: 'proxmoxNodeAPT',
            title: gettext('Updates'),
	    upgradeBtn: {
		xtype: 'button',
		reference: 'upgradeBtn',
		disabled: true,
		text: gettext('Upgrade'),
		handler: function() {
		    Proxmox.Utils.openXtermJsViewer('upgrade', 0, Proxmox.NodeName);
		}
	    },
	    itemId: 'updates',
	    nodename: Proxmox.NodeName
	},
	{
	    xtype: 'proxmoxLogView',
	    itemId: 'logs',
	    title: gettext('Syslog'),
	    url: "/api2/extjs/nodes/" + Proxmox.NodeName + "/syslog",
	    log_select_timespan: 1
	},
	{
	    xtype: 'proxmoxNodeTasks',
	    itemId: 'tasks',
	    title: gettext('Tasks'),
	    height: 'auto',
	    nodename: Proxmox.NodeName
	}
    ]
});


/*global Proxmox*/
Ext.define('PMG.LDAPProfileSelector', {
    extend: 'Proxmox.form.ComboGrid',
    alias: 'widget.pmgLDAPProfileSelector',

    store: {
	fields: [ 'profile', 'disable', 'comment' ],
	proxy: {
	    type: 'proxmox',
	    url: '/api2/json/config/ldap'
	},
	filterOnLoad: true,
	sorters: [
	    {
		property : 'profile',
		direction: 'ASC'
	    }
	]
    },

    valueField: 'profile',
    displayField: 'profile',

    allowBlank: false,

    listConfig: {
	columns: [
	    {
		header: gettext('Profile'),
		dataIndex: 'profile',
		hideable: false,
		width: 100
	    },
	    {
		header: gettext('Enabled'),
		width: 80,
		sortable: true,
		dataIndex: 'disable',
		renderer: Proxmox.Utils.format_neg_boolean
	    },
	    {
		header: gettext('Comment'),
		sortable: false,
		renderer: Ext.String.htmlEncode,
		dataIndex: 'comment',
		flex: 1
	    }
	]
    },

    initComponent: function() {
	var me = this;

	me.callParent();

	me.store.load();
    }
});
Ext.define('PMG.LDAPGroupSelector', {
    extend: 'Ext.form.ComboBox',
    alias: 'widget.pmgLDAPGroupSelector',

    profile: undefined,
    
    queryMode: 'local',

    store: {
	fields: [ 'dn' ],
	filterOnLoad: true,
	sorters: [
	    {
		property : 'dn',
		direction: 'ASC'
	    }
	]
    },

    valueField: 'dn',
    displayField: 'dn',

    allowBlank: false,

    setProfile: function(profile, force) {
	var me = this;

	if (!force && (profile === undefined || profile === null || me.profile === profile)) {
	    return;
	}

	me.profile = profile;

	me.setValue('');

	me.store.setProxy({
	    type: 'proxmox',
	    url: '/api2/json/config/ldap/' + me.profile + '/groups'
	});

	me.store.load();
    },
    
    initComponent: function() {
	var me = this;

	me.callParent();

	if (me.profile !== undefined) {
	    me.setProfile(me.profile, true);
	}
    }
});

Ext.define('PMG.LDAPGroupInputPanel', {
    extend: 'Proxmox.panel.InputPanel',
    alias: 'widget.pmgLDAPGroupInputPanel',

    onGetValues: function(values) {
	if (values.mode === 'profile-any') {
	    values.mode = 'any';
	} else if (values.mode === 'profile-none') {
	    values.mode = 'none';
	}

	return values;
    },

    setValues: function(values) {
	var me = this;

	if (values.profile !== undefined) {
	    if (values.mode === 'any') {
		values.mode = 'profile-any';
	    } else if (values.mode === 'none') {
		values.mode = 'profile-none';
	    }
	}

	if (values.profile !== undefined) {
	    var groupField = this.lookupReference('groupField');
	    groupField.setProfile(values.profile);
	}

	me.callParent([values]);
    },

    controller: {

	xclass: 'Ext.app.ViewController',

	changeMode: function(f, value) {
	    var groupField = this.lookupReference('groupField');
	    groupField.setDisabled(value !== 'group');
	    groupField.setVisible(value === 'group');
	    var profileField = this.lookupReference('profileField');
	    var enabled = ((value != 'any') && (value != 'none'));
	    profileField.setDisabled(!enabled);
	    profileField.setVisible(enabled);
	},

	changeProfile: function(f, value) {
	    var groupField = this.lookupReference('groupField');
	    groupField.setProfile(value);
	},

	control: {
	    'field[name=mode]': {
		change: 'changeMode'
	    },
	    'field[name=profile]': {
		change: 'changeProfile'
	    }
	}
    },

    items: [
	{
	    xtype: 'proxmoxKVComboBox',
	    name: 'mode',
	    value: 'group',
	    comboItems: [
		[ 'group', gettext('Group member') ],
		[ 'profile-any', gettext('Existing LDAP address')],
		[ 'any', gettext('Existing LDAP address') +
		  ', any profile' ],
		[ 'profile-none', gettext('Unknown LDAP address')],
		[ 'none', gettext('Unknown LDAP address') +
		  ', any profile' ]
	    ],
	    fieldLabel: gettext("Match")
	},
	{
	    xtype: 'pmgLDAPProfileSelector',
	    name: 'profile',
	    reference: 'profileField',
	    fieldLabel: gettext("Profile")
	},
	{
	    xtype: 'pmgLDAPGroupSelector',
	    name: 'group',
	    reference: 'groupField',
	    fieldLabel: gettext("Group")
	}
    ]
});

Ext.define('PMG.LDAPGroupEditor', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pmgLDAPGroupEditor',

    width: 500,

    items: [{ xtype: 'pmgLDAPGroupInputPanel' }]
});
Ext.define('PMG.LDAPUserSelector', {
    extend: 'Proxmox.form.ComboGrid',
    alias: 'widget.pmgLDAPUserSelector',

    profile: undefined,
    
    store: {
	fields: [ 'account', 'pmail', 'dn' ],
	filterOnLoad: true,
	sorters: [
	    {
		property : 'account',
		direction: 'ASC'
	    }
	]
    },

    valueField: 'account',
    displayField: 'account',

    allowBlank: false,

    listConfig: {
	columns: [
	    {
		header: gettext('Account'),
		dataIndex: 'account',
		hideable: false,
		width: 100
	    },
	    {
		header: gettext('EMail'),
		dataIndex: 'pmail',
		width: 150
	    },
	    {
		header: 'DN',
		dataIndex: 'dn',
		width: 200
	    }
	]
    },
    
    setProfile: function(profile, force) {
	var me = this;

	if (!force && (profile === undefined || profile === null || me.profile === profile)) {
	    return;
	}

	me.profile = profile;

	me.setValue('');

	me.store.setProxy({
	    type: 'proxmox',
	    url: '/api2/json/config/ldap/' + me.profile + '/users'
	});

	me.store.load();
    },
    
    initComponent: function() {
	var me = this;

	me.callParent();

	if (me.profile !== undefined) {
	    me.setProfile(me.profile, true);
	}
    }
});

Ext.define('PMG.LDAPUserInputPanel', {
    extend: 'Proxmox.panel.InputPanel',
    alias: 'widget.pmgLDAPUserInputPanel',

    setValues: function(values) {
	var me = this;

	if (values.profile !== undefined) {
	    var accountField = this.lookupReference('accountField');
	    accountField.setProfile(values.profile);
	}

	me.callParent([values]);
    },
    
    controller: {

	xclass: 'Ext.app.ViewController',

	changeProfile: function(f, value) {
	    var accountField = this.lookupReference('accountField');
	    accountField.setProfile(value);
	},

	control: {
	    'field[name=profile]': {
		change: 'changeProfile'
	    }
	}
    },

    items: [
	{
	    xtype: 'pmgLDAPProfileSelector',
	    name: 'profile',
	    reference: 'profileField',
	    fieldLabel: gettext("Profile")
	},
	{
	    xtype: 'pmgLDAPUserSelector',
	    name: 'account',
	    reference: 'accountField',
	    fieldLabel: gettext("Account")
	}
    ]
});

Ext.define('PMG.LDAPUserEditor', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pmgLDAPUserEditor',

    width: 500,

    items: [{ xtype: 'pmgLDAPUserInputPanel' }]
});
/*global Proxmox*/
Ext.define('PMG.RegexTester', {
    extend: 'Ext.form.FieldContainer',
    alias: 'widget.pmgRegexTester',

    // the field reference which holds the regex value
    // has to be a sibling of the RegexTester component
    regexFieldReference: undefined,

    // if true, wraps the regex with ^ and $
    wholeMatch: false,

    layout: 'hbox',
    submitValue: false,

    items: [{
	xtype: 'textfield',
	submitValue: false,
	name: 'teststring',
	isDirty: function () { return false; },
	reset: Ext.emptyFn
    },{
	margin: '0 0 0 5',
	xtype: 'button',
	text: 'Test',
	handler: function() {
	    var me = this.up();
	    var regexField = me.up().down('field[reference=' + me.regexFieldReference +']');
	    var regex = '';

	    if (me.wholeMatch) {
		regex = '^' + regexField.getValue() + '$';
	    } else {
		regex = regexField.getValue();
	    }

	    Proxmox.Utils.API2Request({
		url: '/api2/extjs/config/regextest',
		waitMsgTarget: me.up('window'),
		params: {
		    regex: regex,
		    text: me.down('textfield[name=teststring]').getValue()
		},
		method: 'POST',
		success: function(response) {
		    Ext.Msg.show({
			title: gettext('Success'),
			message: gettext('OK') +
			    ' (elapsed time: ' +
			    response.result.data + 'ms' + ')',
			buttons: Ext.Msg.OK,
			icon: Ext.MessageBox.INFO
		    });
		},
		failure: function(response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	}
    }],

    initComponent: function() {
	var me = this;

	if (!me.regexFieldReference) {
	    throw "No regex field reference given";
	}

	me.callParent();

    }
});
/*global Proxmox*/
Ext.define('pmg-object-group', {
    extend: 'Ext.data.Model',
    fields: [ 'id', 'name', 'info' ],
    idProperty: 'id'
});

Ext.define('pmg-object-list', {
    extend: 'Ext.data.Model',
    fields: [
	'id', 'descr',
	{ name: 'otype', type: 'integer' },
	{ name: 'receivertest', type: 'boolean' }
    ],
    idProperty: 'id'
});


Ext.define('PMG.ObjectGroupList', {
    extend: 'Ext.grid.GridPanel',
    alias: ['widget.pmgObjectGroupList'],

    ogclass: undefined, //  'who', 'when', 'what'

    subject: 'Object Group List', // please overwrite

    baseurl: undefined,

    enableButtons: true,

    inputItems: [
	{
	    xtype: 'textfield',
	    name: 'name',
	    allowBlank: false,
	    fieldLabel: gettext('Name')
	},
	{
	    xtype: 'textareafield',
	    name: 'info',
	    fieldLabel: gettext("Description")
	}
    ],

    reload: function() {
	var me = this;

        me.store.load();
    },

    run_editor: function() {
	var me = this;

	var rec = me.selModel.getSelection()[0];
	if (!rec) {
	    return;
	}

	var config = {
	    url: "/api2/extjs" + me.baseurl +'/' + rec.data.id + '/config',
	    method: 'PUT',
	    subject: me.subject,
	    width: 400,
	    items: me.inputItems
	};

	var win = Ext.createWidget('proxmoxWindowEdit', config);

	win.load();
	win.on('destroy', me.reload, me);
	win.show();
    },

    initComponent : function() {
	var me = this;

	if (!me.ogclass) {
	    throw "ogclass not initialized";
	}

	me.baseurl = "/config/ruledb/" + me.ogclass;

	me.store = new Ext.data.Store({
	    model: 'pmg-object-group',
	    proxy: {
		type: 'proxmox',
		url: "/api2/json" + me.baseurl
	    },
	    sorters: {
		property: 'name',
		order: 'DESC'
	    }
	});

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	var remove_btn = Ext.createWidget('proxmoxStdRemoveButton', {
	    selModel: me.selModel,
	    baseurl: me.baseurl,
	    callback: function() { me.reload(); },
	    getRecordName: function(rec) { return rec.data.name; },
	    waitMsgTarget: me
	});

	var tbar = [
            {
		xtype: 'proxmoxButton',
		text: gettext('Edit'),
		disabled: true,
		selModel: me.selModel,
		handler: function() { me.run_editor(); }
            },
            {
		text: gettext('Create'),
		handler: function() {
		    var config = {
			method: 'POST',
			url: "/api2/extjs" + me.baseurl,
			isCreate: true,
			width: 400,
			subject: me.subject,
			items: me.inputItems
		    };

		    var win = Ext.createWidget('proxmoxWindowEdit', config);

		    win.on('destroy', me.reload, me);
		    win.show();
		}
            },
	    remove_btn
        ];

	Proxmox.Utils.monStoreErrors(me, me.store, true);

	if (me.enableButtons) {
	    me.tbar = tbar;
	}

	Ext.apply(me, {
	    columns: [
		{
		    header: gettext('Name'),
		    sortable: true,
		    flex: 1,
		    dataIndex: 'name',
		    renderer: Ext.String.htmlEncode
		}
	    ],
	    listeners: {
		itemdblclick: function() {
		    if (me.enableButtons) {
			me.run_editor();
		    }
		},
		activate: function() { me.reload(); }
	    }
	});

	me.callParent();

	me.reload(); // initial load
    }
});
/*global Proxmox*/
Ext.define('PMG.ObjectGroup', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgObjectGroup',

    baseurl: undefined,

    otype_list: [],

    hideGroupInfo: false,
    showDirection: false, // only important for SMTP Whitelist

    ogdata: undefined,

    emptyText: gettext('Please select an object.'),

    setBaseUrl: function(baseurl) {
	var me = this;

	me.baseurl = baseurl;

	if (me.baseurl === undefined) {
	    me.store.proxy.setUrl(undefined);
	    me.store.setData([]);
	    me.setButtonState(me.store, [], false);
	} else {
	    var url = '/api2/json' + me.baseurl + '/objects';
	    me.store.proxy.setUrl(url);
	    me.store.load();
	}
    },

    setObjectInfo: function(ogdata) {
	var me = this;

	me.ogdata = ogdata;

	if (me.ogdata === undefined) {

	    me.down('#oginfo').update(me.emptyText);

	} else {

	    var html = '<b>' + Ext.String.htmlEncode(me.ogdata.name) + '</b>';
	    html += "<br><br>";
	    html += Ext.String.htmlEncode(Ext.String.trim(me.ogdata.info));

	    me.down('#oginfo').update(html);
	    me.down('#ogdata').setHidden(false);
	}
    },

    setButtonState: function(store, records, success) {
	var me = this;
	if (!success || !me.baseurl) {
	    me.down('#addMenuButton').setDisabled(true);
	    return;
	}
	me.down('#addMenuButton').setDisabled(false);
    },

    initComponent : function() {
	var me = this;

	me.store = new Ext.data.Store({
	    model: 'pmg-object-list',
	    proxy: {
		type: 'proxmox'
	    },
	    sorters: [
		{
		    property : 'receivertest'
		},
		{
		    property : 'otype',
		    direction: 'ASC'
		}
	    ]
	});

	me.columns = [{
	    header: gettext('Type'),
	    dataIndex: 'otype',
	    renderer: PMG.Utils.format_otype,
	    width: 200
	}];

	if (me.showDirection) {
	    me.columns.push({
		header: gettext('Direction'),
		dataIndex: 'receivertest',
		renderer: function(value) {
		    return value ? PMG.Utils.receiverText : PMG.Utils.senderText;
		}
	    });
	}

	me.columns.push({
	    header: gettext('Value'),
	    dataIndex: 'descr',
	    renderer: Ext.String.htmlEncode,
	    flex: 1
	});

	var reload = function() {
            me.store.load();
        };

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	var remove_btn = Ext.createWidget('proxmoxStdRemoveButton', {
	    selModel: me.selModel,
	    getUrl: function(rec) {
		return me.baseurl + '/objects/' + rec.data.id;
	    },
	    callback: reload,
	    getRecordName: function(rec) {
		return PMG.Utils.format_otype(rec.data.otype) +
		    ': ' + rec.data.descr;
	    },
	    waitMsgTarget: me
	});

	var full_subject = function(subject, receivertest) {
	    if (me.showDirection) {
		var direction = receivertest ?
		    PMG.Utils.receiverText : PMG.Utils.senderText;

		return subject + ' (' + direction + ')';
	    } else {
		return subject;
	    }
	};

	var run_editor = function() {
	    var rec = me.selModel.getSelection()[0];
	    if (!rec) {
		return;
	    }

	    var editor = PMG.Utils.object_editors[rec.data.otype];
	    if (!editor || editor.uneditable) {
		return;
	    }

	    var config = Ext.apply({ method: 'PUT' }, editor);
	    config.subject = full_subject(editor.subject, rec.data.receivertest);
	    config.url = me.baseurl + '/' + editor.subdir + '/' + rec.data.id;

	    var win = Ext.createWidget(config);

	    win.load();
	    win.on('destroy', reload);
	    win.show();
	};

	var menu_items = [];

	Ext.Array.each(me.otype_list, function(otype) {

	    var editor = PMG.Utils.object_editors[otype];

	    var config = Ext.apply({ method: 'POST' }, editor);
	    config.subject = full_subject(editor.subject, editor.receivertest);

	    menu_items.push({
		text: config.subject,
		iconCls: config.iconCls || 'fa fa-question-circle',
		handler: function() {
		    if (me.baseurl == undefined) {
			return;
		    }
		    config.url = me.baseurl + '/' + editor.subdir;
		    var win = Ext.create(config);
		    win.on('destroy', reload);
		    win.show();
		}
	    });
	});

	me.dockedItems = [];

	me.dockedItems.push({
	    xtype: 'toolbar',
	    dock: 'top',
	    items: [
		{
		    text: gettext('Add'),
		    disabled: true,
		    itemId: 'addMenuButton',
		    menu: {
			items: menu_items
		    }
		},
		{
		    xtype: 'proxmoxButton',
		    text: gettext('Edit'),
		    disabled: true,
		    selModel: me.selModel,
		    enableFn: function(rec) {
			var editor = PMG.Utils.object_editors[rec.data.otype];
			return (editor && !editor.uneditable);
		    },
		    handler: run_editor
		},
		remove_btn
	    ]
	});

	me.dockedItems.push({
	    dock: 'top',
	    border: 1,
	    layout: 'anchor',
	    hidden: me.hideGroupInfo ? true : false,
	    itemId: 'ogdata',
	    items: [
		{
		    xtype: 'component',
		    anchor: '100%',
		    itemId: 'oginfo',
		    style: { 'white-space': 'pre' },
		    padding: 10,
		    html: me.emptyText,
		    listeners: {
			dblclick: {
			    fn: function(e, t) {
				if (me.ogdata === undefined) { return; }
				me.fireEvent('dblclickOGInfo', me, e, t, me.ogdata);
			    },
			    element: 'el',
			    scope: this
			}
		    }
		}
	    ]
	});

	Proxmox.Utils.monStoreErrors(me, me.store, true);

	Ext.apply(me, {
	    listeners: {
		itemdblclick: run_editor,
		activate: reload
	    }
	});

	me.callParent();

	me.mon(me.store, 'load', me.setButtonState, me);

	if (me.baseurl) {
	    me.setBaseUrl(me.baseurl); // configure store, load()
	}
    }
});
Ext.define('PMG.ObjectGroupSelector', {
    extend: 'Ext.window.Window',
    alias: 'widget.pmgObjectGroupSelector',

    width: 600,
    layout: 'auto',
    modal: true,
    bodyPadding: 5,

    rulegroup: undefined,

    initComponent : function() {
	var me = this;

	if (!me.rulegroup) {
	    throw "undefined rulegroup";
	}

	var ogclass;

	if (me.rulegroup === 'from') {
	    ogclass = 'who';
	    me.title = gettext('From');
	} else if (me.rulegroup === 'to') {
	    ogclass = 'who';
	    me.title = gettext('To');
	} else if (me.rulegroup === 'when') {
	    ogclass = 'when';
	    me.title = gettext('When');
	} else if (me.rulegroup === 'what') {
	    ogclass = 'what';
	    me.title = gettext('What');
	} else if (me.rulegroup === 'action') {
	    ogclass = 'action';
	    me.title = gettext('Action');
	} else {
	    throw "unknown rulegroup";
	}

	if (me.rulegroup === 'action') {
	    me.items = {
		xtype: 'pmgActionList',
		title: undefined,
		enableButtons: false,
		border: true,
		listeners: {
		    itemdblclick: function(view, rec) {
			me.fireEvent('selectObjectGroup', me, rec);
		    }
		}
	    };
	} else {
	    me.items = {
		xtype: 'pmgObjectGroupList',
		enableButtons: false,
		ogclass: ogclass,
		listeners: {
		    itemdblclick: function(view, rec) {
			me.fireEvent('selectObjectGroup', me, rec);
		    }
		}
	    };
	}

	me.callParent();
    }
});
Ext.define('PMG.ObjectGroupConfiguration', {
    extend: 'Ext.panel.Panel',

    ogclass: undefined, // who, when, what
    otype_list: [],
    
    layout: 'border',
    border: false,
    
    initComponent : function() {
	var me = this;

	if (me.ogclass === undefined) {
	    throw "undefined object group class";
	}
	
	if (!(PMG.Utils.oclass_text[me.ogclass])) {
	    throw "unknown object group class";
	}

	var left = Ext.create('PMG.ObjectGroupList', {
	    width: 250,
	    ogclass: me.ogclass,
	    subject: PMG.Utils.oclass_text[me.ogclass],
	    title: PMG.Utils.oclass_text[me.ogclass],
	    border: false,
	    split: true,
	    region: 'west'
	});
	
	var right = Ext.create('PMG.ObjectGroup', {
	    otype_list: me.otype_list,
	    border: false,
	    region: 'center',
	    listeners: {
		dblclickOGInfo: function(w, e, t, ogdata) {
		    // test if the correct groups is selected (just to be sure)
		    var rec = left.selModel.getSelection()[0];
		    if (rec && rec.data && rec.data.id === ogdata.id) {
			left.run_editor();
			return;
		    }
		}
	    }
	});

	me.mon(left.store, "refresh", function() {
	    var rec = left.selModel.getSelection()[0];
	    if (!(rec && rec.data && rec.data.id)) {
		return;
	    }
	    right.setObjectInfo(rec.data);
	});

	me.mon(left.selModel, "selectionchange", function() {
	    var rec = left.selModel.getSelection()[0];
	    if (!(rec && rec.data && rec.data.id)) {
		right.setObjectInfo(undefined);
		right.setBaseUrl(undefined);
		return;
	    }
	    right.setObjectInfo(rec.data);
	    var baseurl = '/config/ruledb/' + me.ogclass + '/' + rec.data.id;
	    right.setBaseUrl(baseurl);
	});
	
	me.items = [ left, right ];

	me.callParent();
    }
});

Ext.define('PMG.WhoConfiguration', {
    extend: 'PMG.ObjectGroupConfiguration',
    xtype: 'pmgWhoConfiguration',

    ogclass: 'who',
    otype_list: [1000, 1001, 1002, 1003, 1004, 1005, 1006]
});

Ext.define('PMG.WhenConfiguration', {
    extend: 'PMG.ObjectGroupConfiguration',
    xtype: 'pmgWhenConfiguration',

    ogclass: 'when',
    otype_list: [2000]
});

Ext.define('PMG.WhatConfiguration', {
    extend: 'PMG.ObjectGroupConfiguration',
    xtype: 'pmgWhatConfiguration',

    ogclass: 'what',
    otype_list: [3000, 3001, 3002, 3003, 3004, 3005]
});

/*global Proxmox*/
Ext.define('pmg-action-list', {
    extend: 'Ext.data.Model',
    fields: [
	'id', 'name', 'info', 'descr',
	{ name: 'otype', type: 'integer' }
    ],
    idProperty: 'id'
});

Ext.define('PMG.ActionList', {
    extend: 'Ext.grid.GridPanel',
    alias: ['widget.pmgActionList'],

    title: PMG.Utils.oclass_text.action,
    border: false,

    baseurl: '/config/ruledb/action',

    otype_list: [4002, 4003, 4005, 4007, 4009],

    enableButtons: true,

    initComponent : function() {
	var me = this;

	me.store = new Ext.data.Store({
	    model: 'pmg-action-list',
	    proxy: {
		type: 'proxmox',
		url: "/api2/json" + me.baseurl + '/objects'
	    },
	    sorters: {
		property: 'name',
		order: 'DESC'
	    }
	});

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	var reload = function() {
	    me.store.load();
	};

	var run_editor = function() {
	    var rec = me.selModel.getSelection()[0];
	    if (!rec) {
		return;
	    }

	    var editor = PMG.Utils.object_editors[rec.data.otype];
	    if (!editor) {
		return;
	    }

	    var config = Ext.apply({ method: 'PUT' }, editor);

	    config.url = me.baseurl + '/' + editor.subdir + '/' + rec.data.id;

	    var win = Ext.createWidget('proxmoxWindowEdit', config);

	    win.load();
	    win.on('destroy', reload);
	    win.show();
	};

	var remove_btn = Ext.createWidget('proxmoxStdRemoveButton', {
	    selModel: me.selModel,
	    getUrl: function(rec) {
		return me.baseurl + '/objects/' + rec.data.id;
	    },
	    callback: reload,
	    getRecordName: function(rec) { return rec.data.descr; },
	    waitMsgTarget: me
	});

	var menu_items = [];

	Ext.Array.each(me.otype_list, function(otype) {

	    var editor = PMG.Utils.object_editors[otype];

	    var config = Ext.apply({ method: 'POST' }, editor);

	    config.isCreate = true;
	    menu_items.push({
		text: config.subject,
		handler: function() {
		    if (me.baseurl == undefined) {
			return;
		    }
		    config.url = me.baseurl + '/' + editor.subdir;
		    var win = Ext.createWidget('proxmoxWindowEdit', config);
		    win.on('destroy', reload);
		    win.show();
		}
	    });
	});

	var tbar = [
	    {
		text: gettext('Add'),
		menu: new Ext.menu.Menu({
		    items: menu_items
		})
	    },
            {
		xtype: 'proxmoxButton',
		text: gettext('Edit'),
		disabled: true,
		selModel: me.selModel,
		handler: run_editor
            },
	    remove_btn
        ];

	Proxmox.Utils.monStoreErrors(me, me.store, true);

	if (me.enableButtons) {
	    me.tbar = tbar;
	}

	Ext.apply(me, {
	    columns: [
		{
		    header: gettext('Name'),
		    sortable: true,
		    width: 200,
		    dataIndex: 'name',
		    renderer: Ext.String.htmlEncode
		},
		{
		    header: gettext('Description'),
		    sortable: true,
		    width: 300,
		    dataIndex: 'descr',
		    renderer: Ext.String.htmlEncode
		},
		{
		    header: gettext('Comment'),
		    sortable: false,
		    flex: 1,
		    dataIndex: 'info',
		    renderer: Ext.String.htmlEncode
		}
	    ],
	    listeners: {
		itemdblclick: function() {
		    if (me.enableButtons) {
			run_editor();
		    }
		},
		activate: reload
	    }
	});

	me.callParent();

	reload(); // initial load
    }
});
/*global Proxmox*/
/*jslint confusion: true*/
/* bind is a function function */
Ext.define('PMG.RuleInfo', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgRuleInfo',

    controller: {
	xclass: 'Ext.app.ViewController',

	setBaseUrl: function(baseurl) {
	    var me = this;
	    me.getViewModel().set('baseurl', baseurl);
	    me.reload();
	},

	reload: function() {
	    var me = this;
	    var viewmodel = me.getViewModel();
	    var baseurl = viewmodel.get('baseurl');

	    if (!baseurl) {
		me.setRuleInfo(undefined);
		return;
	    }

	    Proxmox.Utils.API2Request({
		url: baseurl + "/config",
		method: 'GET',
		success: function(response, opts) {
		    me.setRuleInfo(response.result.data);
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	removeObjectGroup: function(rec) {
	    var me = this;
	    Ext.Msg.confirm(
		gettext('Confirm'),
		Ext.String.format(
		    gettext('Are you sure you want to remove entry {0}'),
		    "'" + rec.data.name + "'"),
		function(button) {
		    if (button === 'yes') {
			Proxmox.Utils.API2Request({
			    url: me.getViewModel().get('baseurl') + '/' + rec.data.oclass + '/'+ rec.data.typeid,
			    method: 'DELETE',
			    waitMsgTarget: me.getView(),
			    callback: function() {
				me.reload();
			    },
			    failure: function (response, opts) {
				Ext.Msg.alert(gettext('Error'), response.htmlStatus);
			    }
			});
		    }
		}
	    );
	},

	addObjectGroup: function(type, record) {
	    var me = this;
	    var baseurl = me.getViewModel().get('baseurl');
	    var url = baseurl + '/' + type;
	    var id = (type === 'action')?record.data.ogroup:record.data.id;
	    Proxmox.Utils.API2Request({
		url: url,
		params: { ogroup: id },
		method: 'POST',
		waitMsgTarget: me.getView(),
		callback: function() {
		    me.reload();
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	setRuleInfo: function(ruledata) {
	    var me = this;

	    var viewmodel = me.getViewModel();

	    if (ruledata === undefined) {

		viewmodel.set('selectedRule', null);
		viewmodel.get('objects').setData([]);

	    } else {

		ruledata.name = Ext.String.htmlEncode(ruledata.name);
		viewmodel.set('selectedRule', ruledata);

		var data = [];
		Ext.Array.each(['from', 'to', 'when', 'what', 'action'], function(oc) {

		    var store = viewmodel.get(oc + 'objects');
		    if (ruledata[oc] === undefined || store === undefined) { return; }

		    // we build a filter for the objects,
		    // which are already added to the rule,
		    // so what we only show the ones,
		    // which are still available

		    var ids = Ext.Array.pluck(ruledata[oc], 'id');
		    // for the actions, we have a different id field
		    var idField = (oc === 'action')?'ogroup':'id';
		    store.clearFilter();
		    store.addFilter({
			filterFn:function(record){
			    // FIXME
			    // actions have the ogroup as a string
			    // -> parseInt
			    return (ids.indexOf(parseInt(record.data[idField], 10)) === -1);
			}
		    });
		    store.load();
		    Ext.Array.each(ruledata[oc], function(og) {
			data.push({ oclass: oc, name: og.name, typeid: og.id });
		    });
		});

		viewmodel.get('objects').setData(data);
	    }
	},

	removeIconClick: function(gridView, rowindex, colindex, button, event, record) {
	    var me = this;
	    me.removeObjectGroup(record);
	},

	removeDrop: function(gridView, data, overModel) {
	    var me = this;
	    var record = data.records[0]; // only one
	    me.removeObjectGroup(record);
	    return true;
	},

	addIconClick: function(gridView, rowindex, colindex, button, event, record) {
	    var me = this;
	    me.addObjectGroup(gridView.panel.type, record);
	    return true;
	},

	addDrop: function(gridView, data, overModel) {
	    var me = this;
	    var record = data.records[0]; // only one
	    me.addObjectGroup(data.view.panel.type, record);
	    return true;
	},

	control: {
	    'grid[reference=usedobjects]': {
		drop: 'addDrop'
	    },
	    'tabpanel[reference=availobjects] > grid': {
		drop: 'removeDrop'
	    }
	}
    },

    viewModel: {
	data: {
	    baseurl: ''
	},

	stores: {
	    objects: {
		fields: ['oclass', 'name', 'typeid'],
		groupField: 'oclass',
		sorters: 'name'
	    },

	    actionobjects: {
		model: 'pmg-action-list',
		proxy: {
		    type: 'proxmox',
		    url: "/api2/json/config/ruledb/action/objects"
		},
		sorters: 'name'
	    },
	    fromobjects: {
		model: 'pmg-object-group',
		proxy: {
		    type: 'proxmox',
		    url: "/api2/json/config/ruledb/who"
		},
		sorters: 'name'
	    },
	    toobjects: {
		model: 'pmg-object-group',
		proxy: {
		    type: 'proxmox',
		    url: "/api2/json/config/ruledb/who"
		},
		sorters: 'name'
	    },
	    whatobjects: {
		model: 'pmg-object-group',
		proxy: {
		    type: 'proxmox',
		    url: "/api2/json/config/ruledb/what"
		},
		sorters: 'name'
	    },
	    whenobjects: {
		model: 'pmg-object-group',
		proxy: {
		    type: 'proxmox',
		    url: "/api2/json/config/ruledb/when"
		},
		sorters: 'name'
	    }
	}
    },


    defaults: {
	padding: '5 10 5 10'
    },

    bodyPadding: '5 0 5 0',

    layout: {
	type: 'vbox',
	align: 'stretch'
    },

    scrollable: true,

    items: [
	{
	    xtype: 'panel',
	    bodyPadding: '10 10 10 10',
	    data: {
		name: ''
	    },
	    bind: {
		data: {
		    name: '{selectedRule.name}',
		    priority: '{selectedRule.priority}',
		    active: '{selectedRule.active}',
		    direction: '{selectedRule.direction}',
		    selected: '{selectedRule}'
		}
	    },
	    tpl: [
		'<tpl if="selected">',
		'<b>{name}</b><br><br>',
		'Priority: {priority}<br>',
		'Direction: {[PMG.Utils.format_rule_direction(values.direction)]}<br>',
		'Active: {[Proxmox.Utils.format_boolean(values.active)]}<br>',
		'<tpl else>',
		gettext('Please select a rule.'),
		'</tpl>'
	    ]
	},
	{
	    xtype: 'grid',
	    reference: 'usedobjects',
	    hidden: true,
	    emptyText: gettext('No Objects'),
	    features: [{
		id: 'group',
		ftype: 'grouping',
		enableGroupingMenu: false,
		collapsible: false,
		groupHeaderTpl: [
		    '{[PMG.Utils.format_oclass(values.name)]}'
		]
	    }],

	    title: gettext('Used Objects'),

	    viewConfig: {
		plugins: {
		    ptype: 'gridviewdragdrop',
		    copy: true,
		    dragGroup: 'usedobjects',
		    dropGroup: 'unusedobjects',

		    // do not show default grid dragdrop behaviour
		    dropZone: {
			indicatorHtml: '',
			indicatorCls: '',
			handleNodeDrop: Ext.emptyFn
		    }
		}
	    },

	    columns: [
		{
		    header: gettext('Type'),
		    dataIndex: 'oclass',
		    hidden: true
		},
		{
		    header: gettext('Name'),
		    dataIndex: 'name',
		    flex: 1
		},
		{
		    text: '',
		    xtype: 'actioncolumn',
		    align: 'center',
		    width: 40,
		    items: [
			{
			    iconCls: 'fa fa-fw fa-minus-circle',
			    tooltip: gettext('Remove'),
			    handler: 'removeIconClick'
			}
		    ]
		}
	    ],

	    bind: {
		store: '{objects}',
		hidden: '{!selectedRule}'
	    }
	},
	{
	    xtype: 'tabpanel',
	    title: gettext('Available Objects'),
	    reference: 'availobjects',
	    hidden: true,
	    bind: {
		hidden: '{!selectedRule}'
	    },
	    defaults: {
		xtype: 'grid',
		emptyText: gettext('No Objects'),
		viewConfig: {
		    plugins: {
			ptype: 'gridviewdragdrop',
			dragGroup: 'unusedobjects',
			dropGroup: 'usedobjects',

			// do not show default grid dragdrop behaviour
			dropZone: {
			    indicatorHtml: '',
			    indicatorCls: '',
			    handleNodeDrop: Ext.emptyFn
			}
		    }
		},
		columns: [
		    {
			header: gettext('Name'),
			dataIndex: 'name',
			flex: 1
		    },
		    {
			text: '',
			xtype: 'actioncolumn',
			align: 'center',
			width: 40,
			items: [
			    {
				iconCls: 'fa fa-fw fa-plus-circle',
				tooltip: gettext('Add'),
				handler: 'addIconClick'
			    }
			]
		    }
		]
	    },
	    items: [
		{
		    title: gettext('Action'),
		    bind: {
			store: '{actionobjects}'
		    },
		    type: 'action',
		    iconCls: 'fa fa-flag'
		},
		{
		    title: gettext('From'),
		    iconCls: 'fa fa-user-circle',
		    type: 'from',
		    bind: {
			store: '{fromobjects}'
		    }
		},
		{
		    title: gettext('To'),
		    iconCls: 'fa fa-user-circle',
		    type: 'to',
		    bind: {
			store: '{toobjects}'
		    }
		},
		{
		    title: gettext('What'),
		    iconCls: 'fa fa-cube',
		    type: 'what',
		    bind: {
			store: '{whatobjects}'
		    }
		},
		{
		    title: gettext('When'),
		    iconCls: 'fa fa-clock-o',
		    type: 'when',
		    bind: {
			store: '{whenobjects}'
		    }
		}
	    ]
	}
    ]
});
Ext.define('PMG.RuleEditor', {
    extend: 'Proxmox.window.Edit',
    xtype: 'ruleeditwindow',

    url: undefined,

    method: 'PUT',

    subject: gettext('Rules'),

    width: 400,

    items: [
	{
	    xtype: 'textfield',
	    name: 'name',
	    allowBlank: false,
	    fieldLabel: gettext('Name')
	},
	{
	    xtype: 'proxmoxintegerfield',
	    name: 'priority',
	    allowBlank: false,
	    minValue: 0,
	    maxValue: 100,
	    fieldLabel: gettext('Priority')
	},
	{
	    xtype: 'proxmoxKVComboBox',
	    name: 'direction',
	    comboItems: [
		[0, PMG.Utils.rule_direction_text[0]],
		[1, PMG.Utils.rule_direction_text[1]],
		[2, PMG.Utils.rule_direction_text[2]]],
	    value: 2,
	    fieldLabel: gettext('Direction')
	},
	{
	    xtype: 'proxmoxcheckbox',
	    name: 'active',
	    defaultValue: 0,
	    uncheckedValue: 0,
	    checked: false,
	    fieldLabel: gettext('Active')
	}
    ]
});
Ext.define('PMG.image.Logo', {
    extend: 'Ext.Img',
    xtype: 'proxmoxlogo',

    height: 30,
    width: 172,
    src: '/pve2/images/proxmox_logo.png',
    alt: 'Proxmox',
    autoEl: {
	tag: 'a',
	href: 'https://www.proxmox.com',
	target: '_blank'
    }
});
/*global Proxmox*/
Ext.define('PMG.MainView', {
    extend: 'Ext.container.Container',
    xtype: 'mainview',

    title: 'Proxmox Mail Gateway',

    controller: {
	xclass: 'Ext.app.ViewController',
	routes: {
	    ':path:subpath': {
		action: 'changePath',
		before: 'beforeChangePath',
                conditions : {
		    ':path'    : '(?:([%a-zA-Z0-9\\-\\_\\s,]+))',
		    ':subpath' : '(?:(?::)([%a-zA-Z0-9\\-\\_\\s,]+))?'
		}
	    }
	},

	beforeChangePath: function(path, subpath, action) {
	    var me = this;

	    if (!Ext.ClassManager.getByAlias('widget.'+ path)) {
		console.warn('xtype "'+path+'" not found');
		action.stop();
		return;
	    }

	    var lastpanel = me.lookupReference('contentpanel').getLayout().getActiveItem();
	    if (lastpanel && lastpanel.xtype === path) {
		// we have the right component already,
		// we just need to select the correct tab
		// default to the first
		subpath = subpath || 0;
		if (lastpanel.getActiveTab) {
		    // we assume lastpanel is a tabpanel
		    if (lastpanel.getActiveTab().getItemId() !== subpath) {
			// set the active tab
			lastpanel.setActiveTab(subpath);
		    }
		    // else we are already there
		}
		action.stop();
		return;
	    }

	    action.resume();
	},

	changePath: function(path,subpath) {
	    var me = this;
	    var contentpanel = me.lookupReference('contentpanel');
	    var lastpanel = contentpanel.getLayout().getActiveItem();

	    var obj = contentpanel.add({ xtype: path });
	    var treelist = me.lookupReference('navtree');

	    treelist.suspendEvents();
	    treelist.select(path);
	    treelist.resumeEvents();

	    if (Ext.isFunction(obj.setActiveTab)) {
		obj.setActiveTab(subpath || 0);
		obj.addListener('tabchange', function(tabpanel, newc, oldc) {
		    var newpath = path;

		    // only add the subpath part for the
		    // non-default tabs
		    if (tabpanel.items.findIndex('id', newc.id) !== 0) {
			newpath += ":" + newc.getItemId();
		    }

		    me.redirectTo(newpath);
		});
	    }

	    contentpanel.setActiveItem(obj);

	    if (lastpanel) {
		contentpanel.remove(lastpanel, { destroy: true });
	    }

	},

	logout: function() {
	    PMG.app.logout();
	},

	navigate: function(treelist, item) {
	    this.redirectTo(item.get('path'));
	},

	control: {
	    'button[reference=logoutButton]': {
		click: 'logout'
	    }
	},

	init: function(view) {
	    var me = this;

	    // load username
	    me.lookupReference('usernameinfo').update({username:Proxmox.UserName});

	    // show login on requestexception
	    // fixme: what about other errors
	    Ext.Ajax.on('requestexception', function(conn, response, options) {
		if (response.status == 401) { // auth failure
		    me.logout();
		}
	    });

	    // get ticket periodically
	    Ext.TaskManager.start({
		run: function() {
		    var ticket = Proxmox.Utils.authOK();
		    if (!ticket || !Proxmox.UserName) {
			return;
		    }

		    Ext.Ajax.request({
			params: {
			    username: Proxmox.UserName,
			    password: ticket
			},
			url: '/api2/json/access/ticket',
			method: 'POST',
			failure: function() {
			    me.logout();
			},
			success: function(response, opts) {
			    var obj = Ext.decode(response.responseText);
			    PMG.Utils.updateLoginData(obj.data);
			}
		    });
		},
		interval: 15*60*1000
	    });

	    // select treeitem and load page from url fragment
	    var token = Ext.util.History.getToken() || 'pmgDashboard';
	    this.redirectTo(token, true);

	}
    },

    plugins: 'viewport',

    layout: { type: 'border' },

    items: [
	{
	    region: 'north',
	    xtype: 'container',
	    layout: {
		type: 'hbox',
		align: 'middle'
	    },
	    margin: '2 5 2 5',
	    height: 38,
	    items: [
		{
		    xtype: 'proxmoxlogo'
		},
		{
		    xtype: 'versioninfo'
		},
		{
		    flex: 1
		},
		{
		    baseCls: 'x-plain',
		    reference: 'usernameinfo',
		    padding: '0 5',
		    tpl: Ext.String.format(gettext("You are logged in as '{0}'"), '{username}')
		},
		{
		    xtype: 'proxmoxHelpButton',
		    hidden: false,
		    baseCls: 'x-btn',
		    iconCls: 'fa fa-info-circle x-btn-icon-el-default-toolbar-small ',
		    margin: '0 5 0 0',
		    listenToGlobalEvent: false,
		    onlineHelp: 'pmg_documentation_index'
		},
		{
		    reference: 'logoutButton',
		    xtype: 'button',
		    iconCls: 'fa fa-sign-out',
		    text: gettext('Logout')
		}
	    ]
	},
	{
	    xtype: 'panel',
	    scrollable: 'y',
	    border: false,
	    region: 'west',
	    layout: {
		type: 'vbox',
		align: 'stretch'
	    },
	    items: [{
		xtype: 'navigationtree',
		minWidth: 180,
		reference: 'navtree',
		// we have to define it here until extjs 6.2
		// because of a bug where a viewcontroller does not detect
		// the selectionchange event of a treelist
		listeners: {
		    selectionchange: 'navigate'
		}
	    }, {
		xtype: 'box',
		cls: 'x-treelist-nav',
		flex: 1
	    }]
	},
	{
	    xtype: 'panel',
	    layout: { type: 'card' },
	    region: 'center',
	    border: false,
	    reference: 'contentpanel'
	}
    ]
});
/*global Proxmox*/
Ext.define('PMG.QuarantineList', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgQuarantineList',

    emptyText: gettext('No E-Mail address selected'),
    viewConfig: {
	deferEmptyText: false
    },

    config: {
	emailSelection: false,
	notFoundText: gettext('No data in database.')
    },

    statics: {
	from: 0,
	to: 0
    },

    controller: {
	xclass: 'Ext.app.ViewController',

        init: function(view) {
	    var me = this;
	    if (PMG.view === 'quarantineview') {
		view.emailSelection = false;
		me.setEmptyText();
	    }
	    var emailCombobox = me.lookupReference('email');
	    emailCombobox.setVisible(view.emailSelection);
	    emailCombobox.setDisabled(!view.emailSelection);

	    var from;
	    if (PMG.QuarantineList.from != 0) {
		from = new Date(PMG.QuarantineList.from * 1000);
	    } else {
		from = new Date();
	    }

	    var to;
	    if (PMG.QuarantineList.to != 0) {
		to = new Date(PMG.QuarantineList.to * 1000);
	    } else {
		to = new Date();
	    }

	    // we to this to trigger the change event of those fields
	    me.lookupReference('from').setValue(from);
	    me.lookupReference('to').setValue(to);

	    Proxmox.Utils.monStoreErrors(view.getView(), view.getStore());
	    me.load(function() {
		if (view.cselect) {
		    view.setSelection(view.getStore().getById(view.cselect));
		}
	    });
        },
	// extjs has no method to dynamically change the emptytext on
	// grids, so we have to do it this way
	setEmptyText: function(emptyText) {
	    var me = this;
	    var view = me.getView();
	    var tableview = view.getView();
	    tableview.emptyText = '<div class="x-grid-empty">'+ (emptyText || view.notFoundText) + '</div>';
	},

	load: function(callback) {
	    var me = this;
	    var view = me.getView();
	    if (view.emailSelection) {
		if (!me.lookupReference('email').getSelection()) {
		    // if the combobox has no selection we do not reload
		    return;
		}
		me.setEmptyText();
	    }
	    view.getStore().load(callback);
	},

	setFrom: function(from) {
	    var me = this.getView();
	    var params = me.getStore().getProxy().getExtraParams();
	    params.starttime = from;
	    PMG.QuarantineList.from = from;
	    me.getStore().getProxy().setExtraParams(params);
	},

	setTo: function(to) {
	    var end_of_to = to + 24*60*60; // we want the end of the day
	    var me = this.getView();
	    var params = me.getStore().getProxy().getExtraParams();
	    params.endtime = end_of_to;
	    PMG.QuarantineList.to = to; // we save the start of the day here
	    me.getStore().getProxy().setExtraParams(params);
	},

	setUser: function(user) {
	    var me = this.getView();
	    var params = me.getStore().getProxy().getExtraParams();
	    params.pmail = user;
	    me.getStore().getProxy().setExtraParams(params);
	    me.user = user;
	},

	changeTime: function(field, value) {
	    var me = this;
	    var list = me.getView();

	    if (!value) {
		return;
	    }

	    var val = value.getTime()/1000;
	    var combobox = me.lookupReference('email');
	    var params = combobox.getStore().getProxy().getExtraParams();

	    var to = me.lookupReference('to');
	    var from = me.lookupReference('from');

	    if (field.name === 'from') {
		me.setFrom(val);
		params.starttime = val;
		to.setMinValue(value);

	    } else if (field.name === 'to') {
		me.setTo(val);
		params.endtime = val + 24*60*60;
		from.setMaxValue(value);
	    } else {
		return;
	    }

	    // the combobox does not know anything about the extraparams
	    // so we disable queryCaching until we expand (and query) again
	    combobox.queryCaching = false;
	    combobox.getStore().getProxy().setExtraParams(params);

	    me.load();
	},

	resetEmail: function() {
	    var me = this;
	    var view = me.getView();
	    if (view.emailSelection) {
		me.setUser(undefined);
	    }
	},

	changeEmail: function(tb, value) {
	    var me = this;
	    me.setUser(value);
	    me.load();
	},

	setQueryCaching: function() {
	    this.lookupReference('email').queryCaching = true;
	},

	control: {
	    '#':{
		beforedestroy: 'resetEmail'
	    },
	    'combobox[reference=email]': {
		change: {
		    fn: 'changeEmail',
		    buffer: 500
		},
		expand: 'setQueryCaching'
	    },
	    datefield: {
		change: {
		    fn: 'changeTime'
		}
	    }

	}
    },

    features: [
	{
	    ftype: 'grouping',
	    groupHeaderTpl: '{columnName}: {name} ({children.length})'
	}
    ],

    tbar: {
	layout: {
	    type: 'vbox',
	    align: 'stretch'
	},
	defaults: {
	    margin: 2
	},
	items: [
	    {
		fieldLabel: gettext('From'),
		reference: 'from',
		xtype: 'datefield',
		format: 'Y-m-d',
		name: 'from'
	    },
	    {
		fieldLabel: gettext('To'),
		reference: 'to',
		xtype: 'datefield',
		format: 'Y-m-d',
		name: 'to'
	    },
	    {
		xtype: 'combobox',
		hidden: true,
		displayField: 'mail',
		valueField: 'mail',
		listConfig: {
		    emptyText:
			'<div class="x-grid-empty">' +
			    gettext('No Data in Database') +
			'</div>'
		},
		store: {
		    proxy: {
			type: 'proxmox',
			url: '/api2/json/quarantine/spamusers'
		    },
		    fields: [
			{
			    name: 'mail',
			    renderer: Ext.htmlEncode
			}
		    ]
		},
		queryParam: false,
		queryCaching: false,
		editable: false,
		reference: 'email',
		name: 'email',
		fieldLabel: 'E-Mail'
	    }
	]
    }
});
Ext.define('PMG.grid.SpamInfoGrid',{
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgSpamInfoGrid',

    hideHeaders: true,

    store: {
	autoDestroy: true,
	fields: [ 'desc', 'name', { type: 'number', name: 'score' } ],
	proxy: {
	    type: 'proxmox',
	    root: 'data.spaminfo'
	}
    },

    setID: function(rec) {
	var me = this;
	if (!rec || !rec.data || !rec.data.id) {
	    me.getStore().removeAll();
	    return;
	}
	var url = '/api2/json/quarantine/content?id=' + rec.data.id;
	me.store.proxy.setUrl(url);
	me.store.load();
    },

    emptyText: gettext('No Spam Info'),
    hidden: true,

    features: [{
	ftype: 'summary'
    }],

    columns: [
	{
	    text: gettext('Test Name'),
	    dataIndex: 'name',
	    flex: 2,
	    summaryType: 'count',
	    summaryRenderer: function(value, summaryData, dataIndex, metaData) {
		return gettext('Spamscore');
	    }
	},
	{
	    text: gettext('Score'),
	    dataIndex: 'score',
	    align: 'right',
	    summaryType: 'sum',
	    summaryRenderer: function(value, summaryData, dataIndex, metaData) {
		return Ext.util.Format.round(value, 5);
	    }
	},
	{
	    text: gettext('Description'),
	    dataIndex: 'desc',
	    flex: 3
	}
    ]
});
/*global Proxmox*/
/*jslint confusion: true*/
/*format is a string and a function*/
Ext.define('pmg-spam-archive', {
    extend: 'Ext.data.Model',
    fields: [
	{ type: 'number', name: 'spamavg' },
	{ type: 'integer', name: 'count' },
        { type: 'date', dateFormat: 'timestamp', name: 'day' }
    ],
    proxy: {
        type: 'proxmox',
        url: "/api2/json/quarantine/spam"
    },
    idProperty: 'day'
});

Ext.define('pmg-spam-list', {
    extend: 'Ext.data.Model',
    fields: [ 'id', 'envelope_sender', 'from', 'sender', 'receiver', 'subject',
	{ type: 'number', name: 'spamlevel' },
	{ type: 'integer', name: 'bytes' },
	{ type: 'date', dateFormat: 'timestamp', name: 'time' },
	{
	    type: 'string',
	    name: 'day',
	    convert: function(v, rec) {
		return Ext.Date.format(rec.get('time'), 'Y-m-d');
	    }, depends: ['time']
	}
    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/quarantine/spam"
    },
    idProperty: 'id'
});

Ext.define('PMG.SpamQuarantine', {
    extend: 'Ext.container.Container',
    xtype: 'pmgSpamQuarantine',

    border: false,
    layout: { type: 'border' },

    defaults: { border: false },

    // from mail link
    cselect: undefined,

    controller: {

	xclass: 'Ext.app.ViewController',

	updatePreview: function(raw, rec) {
	    var preview = this.lookupReference('preview');

	    if (!rec || !rec.data || !rec.data.id)  {
		preview.update('');
		preview.setDisabled(true);
		return;
	    }

	    var url = '/api2/htmlmail/quarantine/content?id=' + rec.data.id + ((raw)?'&raw=1':'');
	    preview.setDisabled(false);
	    preview.update("<iframe frameborder=0 width=100% height=100% sandbox='allow-same-origin' src='" + url +"'></iframe>");
	},

	toggleRaw: function(button) {
	    var me = this;
	    var list = this.lookupReference('list');
	    var rec = list.selModel.getSelection()[0];
	    me.raw = !me.raw;
	    me.updatePreview(me.raw, rec);
	},

	btnHandler: function(button, e) {
	    var list = this.lookupReference('list');
	    var selected = list.getSelection();
	    if (!selected.length) {
		return;
	    }

	    var action = button.reference;

	    PMG.Utils.doQuarantineAction(action, selected[0].data.id, function() {
		list.getController().load();
	    });
	},

	onSelectMail: function() {
	    var me = this;
	    var list = this.lookupReference('list');
	    var rec = list.selModel.getSelection()[0];

	    me.updatePreview(me.raw || false, rec);
	    me.lookupReference('spaminfo').setID(rec);
	},

	toggleSpamInfo: function(btn) {
	    var grid = this.lookupReference('spaminfo');
	    grid.setVisible(!grid.isVisible());
	},

	init: function(view) {
	    this.lookup('list').cselect = view.cselect;
	},

	control: {
	    'button[reference=raw]': {
		click: 'toggleRaw'
	    },
	    'button[reference=spam]': {
		click: 'toggleSpamInfo'
	    },
	    'pmgQuarantineList': {
		selectionChange: 'onSelectMail'
	    }
	}
    },

    items: [
	{
	    title: gettext('Spam Quarantine'),
	    xtype: 'pmgQuarantineList',
	    emailSelection: true,
	    reference: 'list',
	    region: 'west',
	    width: 500,
	    split: true,
	    collapsible: false,
	    store: {
		model: 'pmg-spam-list',
		groupField: 'day',
		groupDir: 'DESC',
		sorters: [{
		    property: 'time',
		    direction: 'DESC'
		}]
	    },

	    columns: [
		{
		    header: gettext('Sender/Subject'),
		    dataIndex: 'subject',
		    renderer: PMG.Utils.sender_renderer,
		    flex: 1
		},
		{
		    header: gettext('Score'),
		    dataIndex: 'spamlevel',
		    align: 'right',
		    width: 70
		},
		{
		    header: gettext('Size') + ' (KB)',
		    renderer: function(v) { return Ext.Number.toFixed(v/1024, 0); },
		    dataIndex: 'bytes',
		    align: 'right',
		    width: 90
		},
		{
		    header: gettext('Date'),
		    dataIndex: 'day',
		    hidden: true
		},
		{
		    xtype: 'datecolumn',
		    header: gettext('Time'),
		    dataIndex: 'time',
		    format: 'H:i:s'
		}
	    ]
	},
	{
	    title: gettext('Selected Mail'),
	    border: false,
	    region: 'center',
	    split: true,
	    reference: 'preview',
	    disabled: true,
	    dockedItems: [
		{
		    xtype: 'toolbar',
		    dock: 'top',
		    items: [
			{
			    xtype: 'button',
			    reference: 'raw',
			    text: gettext('Toggle Raw'),
			    enableToggle: true,
			    iconCls: 'fa fa-file-code-o'
			},
			{
			    xtype: 'button',
			    reference: 'spam',
			    text: gettext('Toggle Spam Info'),
			    enableToggle: true,
			    iconCls: 'fa fa-bullhorn'
			},
			'->',
			{
			    reference: 'whitelist',
			    text: gettext('Whitelist'),
			    iconCls: 'fa fa-check',
			    handler: 'btnHandler'
			},
			{
			    reference: 'blacklist',
			    text: gettext('Blacklist'),
			    iconCls: 'fa fa-times',
			    handler: 'btnHandler'
			},
			{
			    reference: 'deliver',
			    text: gettext('Deliver'),
			    iconCls: 'fa fa-paper-plane-o',
			    handler: 'btnHandler'
			},
			{
			    reference: 'delete',
			    text: gettext('Delete'),
			    iconCls: 'fa fa-trash-o',
			    handler: 'btnHandler'
			}
		    ]
		},
		{
		    xtype: 'pmgSpamInfoGrid',
		    border: false,
		    reference: 'spaminfo'
		}
	    ]
	}
    ]
});
/*global Proxmox*/
Ext.define('pmg-address-list', {
    extend: 'Ext.data.Model',
    fields: [ 'address' ],
    idProperty: 'address'
});

// base class - do not use directly
Ext.define('PMG.UserBlackWhiteList', {
    extend: 'Ext.grid.GridPanel',

    border: false,
    listname: undefined, // 'blacklist' or 'whitelist',

    emptyText: gettext('No data in database.'),

    controller: {

        xclass: 'Ext.app.ViewController',

	onAddAddress: function() {
	    var me = this.getView();
	    var params = me.getStore().getProxy().getExtraParams() || {};

	    var url = '/quarantine/' + me.listname;

	    var items = [{
		xtype: 'proxmoxtextfield',
		name: 'address',
		fieldLabel: gettext("Address")
	    }];

	    Ext.Object.each(params, function(key, value) {
		items.push({
		    xtype: 'hidden',
		    name: key,
		    value: value
		});
	    });

	    var config = {
		method: 'POST',
		url: url,
		isCreate: true,
		isAdd: true,
		items: items
	    };

	    if (me.listname === 'blacklist') {
		config.subject = gettext("Blacklist");
	    } else if (me.listname == 'whitelist') {
		config.subject = gettext("Whitelist");
	    } else {
		throw "unknown list - internal error";
	    }

	    var win = Ext.createWidget('proxmoxWindowEdit', config);
	    win.on('destroy', function() { me.store.load(); });
	    win.show();
	},

	onRemoveAddress: function() {
	    var me = this.getView();
	    var rec = me.selModel.getSelection()[0];
	    if (!rec) {
		return;
	    }

	    var params = me.getStore().getProxy().getExtraParams() || {};
	    var url = '/quarantine/' + me.listname + '/' + rec.getId();

	    Proxmox.Utils.API2Request({
		url: url + '?' + Ext.Object.toQueryString(params),
		method: 'DELETE',
		waitMsgTarget: me,
		callback: function(options, success, response) {
		    me.store.load();
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	changeEmail: function(combobox, value) {
	    var view = this.getView();
	    if (value && combobox.isValid()) {
		view.getStore().getProxy().setExtraParams({
		    pmail: value
		});
		view.getStore().load();
	    }
	},

	init: function(view) {
	    if (PMG.view === 'quarantineview') {
		this.lookupReference('email').setVisible(false);
		view.getStore().load();
	    }
	    Proxmox.Utils.monStoreErrors(view.getView(), view.getStore(), true);
	},

	control: {
	    'combobox':{
		change: {
		    fn: 'changeEmail',
		    buffer: 500
		}
	    }
	}
    },

    tbar: [
	{
	    xtype: 'combobox',
	    displayField: 'mail',
	    vtype: 'email',
	    allowBlank: false,
	    valueField: 'mail',
	    store: {
		proxy: {
		    type: 'proxmox',
		    url: '/api2/json/quarantine/quarusers'
		},
		fields: [
		    {
			name: 'mail',
			renderer: Ext.htmlEncode
		    }
		]
	    },
	    queryParam: false,
	    queryCaching: false,
	    editable: true,
	    reference: 'email',
	    name: 'email',
	    fieldLabel: 'E-Mail'
	},
	{
	    text: gettext('Add'),
	    handler: 'onAddAddress'
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Remove'),
	    disabled: true,
	    handler: 'onRemoveAddress',
	    confirmMsg: function(rec) {
		var me = this;

		var name = rec.getId();
		return Ext.String.format(
		    gettext('Are you sure you want to remove entry {0}'),
		    "'" + Ext.String.htmlEncode(name) + "'");
	    }
	}
    ],

    columns: [
        {
            header: gettext('Address'),
            dataIndex: 'address',
	    renderer: Ext.String.htmlEncode,
	    flex: 1
        }
    ]
});

Ext.define('PMG.UserBlacklist', {
    extend: 'PMG.UserBlackWhiteList',
    xtype: 'pmgUserBlacklist',

    title: gettext('Blacklist'),

    listname: 'blacklist',

    store: {
	model: 'pmg-address-list',
	autoDestroy: true,
	proxy: {
            type: 'proxmox',
            url: "/api2/json/quarantine/blacklist"
	}
    },

    dockedItems: [
	{
            dock: 'top',
	    bodyStyle: {
		padding: '10px',
		'border-left': '0px',
		'border-right': '0px'
	    },
            html: gettext('With this feature, you can manually mark E-mails from certain domains or addresses as spam.') + '<br><br>' +
		'<b>*.com</b> (all mails from <b>.com</b> domains)' + '<br>' +
		'<b>*@example.com</b> (all mails from domain <b>example.com</b>)' + '<br>' +
		'<b>john@example.com</b> (all mails from <b>john@example.com</b>)'

        }
    ]
});

Ext.define('PMG.UserWhitelist', {
    extend: 'PMG.UserBlackWhiteList',
    xtype: 'pmgUserWhitelist',

    title: gettext('Whitelist'),

    listname: 'whitelist',

    store: {
	model: 'pmg-address-list',
	autoDestroy: true,
	proxy: {
            type: 'proxmox',
            url: "/api2/json/quarantine/whitelist"
	}
    },

    dockedItems: [
	{
            dock: 'top',
	    bodyStyle: {
		padding: '10px',
		'border-left': '0px',
		'border-right': '0px'
	    },
            html: gettext('With this feature, you can manually bypass spam checking for certain domains or E-mail addresses.') + '<br><br>' +
		'<b>*.com</b> (all mails from <b>.com</b> domains)' + '<br>' +
		'<b>*@example.com</b> (all mails from domain <b>example.com</b>)' + '<br>' +
		'<b>john@example.com</b> (all mails from <b>john@example.com</b>)'
        }
    ]
});
/*global Proxmox*/
Ext.define('PMG.QuarantineNavigationTree', {
    extend: 'Ext.list.Tree',
    xtype: 'quarantinenavigationtree',

    select: function(path) {
	var me = this;
	var item = me.getStore().findRecord('path', path, 0, false, true, true);
	me.setSelection(item);
    },

    store: {
	root: {
	    expanded: true,
	    children: [
		{
		    text: gettext('Spam Quarantine'),
		    iconCls: 'fa fa-cubes',
		    path: 'pmgSpamQuarantine',
		    expanded: true,
		    children: [
			{
			    text: gettext('Whitelist'),
			    iconCls: 'fa fa-file-o',
			    path: 'pmgUserWhitelist',
			    leaf: true
			},
			{
			    text: gettext('Blacklist'),
			    iconCls: 'fa fa-file',
			    path: 'pmgUserBlacklist',
			    leaf: true
			}
		    ]
		}
	    ]
	}
    },

    animation: false,
    expanderOnly: true,
    expanderFirst: false,
    ui: 'nav'
});

Ext.define('PMG.QuarantineView', {
    extend: 'Ext.container.Container',
    xtype: 'quarantineview',

    title: 'Proxmox Mail Gateway Quarantine',

    controller: {
	xclass: 'Ext.app.ViewController',
	routes: {
	    ':path:subpath': {
		action: 'changePath',
		before: 'beforeChangePath',
                conditions : {
		    ':path'    : '(?:([%a-zA-Z0-9\\-\\_\\s,]+))',
		    ':subpath' : '(?:(?::)([%a-zA-Z0-9\\-\\_\\s,]+))?'
		}
	    }
	},

	beforeChangePath: function(path, subpath, action) {
	    var me = this;

	    if (!Ext.ClassManager.getByAlias('widget.'+ path)) {
		console.warn('xtype "'+path+'" not found');
		action.stop();
		return;
	    }

	    var lastpanel = me.lookupReference('contentpanel').getLayout().getActiveItem();
	    if (lastpanel && lastpanel.xtype === path) {
		// we have the right component already,
		// we just need to select the correct tab
		// default to the first
		subpath = subpath || 0;
		if (lastpanel.getActiveTab) {
		    // we assume lastpanel is a tabpanel
		    if (lastpanel.getActiveTab().getItemId() !== subpath) {
			// set the active tab
			lastpanel.setActiveTab(subpath);
		    }
		    // else we are already there
		}
		action.stop();
		return;
	    }

	    action.resume();
	},

	changePath: function(path,subpath) {
	    var me = this;
	    var contentpanel = me.lookupReference('contentpanel');
	    var lastpanel = contentpanel.getLayout().getActiveItem();

	    var obj = contentpanel.add({ xtype: path, cselect: subpath });
	    var treelist = me.lookupReference('navtree');

	    treelist.suspendEvents();
	    treelist.select(path);
	    treelist.resumeEvents();

	    if (Ext.isFunction(obj.setActiveTab)) {
		obj.setActiveTab(subpath || 0);
		obj.addListener('tabchange', function(tabpanel, newc, oldc) {
		    var newpath = path;

		    // only add the subpath part for the
		    // non-default tabs
		    if (tabpanel.items.findIndex('id', newc.id) !== 0) {
			newpath += ":" + newc.getItemId();
		    }

		    me.redirectTo(newpath);
		});
	    }

	    contentpanel.setActiveItem(obj);

	    if (lastpanel) {
		contentpanel.remove(lastpanel, { destroy: true });
	    }
	},

	logout: function() {
	    PMG.app.logout();
	},

	navigate: function(treelist, item) {
	    this.redirectTo(item.get('path'));
	},

	execQuarantineAction: function(qa) {
	    PMG.Utils.doQuarantineAction(qa.action, qa.cselect);
	},

	control: {
	    'button[reference=logoutButton]': {
		click: 'logout'
	    }
	},

	init: function(view) {
	    var me = this;

	    // load username
	    var username = Proxmox.UserName.replace(/\@quarantine$/, '');
	    me.lookupReference('usernameinfo').update({username: username});

	    // show login on requestexception
	    // fixme: what about other errors
	    Ext.Ajax.on('requestexception', function(conn, response, options) {
		if (response.status == 401) { // auth failure
		    me.logout();
		}
	    });

	    var qa = PMG.Utils.extractQuarantineAction();
	    var token;
	    if (qa) {
		token = 'pmgSpamQuarantine';
		if (qa.action === 'blacklist') { token = 'pmgUserBlacklist'; }
		if (qa.action === 'whitelist') { token = 'pmgUserWhitelist'; }
		if (qa.cselect) {
		    token += ':' + qa.cselect;
		}
		this.redirectTo(token, true);
		if (qa.action) {
		    me.execQuarantineAction(qa);
		}
	    } else {
		// select treeitem and load page from url fragment
		
		token = Ext.util.History.getToken() || 'pmgSpamQuarantine';
		this.redirectTo(token, true);
	    }
	}
    },

    plugins: 'viewport',

    layout: {
	type: 'border'
    },

    items: [
	{
	    region: 'north',
	    xtype: 'container',
	    layout: {
		type: 'hbox',
		align: 'middle'
	    },
	    margin: '2 5 2 5',
	    height: 38,
	    items: [
		{
		    xtype: 'proxmoxlogo'
		},
		{
		    xtype: 'versioninfo'
		},
		{
		    flex: 1
		},
		{
		    baseCls: 'x-plain',
		    reference: 'usernameinfo',
		    padding: '0 5',
		    tpl: Ext.String.format(gettext("You are logged in as '{0}'"), '{username}')
		},
		{
		    reference: 'logoutButton',
		    xtype: 'button',
		    iconCls: 'fa fa-sign-out',
		    text: gettext('Logout')
		}
	    ]
	},
	{
	    xtype: 'quarantinenavigationtree',
	    reference: 'navtree',
	    minWidth: 177,
	    border: false,
	    region: 'west',
	    // we have to define it here until extjs 6.2
	    // because of a bug where a viewcontroller does not detect
	    // the selectionchange event of a treelist
	    listeners: {
		selectionchange: 'navigate'
	    }
	},
	{
	    xtype: 'panel',
	    layout: {
		type: 'card'
	    },
	    region: 'center',
	    border: false,
	    reference: 'contentpanel'
	}
    ]
});
/*global Proxmox*/
/*jslint confusion: true*/
/* load is a function and string
 * hours is a number and string
 * timespan is a number and string
 * bind is a function and object
 * handler is a function and string
 */
Ext.define('PMG.Dashboard', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgDashboard',

    controller: {
	xclass: 'Ext.app.ViewController',

	openDashboardOptions: function() {
	    var me = this;
	    var viewModel = me.getViewModel();
	    Ext.create('Ext.window.Window', {
		modal: true,
		width: 300,
		title: gettext('Dashboard Options'),
		layout: {
		    type: 'auto'
		},
		items: [{
		    xtype: 'form',
		    bodyPadding: '10 10 10 10',
		    defaultButton: 'savebutton',
		    items: [{
			xtype: 'proxmoxintegerfield',
			itemId: 'hours',
			labelWidth: 100,
			anchor: '100%',
			allowBlank: false,
			minValue: 1,
			maxValue: 24,
			value: viewModel.get('hours'),
			fieldLabel: gettext('Hours to show')
		    }],
		    buttons: [{
			text: gettext('Save'),
			reference: 'loginButton',
			formBind: true,
			handler: function() {
			    var win = this.up('window');
			    var hours = win.down('#hours').getValue();
			    me.setHours(hours, true);
			    win.close();
			}
		    }]
		}]
	    }).show();
	},

	setHours: function(hours, setState) {
	    var me = this;
	    var viewModel = me.getViewModel();
	    viewModel.set('hours', hours);
	    viewModel.notify();

	    Ext.Array.forEach(['recentmails', 'receivers'], function(item) {
		viewModel.getStore(item).reload();
	    });

	    if (setState) {
		var sp = Ext.state.Manager.getProvider();
		sp.set('dashboard-hours', hours);
	    }
	},

	updateMailStats: function(store, records, success) {
	    if (!success) {
		return;
	    }
	    var me = this;
	    var viewModel = me.getViewModel();

	    var count = 0;
	    var bytes_in = 0;
	    var bytes_out = 0;
	    var ptime = 0;
	    var avg_ptime = 'N/A';

	    records.forEach(function(item) {
		bytes_in += item.data.bytes_in;
		bytes_out += item.data.bytes_out;
		// unnormalize
		count += (item.data.count*item.data.timespan)/60;
		ptime += item.data.ptimesum;
	    });

	    if (count) {
		avg_ptime = (ptime/count).toFixed(2) + " s";
	    }

	    viewModel.set('bytes_in', Proxmox.Utils.format_size(bytes_in));
	    viewModel.set('bytes_out', Proxmox.Utils.format_size(bytes_out));
	    viewModel.set('avg_ptime', avg_ptime);
	},

	updateClusterStats: function(store, records, success) {
	    if (!success) {
		return;
	    }
	    var me = this;
	    var viewmodel = me.getViewModel();

	    var subStatus = 2; // 2 = all good, 1 = different leves, 0 = none
	    var subLevel = "";

	    var cpu = 0;
	    var mem = 0;
	    var hd = 0;
	    var count = records.length;

	    records.forEach(function(item) {
		// subscription level check
		if (subStatus && item.data.level) {
		    if (subLevel !== "" && subLevel !== item.data.level) {
			subStatus = 1;
		    } else if (subLevel === "") {
			subLevel = item.data.level;
		    }
		} else {
		    subStatus = 0;
		}

		// resources count
		cpu += item.data.cpu;
		mem += (item.data.memory.used/item.data.memory.total);
		hd += (item.data.rootfs.used/item.data.rootfs.total);
	    });

	    var subscriptionPanel = me.lookup('subscription');
	    subscriptionPanel.setSubStatus(subStatus);

	    cpu = cpu/count;
	    mem = mem/count;
	    hd = hd/count;

	    var cpuPanel = me.lookup('cpu');
	    cpuPanel.updateValue(cpu);

	    var memPanel = me.lookup('mem');
	    memPanel.updateValue(mem);

	    var hdPanel = me.lookup('hd');
	    hdPanel.updateValue(hd);
	},

	init: function(view) {
	    var me = this;
	    var sp = Ext.state.Manager.getProvider();
	    var hours = sp.get('dashboard-hours') || 12;
	    me.setHours(hours, false);
	}
    },

    viewModel: {
	data: {
	    timespan: 300, // in seconds
	    hours: 12, // in hours
	    'bytes_in': 0,
	    'bytes_out': 0,
	    'avg_ptime': 0.0
	},

	stores: {
	    cluster: {
		storeid: 'dash-cluster',
		type: 'update',
		interval: 5000,
		autoStart: true,
		autoLoad: true,
		autoDestroy: true,
		proxy: {
		    extraParams: { list_single_node: 1 },
		    type: 'proxmox',
		    url: '/api2/json/config/cluster/status'
		},
		listeners: {
		    load: 'updateClusterStats'
		}
	    },
	    recentmails: {
		storeid: 'dash-recent',
		interval: 5000,
		type: 'update',
		autoStart: true,
		autoLoad: true,
		autoDestroy: true,
		proxy: {
		    type: 'proxmox',
		    url: '/api2/json/statistics/recent',
		    extraParams: {
			hours: '{hours}',
			timespan: '{timespan}'
		    }
		},
		fields: [
		    {
			type: 'number', name: 'count',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    {
			type: 'number', name: 'count_in',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    {
			type: 'number', name: 'count_out',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    {
			type: 'number', name: 'spam',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    {
			type: 'number', name: 'spam_in',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    {
			type: 'number', name: 'spam_out',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    {
			type: 'number', name: 'virus',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    {
			type: 'number', name: 'virus_in',
			convert: PMG.Utils.convert_field_to_per_min
		    },
		    { type: 'integer', name: 'virus_out' },
		    { type: 'integer', name: 'bytes_in' },
		    { type: 'integer', name: 'bytes_out' },
		    { type: 'number', name: 'ptimesum' },
		    { type: 'date', dateFormat: 'timestamp', name: 'time' }
		],
		listeners: {
		    load: 'updateMailStats'
		}
	    },
	    receivers: {
		storeid: 'dash-receivers',
		interval: 10000,
		type: 'update',
		autoStart: true,
		autoLoad: true,
		autoDestroy: true,
		proxy: {
		    type: 'proxmox',
		    url: '/api2/json/statistics/recentreceivers',
		    extraParams: {
			hours: '{hours}'
		    }
		},
		fields: [
		    { type: 'integer', name: 'count' },
		    { type: 'string', name: 'receiver' }
		]
	    }
	}
    },

    bind: {
	title: gettext('Dashboard') + ' (' +
	    Ext.String.format(gettext('{0} hours'), '{hours}') + ')'
    },

    layout: {
	type: 'column'
    },
    border: false,

    bodyPadding: '20 0 0 20',

    defaults: {
	columnWidth: 0.5,
	xtype: 'panel',
	margin: '0 20 20 0'
    },

    tools: [
	{
	    type: 'gear',
	    handler: 'openDashboardOptions'
	}
    ],

    scrollable: true,

    items: [
	{
	    height: 300,
	    flex: 1,
	    iconCls: 'fa fa-tachometer',
	    title: gettext('E-Mail Volume'),
	    layout: {
		type: 'vbox',
		align: 'stretch'
	    },
	    defaults: {
		xtype: 'pmgMiniGraph',
		bind: {
		    store: '{recentmails}'
		}
	    },
	    items: [
		{
		    fields: ['count'],
		    fieldTitles: [ gettext('Mails / min') ],
		    seriesConfig: {
			colors: [ '#00617F' ],
			style: {
			    opacity: 0.60,
			    lineWidth: 1
			},
			highlightCfg: {
			    opacity: 1,
			    scaling: 1
			}
		    }
		},
		{
		    fields: ['spam'],
		    fieldTitles: [ gettext('Spam / min') ],
		    seriesConfig: {
			colors: [ '#E67300' ],
			style: {
			    opacity: 0.60,
			    lineWidth: 1
			},
			highlightCfg: {
			    opacity: 1,
			    scaling: 1
			}
		    }
		}
	    ]
	},
	{
	    xtype: 'container',
	    height: 300,
	    layout: {
		type: 'vbox',
		align: 'stretch'
	    },
	    items: [
		{
		    xtype: 'pmgMailProcessing',
		    title: gettext('E-Mail Processing'),
		    iconCls: 'fa fa-hourglass-half',
		    height: 180,
		    bind: {
			data: {
			    'bytes_in': '{bytes_in}',
			    'bytes_out': '{bytes_out}',
			    'avg_ptime': '{avg_ptime}'
			}
		    }
		},
		{
		    iconCls: 'fa fa-ticket',
		    title: 'Subscription',
		    reference: 'subscription',
		    xtype: 'pmgSubscriptionInfo',
		    margin: '10 0 0 0',
		    height: 110
		}
	    ]
	},
	{
	    height: 250,
	    iconCls: 'fa fa-tasks',
	    title: 'Node Resources',
	    bodyPadding: '0 20 0 20',
	    layout: {
		type: 'hbox',
		align: 'center'
	    },
	    defaults: {
		xtype: 'proxmoxGauge',
		spriteFontSize: '20px',
		flex: 1
	    },
	    items: [
		{
		    title: gettext('CPU'),
		    reference: 'cpu'
		},
		{
		    title: gettext('Memory'),
		    reference: 'mem'
		},
		{
		    title: gettext('Storage'),
		    reference: 'hd'
		}
	    ]
	},
	{
	    height: 250,
	    iconCls: 'fa fa-list',
	    title: gettext('Top Receivers'),

	    bodyPadding: '20 20 20 20',
	    layout: {
		type: 'vbox',
		pack: 'center',
		align: 'stretch'
	    },
	    items: [{
		xtype: 'grid',
		bind: {
		    store: '{receivers}'
		},

		emptyText: gettext('No data in database'),

		// remove all borders/lines/headers
		border: false,
		bodyBorder: false,
		hideHeaders: true,
		header: false,
		columnLines: false,
		rowLines: false,
		viewConfig: {
		    stripeRows: false
		},

		columns: [
		    {
			dataIndex: 'receiver',
			flex: 1,
			text: gettext('Receiver')
		    },
		    {
			dataIndex: 'count',
			align: 'right',
			text: gettext('Count')
		    }
		]
	    }]
	}
    ]
});
Ext.define('PMG.dashboard.MailProcessing', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgMailProcessing',

    setData: function(data) {
	var me = this;
	me.down('#ptime').update(data);
	me.down('#traffic').update(data);
    },

    layout: {
	type: 'hbox',
	align: 'center',
	pack: 'center'
    },

    defaults: {
	xtype: 'box',
	flex: 1,
	style: {
	    'text-align':'center'
	}
    },

    items: [
	{
	    itemId: 'traffic',
	    data: {
		'bytes_in': 0,
		'bytes_out': 0
	    },
	    tpl: [
		'<h3><i class="fa fa-exchange green"></i> ' + gettext('Traffic') + '</h3>',
		'<table class="dash"><tr>',
		'<td class="right half"><h2>{bytes_in}</h2></td>',
		'<td class="left">' + PMG.Utils.format_rule_direction(0) + '</td>',
		'</tr><tr>',
		'<td class="right half"><h2>{bytes_out}</h2></td>',
		'<td class="left">' + PMG.Utils.format_rule_direction(1) + '</td>',
		'</tr></table>'
	    ]
	},
	{
	    itemId: 'ptime',
	    data: {
		'avg_ptime': 0
	    },
	    tpl: [
		'<h3><i class="fa fa-clock-o"></i> ' + gettext('Avg. Processing Time') + '</h3>',
		'<p><h2>{avg_ptime}</h2></p>'
	    ]
	}
    ]
});
Ext.define('PMG.dashboard.SubscriptionInfo', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgSubscriptionInfo',

    data: {
	icon: 'question-circle',
	message: gettext('Unknown')
    },

    style: {
	cursor: 'pointer'
    },

    setSubStatus: function(status) {
	var me = this;
	var data = {};

	switch (status) {
	    case 2:
		data.icon = 'check green';
		data.message = gettext('Your subscription status is valid.');
		break;
	    case 1: 
		data.icon = 'exclamation-triangle yellow';
		data.message = gettext('Warning: Your subscription levels are not the same.');
		break;
	    case 0: 
		data.icon = 'times-circle red';
		data.message = gettext('You have at least one node without subscription.');
		break;
	    default:
		throw 'invalid subscription status';
	}
	me.update(data);
    },
    tpl: [
	'<table style="height: 100%;" class="dash">',
	'<tr><td class="center">',
	'<i class="fa fa-3x fa-{icon}"></i>',
	'</td><td class="center">{message}</td></tr>',
	'</table>'
    ],

    listeners: {
	click: {
	    element: 'body',
	    fn: function() {
		var mainview = this.component.up('mainview');
		mainview.getController().redirectTo('pmgSubscription');
	    }
	}
    }
});
Ext.define('PMG.dashboard.MiniGraph', {
    extend: 'Proxmox.widget.RRDChart',
    xtype: 'pmgMiniGraph',

    legend: undefined,
    interactions: undefined,
    insetPadding: 20,
    padding: '5 5 0 0',
    axes: [
	{
	    type: 'numeric',
	    position: 'left',
	    minimum: 0,
	    grid: true,
	    majorTickSteps: 2,
	    label: {
		fillStyle: '#5f5f5f'
	    },
	    style: {
		axisLine: false,
		majorTickSize: 0
	    }
	},
	{
	    type: 'time',
	    position: 'bottom',
	    dateFormat: 'H:i',
	    fields: ['time'],
	    label: {
		fillStyle: '#5f5f5f'
	    },
	    style: {
		axisLine: false,
		majorTickSize: 0
	    }
	}
    ],
    border: false,
    flex: 1,
    noTool: true
});
/*global Proxmox*/
Ext.define('PMG.view.main.VersionInfo',{
    extend: 'Ext.Component',
    xtype: 'versioninfo',

    makeApiCall: true,

    data: {
	version: false
    },

    tpl: [
	'Mail Gateway',
	'<tpl if="version">',
	' {version}-{release}/{repoid}',
	'</tpl>',
	' <a href="https://bugzilla.proxmox.com">BETA</a>'
    ],

    initComponent: function() {
	var me = this;
	me.callParent();

	if (me.makeApiCall) {
	    Proxmox.Utils.API2Request({
		url: '/version',
		method: 'GET',
		success: function(response) {
		    me.update(response.result.data);
		}
	    });
	}
    }
});
/*global Proxmox*/
Ext.define('pmg-mail-tracker', {
    extend: 'Ext.data.Model',
    fields: [
	'id' , 'from', 'to', 'dstatus', 'rstatus', 'qid', 'msgid', 'client',
	{ type: 'number', name: 'size' },
	{ type: 'date', dateFormat: 'timestamp', name: 'time' }
    ],
    proxy: {
	type: 'proxmox'
    },
    // do not use field 'id', because "id/to" is the unique Id
    // this way we display an entry for each receiver
    idProperty: 'none'
});


Ext.define('PMG.MailTrackerFilter', {
    extend: 'Ext.container.Container',
    xtype: 'pmgMailTrackerFilter',

    layout: {
	type: 'hbox'
    },

    controller: {

        xclass: 'Ext.app.ViewController',

	onFilterChange: function() {
	    var view = this.getView();
	    view.fireEvent('filterChanged');
	},

	onSpecialKey: function(field, e) {
	    if (e.getKey() == e.ENTER) {
		this.onFilterChange();
	    }
	}
    },

    getFilterParams: function() {
	var me = this;
	var param = {};

	var names = ['from', 'target', 'xfilter', 'starttime', 'endtime', 'ndr', 'greylist'];
	Ext.Array.each(names, function(name) {
	    var value = me.lookupReference(name).getSubmitValue();
	    if (value) { param[name] = value; }
	});

	return param;
    },

    items: [
	{
	    width: 400,
	    border: false,
	    padding: 10,
	    layout: {
		type: 'vbox',
		align: 'stretch'
	    },
	    items: [
		{
		    fieldLabel: gettext('Sender'),
		    xtype: 'textfield',
		    listeners: { specialkey: 'onSpecialKey' },
		    reference: 'from'
		},
		{
		    fieldLabel: gettext('Receiver'),
		    xtype: 'textfield',
		    listeners: { specialkey: 'onSpecialKey' },
		    reference: 'target'
		},
		{
		    fieldLabel: gettext('Filter'),
		    xtype: 'textfield',
		    listeners: { specialkey: 'onSpecialKey' },
		    reference: 'xfilter'
		}
	    ]
	},
	{
	    width: 330,
	    border: false,
	    padding: 10,
	    layout: {
		type: 'vbox',
		align: 'stretch'
	    },
	    items: [
		{
		    fieldLabel: gettext('Start'),
		    reference: 'starttime',
		    listeners: { change: 'onFilterChange' },
		    value: (function() {
			var now = new Date();
			return new Date(now.getTime() - 3600000);
		    }()),
		    xtype: 'promxoxDateTimeField'
		},
		{
		    fieldLabel: gettext('End'),
		    reference: 'endtime',
		    listeners: { change: 'onFilterChange' },
		    xtype: 'promxoxDateTimeField'
		},
		{
		    layout: {
			type: 'hbox'
		    },
		    border: false,
		    items: [
			{
			    boxLabel: gettext('Include NDRs'),
			    xtype: 'proxmoxcheckbox',
			    listeners: { change: 'onFilterChange' },
			    reference: 'ndr',
			    name: 'ndrs'
			},
			{
			    boxLabel: gettext('Include Greylist'),
			    xtype: 'proxmoxcheckbox',
			    listeners: { change: 'onFilterChange' },
			    margin: { left: 20 },
			    reference: 'greylist',
			    name: 'greylist'
			}
		    ]
		}
	    ]
	}
    ]
});

Ext.define('PMG.MaiLogWindow', {
    extend: 'Ext.window.Window',
    xtype: 'pmgMaiLogWindow',

    title: gettext('Syslog'),

    logid: undefined,
    starttime: undefined,
    endtime: undefined,

    width: 1024,
    height: 400,
    scrollable: true,

    layout: {
	type: 'auto'
    },
    modal: true,
    bodyPadding: 5,

    load: function() {
	var me = this;

	Proxmox.Utils.API2Request({
	    method: 'GET',
	    params: { starttime: me.starttime, endtime: me.endtime },
	    url: '/nodes/' + Proxmox.NodeName + '/tracker/' + me.logid,
	    waitMsgTarget: me,
	    failure: function(response, opts) {
		me.update(gettext('Error') + " " + response.htmlStatus);
	    },
	    success: function(response, opts) {
		var data = response.result.data;

		var logs = "<pre style='margin: 0;'>";
		Ext.Array.each(data.logs, function(line) {
		    logs += Ext.htmlEncode(line) + "\n";
		});
		logs += "</pre>";
		me.update(logs);
	    }
	});
    },

    initComponent : function() {
	var me = this;

	if (!me.logid) {
	    throw "no logid specified";
	}

	if (!me.starttime) {
	    throw "no starttime specified";
	}
	if (!me.endtime) {
	    throw "no endtime specified";
	}

	me.callParent();

	me.setHtml('Loading...');
	me.load();
    }
});

Ext.define('PMG.MailTracker', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgMailTracker',

    title: gettext('Tracking Center'),

    border: false,

    emptyText: gettext('No data in database.'),
    disableSelection: true,

    viewConfig: {
	deferEmptyText: false,
	enableTextSelection: true,
	getRowClass: function(record, index) {
	    var status = record.data.rstatus || record.data.dstatus;
	    return PMG.Utils.mail_status_map[status];
	}
    },

    plugins: [
	{
	    ptype: 'rowexpander',
	    rowBodyTpl: '<p class="logs">{logs}</p>'
	}
    ],

    store: {
	autoDestroy: true,
	model: 'pmg-mail-tracker'
    },

    controller: {

        xclass: 'Ext.app.ViewController',

	onSearch: function() {
	    var view = this.getView();
	    var filter = this.lookupReference('filter');
	    var status = this.lookupReference('status');
	    var params = filter.getFilterParams();
	    status.update(''); // clear status before load
	    view.store.proxy.setExtraParams(params);
	    view.store.proxy.setUrl('/api2/json/nodes/' + Proxmox.NodeName + '/tracker');
	    view.store.load(function(records, operation, success) {
		var response = operation.getResponse();
		if (success) {
		    // fixme: howto avoid duplicate Ext.decode ?
		    var result = Ext.decode(response.responseText);
		    if (result.changes) {
			status.update(result.changes);
		    }
		}
	    });
	},

	showDetails: function(rowNode, record) {
	    var view = this.getView();

	    var params = view.store.proxy.getExtraParams();

	    Proxmox.Utils.API2Request({
		method: 'GET',
		params: { starttime: params.starttime, endtime: params.endtime },
		url: '/nodes/' + Proxmox.NodeName + '/tracker/' + record.data.id,
		waitMsgTarget: view,
		failure: function(response, opts) {
		    record.set('logs',gettext('Error') + " " + response.htmlStatus);
		},
		success: function(response, opts) {
		    var data = response.result.data;
		    var logs = "";

		    Ext.Array.each(data.logs, function(line) {
			logs += Ext.htmlEncode(line) + "<br>";
		    });

		    record.set('logs', logs);
		}
	    });
	},

	control: {
	    'gridview': {
		expandbody: 'showDetails'
	    }
	}
    },

    dockedItems: [
	{
	    xtype: 'pmgMailTrackerFilter',
	    reference: 'filter',
	    listeners:  { filterChanged: 'onSearch' },
	    border: false,
	    dock: 'top'
	},
	{
	    xtype: 'toolbar',
	    items: [
		{ text: 'Search', handler: 'onSearch' },
		{ xtype: 'component', html: '', reference: 'status' }
	    ]
	}
    ],

    columns: [
	{
	    xtype: 'datecolumn',
	    header: gettext('Time'),
	    width: 120,
	    dataIndex: 'time',
	    format: 'M d H:i:s'
	},
	{
	    header: gettext('From'),
	    flex: 1,
	    dataIndex: 'from',
	    renderer: Ext.htmlEncode
	},
	{
	    header: gettext('To'),
	    flex: 1,
	    dataIndex: 'to',
	    renderer: Ext.htmlEncode
	},
	{
	    header: gettext('Status'),
	    width: 150,
	    renderer: function(v, metaData, rec) {
		var returntext = 'unknown';
		var icon = 'question-circle';
		var rstatus = rec.data.rstatus;
		if (v !== undefined && v !== '') {
		    var vtext = PMG.Utils.mail_status_map[v] || v;
		    icon = v;
		    if (rstatus !== undefined && rstatus !== '') {
			var rtext = PMG.Utils.mail_status_map[rstatus] || rstatus;
			returntext = vtext + '/' + rtext;
			icon = rstatus;
		    } else if (rec.data.qid !== undefined) {
			returntext = 'queued/' + vtext;
		    } else {
			returntext = vtext;
		    }
		}

		return PMG.Utils.format_status_icon(icon) + returntext;
	    },
	    dataIndex: 'dstatus'
	},
	{
	    header: gettext('Size'),
	    hidden: true,
	    dataIndex: 'size'
	},
	{
	    header: 'MSGID',
	    width: 300,
	    hidden: true,
	    dataIndex: 'msgid',
	    renderer: Ext.htmlEncode
	},
	{
	    header: gettext('Client'),
	    width: 200,
	    hidden: true,
	    dataIndex: 'client',
	    renderer: Ext.htmlEncode
	}
    ],

    initComponent: function() {
	var me = this;

	me.callParent();

	Proxmox.Utils.monStoreErrors(me.getView(), me.store);
    }
});
Ext.define('PMG.store.NavigationStore', {
    extend: 'Ext.data.TreeStore',

    storeId: 'NavigationStore',

    root: {
	expanded: true,
	children: [
	    {
		text: gettext('Dashboard'),
		iconCls: 'fa fa-tachometer',
		path: 'pmgDashboard',
		leaf: true
	    },
	    {
		text: gettext('Mail Filter'),
		iconCls: 'fa fa-envelope',
		path: 'pmgRuleConfiguration',
		expanded: true,
		children: [
		    {
			text: gettext('Action Objects'),
			iconCls: 'fa fa-flag',
			path: 'pmgActionList',
			leaf: true
		    },
		    {
			text: gettext('Who Objects'),
			iconCls: 'fa fa-user-circle',
			path: 'pmgWhoConfiguration',
			leaf: true
		    },
		    {
			text: gettext('What Objects'),
			iconCls: 'fa fa-cube',
			path: 'pmgWhatConfiguration',
			leaf: true
		    },
		    {
			text: gettext('When Objects'),
			iconCls: 'fa fa-clock-o',
			path: 'pmgWhenConfiguration',
			leaf: true
		    }
		]
	    },
	    {
		text: gettext('Configuration'),
		iconCls: 'fa fa-gears',
		path: 'pmgSystemConfiguration',
		expanded: true,
		children: [
		    {
			text: gettext('Mail Proxy'),
			iconCls: 'fa fa-envelope-o',
			path: 'pmgMailProxyConfiguration',
			leaf: true
		    },
		    {
			text: gettext('Spam Detector'),
			iconCls: 'fa fa-bullhorn',
			path: 'pmgSpamDetectorConfiguration',
			leaf: true
		    },
		    {
			text: gettext('Virus Detector'),
			iconCls: 'fa fa-bug',
			path: 'pmgVirusDetectorConfiguration',
			leaf: true
		    },
		    {
			text: gettext('User Management'),
			iconCls: 'fa fa-users',
			path: 'pmgUserManagement',
			leaf: true
		    },
		    {
			text: gettext('Cluster'),
			iconCls: 'fa fa-server',
			path: 'pmgClusterAdministration',
			leaf: true
		    },
		    {
			text: gettext('Subscription'),
			iconCls: 'fa fa-support',
			path: 'pmgSubscription',
			leaf: true
		    }
		]
	    },
	    {
		text: gettext('Administration'),
		iconCls: 'fa fa-wrench',
		path: 'pmgServerAdministration',
		expanded: true,
		children: [
		    {
			text: gettext('Spam Quarantine'),
			iconCls: 'fa fa-bullhorn',
			path: 'pmgSpamQuarantine',
			leaf: true
		    },
		    {
			text: gettext('Virus Quarantine'),
			iconCls: 'fa fa-bug',
			path: 'pmgVirusQuarantine',
			leaf: true
		    },
		    {
			text: gettext('User Whitelist'),
			iconCls: 'fa fa-file-o',
			path: 'pmgUserWhitelist',
			leaf: true
		    },
		    {
			text: gettext('User Blacklist'),
			iconCls: 'fa fa-file',
			path: 'pmgUserBlacklist',
			leaf: true
		    },
		    {
			text: gettext('Tracking Center'),
			iconCls: 'fa fa-map-o',
			path: 'pmgMailTracker',
			leaf: true
		    },
		    {
			text: gettext('Queues'),
			iconCls: 'fa fa-bars',
			path: 'pmgQueueAdministration',
			leaf: true
		    }
		]
	    },
	    {
		text: gettext('Statistics'),
		iconCls: 'fa fa-bar-chart',
		path: 'pmgGeneralMailStatistics',
		expanded: true,
		children: [
		    {
			text: gettext('Spam Scores'),
			iconCls: 'fa fa-table',
			path: 'pmgSpamScoreDistribution',
			leaf: true
		    },
		    {
			text: gettext('Virus  Charts'),
			iconCls: 'fa fa-bug',
			path: 'pmgVirusCharts',
			leaf: true
		    },
		    {
			text: gettext('Hourly Distribution'),
			iconCls: 'fa fa-area-chart',
			path: 'pmgHourlyMailDistribution',
			leaf: true
		    },
		    {
			text: gettext('Postscreen'),
			iconCls: 'fa fa-line-chart',
			path: 'pmgRBLStatistics',
			leaf: true
		    },
		    {
			text: gettext('Domain'),
			iconCls: 'fa fa-table',
			path: 'pmgDomainStatistics',
			leaf: true
		    },
		    {
			text: gettext('Sender'),
			iconCls: 'fa fa-table',
			path: 'pmgSenderStatistics',
			leaf: true
		    },
		    {
			text: gettext('Receiver'),
			iconCls: 'fa fa-table',
			path: 'pmgReceiverStatistics',
			leaf: true
		    },
		    {
			text: gettext('Contact'),
			iconCls: 'fa fa-table',
			path: 'pmgContactStatistics',
			leaf: true
		    }
		]
	    }
	]
    }
});

Ext.define('PMG.view.main.NavigationTree', {
    extend: 'Ext.list.Tree',
    xtype: 'navigationtree',

    select: function(path) {
	var me = this;
	var item = me.getStore().findRecord('path', path, 0, false, true, true);
	me.setSelection(item);
    },

    animation: false,
    expanderOnly: true,
    expanderFirst: false,
    store: 'NavigationStore',
    ui: 'nav'
});
/*global Proxmox*/
/*jslint confusion: true*/
/* bind is a function and object,
 * callback is a function and string
 */
Ext.define('pmg-rule-list', {
    extend: 'Ext.data.Model',
    fields: [
	'id', 'name',
	{ name: 'active', type: 'boolean' },
	{ name: 'direction', type: 'integer' },
	{ name: 'priority', type: 'integer' }
    ],
    idProperty: 'id'
});

Ext.define('PMG.RulesConfiguration', {
    extend: 'Ext.container.Container',
    xtype: 'pmgRuleConfiguration',

    layout: 'border',
    border: false,
    defaults: {
	border: false
    },

    controller: {
	xclass: 'Ext.app.ViewController',

	selectedRuleChange: function(grid, selected, eOpts) {
	    var me = this;
	    var infoPanel = me.lookupReference('infopanel');
	    var baseurl = '';

	    if (selected.length > 0) {
		baseurl = '/config/ruledb/rules/' + selected[0].data.id;
	    }

	    infoPanel.getController().setBaseUrl(baseurl);
	},

	editIconClick: function(gridView, rowindex, colindex, column, e, record) {
	    var me = this;
	    me.showEditWindow(gridView, record);
	},

	showEditWindow: function(gridView, record) {
	    var me = this;
	    var win = Ext.create('PMG.RuleEditor', {
		url: '/api2/extjs/config/ruledb/rules/' + record.data.id + '/config',
		listeners: {
		    destroy: function() {
			gridView.getStore().load();
		    }
		}
	    });
	    win.load();
	    win.show();
	},

	toggleIconClick: function(gridView, rowindex, colindex, column, e, record) {
	    var me = this;
	    Proxmox.Utils.API2Request({
		url: '/config/ruledb/rules/' + record.data.id + '/config',
		params: {
		    active: record.data.active ? 0 : 1
		},
		method: 'PUT',
		callback: function() {
		    gridView.getStore().load();
		},
		failure: function(response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	reload: function(){
	    var me = this;
	    me.lookupReference('rulegrid').getStore().load();
	},

	addRule: function() {
	    var me = this;
	    var win = Ext.create('PMG.RuleEditor', {
		url: '/api2/extjs/config/ruledb/rules/',
		method: 'POST',
		isCreate: true,
		listeners: {
		    destroy: function() {
			me.lookupReference('rulegrid').getStore().load();
		    }
		}
	    });
	    win.load();
	    win.show();
	},

	onFactoryDefaults: function() {
	    var me = this;

	    Ext.Msg.confirm(
		gettext('Confirm'),
		gettext('Reset rule database to factory defaults?'),
		function(button) {
		    if (button !== 'yes') {
			return;
		    }
		    var url = '/config/ruledb';
		    Proxmox.Utils.API2Request({
			url: '/config/ruledb',
			method: 'POST',
			waitMsgTarget: me.getView(),
			callback: function() {
			    me.reload();
			},
			failure: function (response, opts) {
			    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
			}
		    });
		}
	    );
	},

	init: function(view) {
	    var grid = this.lookupReference('rulegrid');
	    Proxmox.Utils.monStoreErrors(grid, grid.getStore(), true);
	},

	control: {
	    'grid[reference=rulegrid]': {
		itemdblclick: 'showEditWindow',
		selectionchange: 'selectedRuleChange'
	    },
	    'button[reference=addButton]': {
		click: 'addRule'
	    }
	}
    },

    viewModel: {
	data: {
	    selectedRule: undefined,
	    baseUrl: '/config/ruledb/rules'
	}
    },

    items: [
	{
	    xtype: 'grid',
	    layout: 'fit',
	    title: 'Rules',
	    reference: 'rulegrid',
	    region: 'center',

	    bind: {
		selection: '{selectedRule}'
	    },

	    dockedItems:{
		xtype: 'toolbar',
		reference: 'mytb',
		items: [
		    {
			xtype: 'button',
			text: gettext('Add'),
			iconCls: 'fa fa-plus-circle',
			reference: 'addButton'
		    },
		    {
			xtype: 'proxmoxStdRemoveButton',
			text: gettext('Remove'),
			iconCls: 'fa fa-minus-circle',
			reference: 'removeButton',
			callback: 'reload',
			getRecordName: function(rec) { return rec.data.name; },
			bind: {
			    baseurl: '{baseUrl}'
			}
		    },
		    '->',
		    {
			text: gettext('Factory Defaults'),
			handler: 'onFactoryDefaults'
		    }
		]
	    },

	    viewConfig: {
		getRowClass: function(record, rowIndex) {
		    return record.get('active') ? 'enabled' : 'disabled';
		}
	    },

	    store: {
		autoLoad: true,
		model: 'pmg-rule-list',
		reference: 'rulesStore',
		proxy: {
		    type: 'proxmox',
		    url: '/api2/json/config/ruledb/rules'
		},
		sorters: [
		    {
			property: 'priority',
			direction: 'DESC'
		    },
		    {
			property: 'name'
		    }
		]
	    },

	    sortableColumns: false,
	    columns: [
		{
		    text: 'Active',
		    dataIndex: 'active',
		    hidden : true
		},
		{
		    text: 'Name',
		    dataIndex: 'name',
		    flex: 1
		},
		{
		    text: 'Priority',
		    dataIndex: 'priority'
		},
		{
		    text: 'Direction',
		    dataIndex: 'direction',
		    renderer: PMG.Utils.format_rule_direction
		},
		{
		    text: '',
		    xtype: 'actioncolumn',
		    align: 'center',
		    width: 70,
		    items: [
			{
			    iconCls: 'fa fa-fw fa-pencil',
			    tooltip: 'Edit',
			    handler: 'editIconClick'
			},
			{
			    getClass: function(val, meta, rec) {
				return 'fa fa-fw fa-' + (rec.get('active') ? 'toggle-on' : 'toggle-off');
			    },
			    getTip: function(val, meta, rec) {
				return (rec.get('active') ? 'Deactivate' : 'Activate');
			    },
			    handler: 'toggleIconClick'
			}
		    ]
		}
	    ]
	},
	{
	    region: 'east',
	    reference: 'infopanel',
	    xtype: 'pmgRuleInfo',
	    split: true,
	    width: 440
	}
    ]
});
/*global Proxmox*/
Ext.define('PMG.SystemOptions', {
    extend: 'Proxmox.grid.ObjectGrid',
    xtype: 'pmgSystemOptions',

    monStoreErrors: true,
    interval: 5000,
    cwidth1: 200,

    url: '/api2/json/config/admin',
    editorConfig: {
	url: '/api2/extjs/config/admin'
    },

    controller: {
	xclass: 'Ext.app.ViewController',

	onEdit: function() {
	    var me = this.getView();
	    me.run_editor();
	}
    },

    tbar: [{
	text: gettext('Edit'),
	xtype: 'proxmoxButton',
	disabled: true,
	handler: 'onEdit'
    }],

    listeners: {
	itemdblclick: 'onEdit'
    },

    add_proxy_row: function(name, text, opts) {
	var me = this;

	opts = opts || {};
	me.rows = me.rows || {};

	me.rows[name] = {
	    required: true,
	    defaultValue: Proxmox.Utils.noneText,
	    header: text,
	    editor: {
		xtype: 'proxmoxWindowEdit',
		subject: text,
		items: {
		    xtype: 'proxmoxtextfield',
		    vtype: 'HttpProxy',
		    name: name,
		    deleteEmpty: true,
		    emptyText: Proxmox.Utils.noneText,
		    labelWidth: Proxmox.Utils.compute_min_label_width(
			text, opts.labelWidth),
		    fieldLabel: text
		}
	    }
	};
    },

    initComponent : function() {
	var me = this;

	me.add_boolean_row('dailyreport', gettext('Send daily reports'),
			   { defaultValue: 1});

	me.add_boolean_row('advfilter', gettext('Use advanced statistic filters'),
			   { defaultValue: 1});

	me.add_integer_row('statlifetime', gettext('User statistic lifetime (days)'),
			   { minValue: 1, defaultValue: 7, deleteEmpty: true });

	me.add_text_row('email', gettext("Administrator EMail"),
			{ deleteEmpty: true, defaultValue: Proxmox.Utils.noneText });

	me.add_proxy_row('http_proxy', gettext("HTTP proxy"));

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
	me.on('destroy', me.rstore.stopUpdate);
    }
});
/*global Blob,Proxmox*/
Ext.define('PMG.SubscriptionKeyEdit', {
    extend: 'Proxmox.window.Edit',
    
    title: gettext('Upload Subscription Key'),
    width: 300,
    autoLoad: true,

    onlineHelp: 'getting_help',

    items: {
	xtype: 'textfield',
	name: 'key',
	value: '',
	fieldLabel: gettext('Subscription Key')
    }
});

Ext.define('PMG.Subscription', {
    extend: 'Proxmox.grid.ObjectGrid',
    xtype: 'pmgSubscription',

    title: gettext('Subscription'),

    border: false,

    onlineHelp: 'getting_help',

    viewConfig: {
	enableTextSelection: true
    },

    initComponent : function() {
	var me = this;

	var reload = function() {
	    me.rstore.load();
	};

	var baseurl = '/nodes/' + Proxmox.NodeName + '/subscription';

	var render_status = function(value) {

	    var message = me.getObjectValue('message');

	    if (message) {
		return value + ": " + message;
	    }
	    return value;
	};

	var rows = {
	    productname: {
		header: gettext('Type')
	    },
	    key: {
		header: gettext('Subscription Key')
	    },
	    status: {
		header: gettext('Status'),
		renderer: render_status
	    },
	    message: {
		visible: false
	    },
	    serverid: {
		header: gettext('Server ID')
	    },
	    sockets: {
		header: gettext('Sockets')
	    },
	    checktime: {
		header: gettext('Last checked'),
		renderer: Proxmox.Utils.render_timestamp
	    },
	    nextduedate: {
		header: gettext('Next due date')
	    }
	};

	Ext.apply(me, {
	    url: '/api2/json' + baseurl,
	    cwidth1: 170,
	    tbar: [ 
		{
		    text: gettext('Upload Subscription Key'),
		    handler: function() {
			var win = Ext.create('PMG.SubscriptionKeyEdit', {
			    url: '/api2/extjs/' + baseurl 
			});
			win.show();
			win.on('destroy', reload);
		    }
		},
		{
		    text: gettext('Check'),
		    handler: function() {
			Proxmox.Utils.API2Request({
			    params: { force: 1 },
			    url: baseurl,
			    method: 'POST',
			    waitMsgTarget: me,
			    failure: function(response, opts) {
				Ext.Msg.alert(gettext('Error'), response.htmlStatus);
			    },
			    callback: reload
			});
		    }
		}
	    ],
	    rows: rows
	});

	me.callParent();

	reload();
    }
});
/*global Proxmox*/
Ext.define('pmg-backup-list', {
    extend: 'Ext.data.Model',
    fields: [
	'filename',
	{ type: 'integer', name: 'size' },
	{ type: 'date', dateFormat: 'timestamp', name: 'timestamp' }

    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/nodes/" + Proxmox.NodeName + "/backup"
    },
    idProperty: 'filename'
});

Ext.define('PMG.RestoreWindow', {
    extend: 'Proxmox.window.Edit',
    xtype: 'pmgRestoreWindow',

    showProgress: true,
    title: gettext('Restore'),
    isCreate: true,
    method: 'POST',
    submitText: gettext('Restore'),
    fieldDefaults: {
	labelWidth: 150
    },
    items: [
	{
	    xtype: 'proxmoxcheckbox',
	    name: 'config',
	    fieldLabel: gettext('System Configuration')
	},
	{
	    xtype: 'proxmoxcheckbox',
	    name: 'database',
	    value: 1,
	    uncheckedValue: 0,
	    fieldLabel: gettext('Rule Database'),
	    listeners: {
		change: function(cb, value) {
		    var me = this;
		    me.up().down('field[name=statistic]').setDisabled(!value);
		}
	    }
	},
	{
	    xtype: 'proxmoxcheckbox',
	    name: 'statistic',
	    fieldLabel: gettext('Statistic')
	}
    ],

    initComponent: function() {
	var me = this;

	if (!me.filename) {
	    throw "no filename given";
	}

	me.url = "/nodes/" + Proxmox.NodeName + "/backup/" + encodeURIComponent(me.filename);

	me.callParent();
    }
});

Ext.define('PMG.BackupRestore', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgBackupRestore',

    title: gettext('Backup') + '/' + gettext('Restore'),

    controller: {
	xclass: 'Ext.app.ViewController',

	createBackup: function() {
	    var me = this.getView();
	    Proxmox.Utils.API2Request({
		url: "/nodes/" + Proxmox.NodeName + "/backup",
		method: 'POST',
		waitMsgTarget: me,
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
		success: function(response, opts) {
		    var upid = response.result.data;

		    var win = Ext.create('Proxmox.window.TaskViewer', {
			upid: upid
		    });
		    win.show();
		    me.mon(win, 'close', function() { me.store.load(); });
		}
	    });
	},

	onRestore: function() {
	    var me = this.getView();
	    var rec = me.getSelection()[0];

	    if (!(rec && rec.data && rec.data.filename)) {
		return;
	    }

	    Ext.create('PMG.RestoreWindow', {
		filename: rec.data.filename
	    }).show();
	},

	onAfterRemove: function(btn, res) {
	    var me = this.getView();
	    me.store.load();
	}
    },

    tbar: [
	{
	    text: gettext('Backup'),
	    handler: 'createBackup'
	},
	'-',
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Restore'),
	    handler: 'onRestore',
	    disabled: true
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    baseurl: '/nodes/' + Proxmox.NodeName + '/backup',
	    reference: 'removeBtn',
	    callback: 'onAfterRemove',
	    waitMsgTarget: true
	}
    ],

    store: {
	autoLoad: true,
	model: 'pmg-backup-list',
	sorters: [
	    {
		property: 'timestamp',
		direction: 'DESC'
	    }
	]
    },

    columns: [
	{
	    header: gettext('Filename'),
	    width: 300,
	    sortable: true,
	    renderer: Ext.htmlEncode,
	    dataIndex: 'filename'
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Time'),
	    width: 150,
	    format: 'Y-m-d H:i',
	    sortable: true,
	    dataIndex: 'timestamp'
	},
	{
	    header: gettext('Size'),
	    width: 100,
	    sortable: true,
	    renderer: Proxmox.Utils.format_size,
	    dataIndex: 'size'
	},
	{
	    header: gettext('Download'),
	    renderer: function(filename) {
		return "<a class='download' href='" +
		    "/api2/json/nodes/" + Proxmox.NodeName + "/backup/" + encodeURIComponent(filename) +
		"'><i class='fa fa-fw fa-download'</i></a>";
	    },
	    dataIndex: 'filename'
	}
    ]
});
/*global Proxmox*/

Ext.define('PMG.SystemConfiguration', {
    extend: 'Ext.tab.Panel',
    xtype: 'pmgSystemConfiguration',

    title: gettext('Configuration') + ': ' + gettext('System'),
    border: false,
    scrollable: true,
    defaults: { border: false },
    items: [
	{
	    title: gettext('Network/Time'),
	    itemId: 'network',
	    xtype: 'panel',
	    layout: {
		type: 'vbox',
		align: 'stretch',
		multi: true
	    },
	    bodyPadding: '0 0 10 0',
	    defaults: {
		collapsible: true,
		animCollapse: false,
		margin: '10 10 0 10'
	    },
	    items: [
		{
		    flex: 1,
		    minHeight: 200,
		    title: gettext('Interfaces'),
		    xtype: 'proxmoxNodeNetworkView',
		    types: ['bond'],
		    nodename: Proxmox.NodeName
		},
		{
		    title: gettext('DNS'),
		    xtype: 'proxmoxNodeDNSView',
		    nodename: Proxmox.NodeName
		},
		{
		    title: gettext('Time'),
		    xtype: 'proxmoxNodeTimeView',
		    nodename: Proxmox.NodeName
		}
	    ]
	},
	{
	    itemId: 'options',
            title: gettext('Options'),
	    xtype: 'pmgSystemOptions'
	},
	{
	    itemId: 'backup',
	    xtype: 'pmgBackupRestore'
	}
    ],

    initComponent: function() {
	var me = this;

	me.callParent();

	var networktime = me.getComponent('network');
	Ext.Array.forEach(networktime.query(), function(item) {
	    item.relayEvents(networktime, [ 'activate', 'deactivate', 'destroy']);
	});
    }
});


/*global Proxmox*/
Ext.define('PMG.MailProxyRelaying', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgMailProxyRelaying'],

    monStoreErrors: true,
    
    initComponent : function() {
	var me = this;

	me.add_text_row('relay', gettext('Default Relay'),
			{ deleteEmpty: true, defaultValue: Proxmox.Utils.noneText });

	me.add_integer_row('relayport', gettext('SMTP Port'),
			   { defaultValue: 25, deleteEmpty: true,
			     minValue: 1, maxValue: 65535 });

	me.add_boolean_row('relaynomx', gettext('Disable MX lookup'));

	me.add_text_row('smarthost', gettext('Smarthost'),
			{ deleteEmpty: true, defaultValue: Proxmox.Utils.noneText });

	var baseurl = '/config/mail';

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	Ext.apply(me, {
	    tbar: [{
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { me.run_editor(); },
		selModel: me.selModel
	    }],
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
Ext.define('PMG.MailProxyPorts', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgMailProxyPorts'],

    url: '/api2/json/config/mail',

    monStoreErrors: true,

    editorConfig: {
	url: '/api2/extjs/config/mail'
    },

    interval: 5000,

    cwidth1: 200,

    controller: {

	xclass: 'Ext.app.ViewController',

	onEdit: function() {
	    this.getView().run_editor();
	}
    },

    listeners: {
	itemdblclick: 'onEdit'
    },

    tbar: [
	{
	    text: gettext('Edit'),
	    xtype: 'proxmoxButton',
	    disabled: true,
	    handler: 'onEdit'
	}
    ],

    initComponent : function() {
	var me = this;

	me.add_integer_row('ext_port', gettext('External SMTP Port'),
			   { defaultValue: 26, deleteEmpty: true,
			     minValue: 1, maxValue: 65535 });

	me.add_integer_row('int_port', gettext('Internal SMTP Port'),
			   { defaultValue: 25, deleteEmpty: true,
			     minValue: 1, maxValue: 65535 });

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
/*global Proxmox*/
Ext.define('PMG.MailProxyOptions', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgMailProxyOptions'],

    monStoreErrors: true,

    initComponent : function() {
	var me = this;

	me.add_integer_row('maxsize', gettext('Message Size (bytes)'),
			   { defaultValue: 1024*1024*10,
			     minValue: 1024, deleteEmpty: true });

	me.add_boolean_row('rejectunknown', gettext('Reject Unknown Clients'));

	me.add_boolean_row('rejectunknownsender', gettext('Reject Unknown Senders'));

	me.add_boolean_row('helotests', gettext('SMTP HELO checks'));

	me.add_text_row('dnsbl_sites', gettext('DNSBL Sites'),
			{ deleteEmpty: true, defaultValue: Proxmox.Utils.noneText });

	var render_verifyreceivers = function(value) {
	    if (value === undefined || value === '__default__') {
		return Proxmox.Utils.noText;
	    }
	    return Proxmox.Utils.yesText + ' (' + value + ')';
	};

	/*jslint confusion: true*/
	/*defaultValue is string and number*/
	me.add_combobox_row('verifyreceivers', gettext('Verify Receivers'), {
	    renderer: render_verifyreceivers,
	    defaultValue: '__default__',
	    deleteEmpty: true,
	    comboItems: [
		['__default__', render_verifyreceivers('__default__') ],
		['450', render_verifyreceivers('450') ],
		['550', render_verifyreceivers('550') ]]
	});
	/*jslint confusion: false*/

	me.add_boolean_row('greylist', gettext('Use Greylisting'),
			   { defaultValue: 1 });

	me.add_boolean_row('spf', gettext('Use SPF'), { defaultValue: 1 });

	me.add_boolean_row('hide_received', gettext('Hide Internal Hosts'));

	me.add_integer_row('dwarning', gettext('Delay Warning Time (hours)'),
			   { defaultValue: 4, minValue: 0 });

	me.add_integer_row('conn_count_limit', gettext('Client Connection Count Limit'),
			   { defaultValue: 50, minValue: 0, maxValue: 65535 });

	me.add_integer_row('conn_rate_limit', gettext('Client Connection Rate Limit'),
			   { defaultValue: 0, minValue: 0 });

	me.add_integer_row('message_rate_limit', gettext('Client Message Rate Limit'),
			   { defaultValue: 0, minValue: 0 });

	/*jslint confusion: true*/
	/*defaultValue is string and number*/
	me.add_text_row('banner', gettext('SMTPD Banner'),
			{ deleteEmpty: true, defaultValue: 'ESMTP Proxmox' });
	/*jslint confusion: false*/

	var baseurl = '/config/mail';

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	Ext.apply(me, {
	    tbar: [{
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { me.run_editor(); },
		selModel: me.selModel
	    }],
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
Ext.define('PMG.MailProxyTLS', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgMailProxyTLS'],

    monStoreErrors: true,

    initComponent : function() {
	var me = this;

	me.add_boolean_row('tls', gettext('Enable TLS'));

	me.add_boolean_row('tlslog', gettext('Enable TLS Logging'));

	me.add_boolean_row('tlsheader', gettext('Add TLS received header'));

	var baseurl = '/config/mail';

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	Ext.apply(me, {
	    tbar: [{
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { me.run_editor(); },
		selModel: me.selModel
	    }],
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
/*global Proxmox*/
Ext.define('pmg-transport', {
    extend: 'Ext.data.Model',
    fields: [ 'domain', 'host', { name: 'port', type: 'integer' },
	      { name: 'use_mx', type: 'boolean' }, 'comment' ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/config/transport"
    },
    idProperty: 'domain'
});

Ext.define('PMG.Transport', {
    extend: 'Ext.grid.GridPanel',
    alias: ['widget.pmgTransport'],

    initComponent : function() {
	var me = this;

	var store = new Ext.data.Store({
	    model: 'pmg-transport',
	    sorters: {
		property: 'domain',
		order: 'DESC'
	    }
	});

        var reload = function() {
            store.load();
        };

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	var remove_btn =  Ext.createWidget('proxmoxStdRemoveButton', {
	    selModel: me.selModel,
	    baseurl: '/config/transport',
	    callback: reload,
	    waitMsgTarget: me
	});

	var common_properties = [
	    {
		xtype: 'textfield',
		name: 'host',
		fieldLabel: gettext("Host")
	    },
	    {
		xtype: 'proxmoxintegerfield',
		name: 'port',
		value: 25,
		minValue: 1,
		maxValue: 65535,
		fieldLabel: gettext("Port")
	    },
	    {
		xtype: 'proxmoxcheckbox',
		name: 'use_mx',
		checked: true,
		uncheckedValue: 0,
		fieldLabel: gettext("Use MX")
	    },
	    {
		xtype: 'textfield',
		name: 'comment',
		fieldLabel: gettext("Comment")
	    }
	];

	var edit_properties = common_properties.slice();
	edit_properties.unshift({
	    xtype: 'displayfield',
	    name: 'domain',
	    fieldLabel: gettext("Relay Domain")
	});

	var create_properties = common_properties.slice();
	create_properties.unshift({
	    xtype: 'proxmoxtextfield',
	    name: 'domain',
	    fieldLabel: gettext("Relay Domain")
	});

	var run_editor = function() {
	    var rec = me.selModel.getSelection()[0];
	    if (!rec) {
		return;
	    }

	    var config = {
		url: "/api2/extjs/config/transport/" + rec.data.domain,
		method: 'PUT',
		subject: gettext("Transport"),
		items: edit_properties
	    };

	    var win = Ext.createWidget('proxmoxWindowEdit', config);

	    win.load();
	    win.on('destroy', reload);
	    win.show();
	};

	var tbar = [
            {
		xtype: 'proxmoxButton',
		text: gettext('Edit'),
		disabled: true,
		selModel: me.selModel,
		handler: run_editor
            },
            {
		text: gettext('Create'),
		handler: function() {
		    var config = {
			method: 'POST',
			url: "/api2/extjs/config/transport",
			isCreate: true,
			subject: gettext("Transport"),
			items: create_properties
		    };

		    var win = Ext.createWidget('proxmoxWindowEdit', config);

		    win.on('destroy', reload);
		    win.show();
		}
            },
	    remove_btn
        ];

	Proxmox.Utils.monStoreErrors(me, store, true);

	Ext.apply(me, {
	    store: store,
	    tbar: tbar,
	    viewConfig: {
		trackOver: false
	    },
	    columns: [
		{
		    header: gettext('Relay Domain'),
		    width: 200,
		    sortable: true,
		    dataIndex: 'domain'
		},
		{
		    header: gettext('Host'),
		    width: 200,
		    sortable: true,
		    dataIndex: 'host'
		},
		{
		    header: gettext('Port'),
		    width: 80,
		    sortable: false,
		    dataIndex: 'port'
		},
		{
		    header: gettext('Use MX'),
		    width: 80,
		    renderer: Proxmox.Utils.format_boolean,
		    sortable: false,
		    dataIndex: 'use_mx'
		},
		{
		    header: gettext('Comment'),
		    sortable: false,
		    renderer: Ext.String.htmlEncode,
		    dataIndex: 'comment',
		    flex: 1
		}
	    ],
	    listeners: {
		itemdblclick: run_editor,
		activate: reload
	    }
	});

	me.callParent();
    }
});
/*global Proxmox*/
Ext.define('pmg-mynetworks', {
    extend: 'Ext.data.Model',
    fields: [ 'cidr', 'comment' ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/config/mynetworks"
    },
    idProperty: 'cidr'
});

Ext.define('PMG.MyNetworks', {
    extend: 'Ext.grid.GridPanel',
    alias: ['widget.pmgMyNetworks'],

    initComponent : function() {
	var me = this;

	var store = new Ext.data.Store({
	    model: 'pmg-mynetworks',
	    sorters: {
		property: 'cidr',
		order: 'DESC'
	    }
	});

        var reload = function() {
            store.load();
        };

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	var remove_btn =  Ext.createWidget('proxmoxStdRemoveButton', {
	    selModel: me.selModel,
	    baseurl: '/config/mynetworks',
	    callback: reload,
	    waitMsgTarget: me
	});

	var run_editor = function() {
	    var rec = me.selModel.getSelection()[0];
	    if (!rec) {
		return;
	    }

	    var config = {
		url: "/api2/extjs/config/mynetworks/" + rec.data.cidr,
		method: 'PUT',
		subject: gettext("Trusted Network"),
		items: [
		    {
			xtype: 'displayfield',
			name: 'cidr',
			fieldLabel: 'CIDR'
		    },
		    {
			xtype: 'textfield',
			name: 'comment',
			fieldLabel: gettext("Comment")
		    }
		]
	    };

	    var win = Ext.createWidget('proxmoxWindowEdit', config);

	    win.load();
	    win.on('destroy', reload);
	    win.show();
	};

	var tbar = [
            {
		xtype: 'proxmoxButton',
		text: gettext('Edit'),
		disabled: true,
		selModel: me.selModel,
		handler: run_editor
            },
            {
		text: gettext('Create'),
		handler: function() {
		    /*jslint confusion: true*/
		    var config = {
			method: 'POST',
			url: "/api2/extjs/config/mynetworks",
			isCreate: true,
			subject: gettext("Trusted Network"),
			items: [
			    {
				xtype: 'proxmoxtextfield',
				name: 'cidr',
				fieldLabel: 'CIDR'
			    },
			    {
				xtype: 'proxmoxtextfield',
				name: 'comment',
				fieldLabel: gettext("Comment")
			    }
			]
		    };
		    /*jslint confusion: false*/

		    var win = Ext.createWidget('proxmoxWindowEdit', config);

		    win.on('destroy', reload);
		    win.show();
		}
            },
	    remove_btn
        ];

	Proxmox.Utils.monStoreErrors(me, store, true);

	Ext.apply(me, {
	    store: store,
	    tbar: tbar,
	    viewConfig: {
		trackOver: false
	    },
	    columns: [
		{
		    header: gettext('Trusted Network'),
		    width: 200,
		    sortable: true,
		    dataIndex: 'cidr'
		},
		{
		    header: gettext('Comment'),
		    sortable: false,
		    renderer: Ext.String.htmlEncode,
		    dataIndex: 'comment',
		    flex: 1
		}
	    ],
	    listeners: {
		itemdblclick: run_editor,
		activate: reload
	    }
	});

	me.callParent();
    }
});
/*global Proxmox*/
Ext.define('pmg-domains', {
    extend: 'Ext.data.Model',
    fields: [ 'domain', 'comment' ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/config/domains"
    },
    idProperty: 'domain'
});

Ext.define('PMG.RelayDomains', {
    extend: 'Ext.grid.GridPanel',
    alias: ['widget.pmgRelayDomains'],

    initComponent : function() {
	var me = this;

	var store = new Ext.data.Store({
	    model: 'pmg-domains',
	    sorters: {
		property: 'domain',
		order: 'DESC'
	    }
	});

        var reload = function() {
            store.load();
        };

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	var remove_btn = Ext.createWidget('proxmoxStdRemoveButton', {
	    selModel: me.selModel,
	    baseurl: '/config/domains',
	    callback: reload,
	    waitMsgTarget: me
	});

	var run_editor = function() {
	    var rec = me.selModel.getSelection()[0];
	    if (!rec) {
		return;
	    }

	    var config = {
		url: "/api2/extjs/config/domains/" + rec.data.domain,
		method: 'PUT',
		subject: gettext("Relay Domain"),
		items: [
		    {
			xtype: 'displayfield',
			name: 'domain',
			fieldLabel: gettext("Relay Domain")
		    },
		    {
			xtype: 'textfield',
			name: 'comment',
			fieldLabel: gettext("Comment")
		    }
		]
	    };

	    var win = Ext.createWidget('proxmoxWindowEdit', config);

	    win.load();
	    win.on('destroy', reload);
	    win.show();
	};

	var tbar = [
            {
		xtype: 'proxmoxButton',
		text: gettext('Edit'),
		disabled: true,
		selModel: me.selModel,
		handler: run_editor
            },
            {
		text: gettext('Create'),
		handler: function() {
		    /*jslint confusion: true*/
		    var config = {
			method: 'POST',
			url: "/api2/extjs/config/domains",
			isCreate: true,
			subject: gettext("Relay Domain"),
			items: [
			    {
				xtype: 'proxmoxtextfield',
				name: 'domain',
				fieldLabel: gettext("Relay Domain")
			    },
			    {
				xtype: 'proxmoxtextfield',
				name: 'comment',
				fieldLabel: gettext("Comment")
			    }
			]
		    };
		    /*jslint confusion: false*/

		    var win = Ext.createWidget('proxmoxWindowEdit', config);

		    win.on('destroy', reload);
		    win.show();
		}
            },
	    remove_btn
        ];

	Proxmox.Utils.monStoreErrors(me, store, true);

	Ext.apply(me, {
	    store: store,
	    tbar: tbar,
	    viewConfig: {
		trackOver: false
	    },
	    columns: [
		{
		    header: gettext('Relay Domain'),
		    width: 200,
		    sortable: true,
		    dataIndex: 'domain'
		},
		{
		    header: gettext('Comment'),
		    sortable: false,
		    renderer: Ext.String.htmlEncode,
		    dataIndex: 'comment',
		    flex: 1
		}
	    ],
	    listeners: {
		itemdblclick: run_editor,
		activate: reload
	    }
	});

	me.callParent();
    }
});
Ext.define('PMG.MailProxyConfiguration', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pmgMailProxyConfiguration',

    title: gettext('Configuration') + ': ' +
	gettext('Mail Proxy'),

    border: false,
    defaults: { border: false },

    items: [
	{
	    itemId: 'relaying',
	    title: gettext('Relaying'),
	    xtype: 'pmgMailProxyRelaying'
	},
	{
	    itemId: 'relaydomains',
	    title: gettext('Relay Domains'),
	    xtype: 'pmgRelayDomains'
	},
	{
	    itemId: 'ports',
	    title: gettext('Ports'),
	    xtype: 'pmgMailProxyPorts'
	},
	{
	    itemId: 'options',
            title: gettext('Options'),
	    xtype: 'pmgMailProxyOptions'
	},
	{
	    itemId: 'transports',
            title: gettext('Transports'),
	    xtype: 'pmgTransport'
	},
	{
	    itemId: 'networks',
            title: gettext('Networks'),
	    xtype: 'pmgMyNetworks'
	},
	{
	    itemId: 'tls',
            title: gettext('TLS'),
	    xtype: 'pmgMailProxyTLS'
	},
	{
	    itemId: 'whitelist',
	    title: gettext('Whitelist'),
	    xtype: 'pmgObjectGroup',
	    hideGroupInfo: true,
	    showDirection: true,
	    otype_list: [1000, 1009, 1001, 1007, 1002, 1008, 1003, 1004],
	    baseurl: '/config/whitelist'
	}
   ]
});


Ext.define('PMG.SpamDetectorLanguagesInputPanel', {
    extend: 'Proxmox.panel.InputPanel',
    alias: 'widget.pmgSpamDetectorLanguagesInputPanel',

    languages: [
	[ 'af', 'Afrikaans' ],
	[ 'am', 'Amharic' ],
	[ 'ar', 'Arabic' ],
	[ 'be', 'Byelorussian' ],
	[ 'bg', 'Bulgarian' ],
	[ 'bs', 'Bosnian' ],
	[ 'ca', 'Catalan' ],
	[ 'cs', 'Czech' ],
	[ 'cy', 'Welsh' ],
	[ 'da', 'Danish' ],
	[ 'de', 'German' ],
	[ 'el', 'Greek' ],
	[ 'en', 'English' ],
	[ 'eo', 'Esperanto' ],
	[ 'es', 'Spanish' ],
	[ 'et', 'Estonian' ],
	[ 'eu', 'Basque' ],
	[ 'fa', 'Persian' ],
	[ 'fi', 'Finnish' ],
	[ 'fr', 'French' ],
	[ 'fy', 'Frisian' ],
	[ 'ga', 'Irish' ],
	[ 'gd', 'Scottish' ],
	[ 'he', 'Hebrew' ],
	[ 'hi', 'Hindi' ],
	[ 'hr', 'Croatian' ],
	[ 'hu', 'Hungarian' ],
	[ 'hy', 'Armenian' ],
	[ 'id', 'Indonesian' ],
	[ 'is', 'Icelandic' ],
	[ 'it', 'Italian' ],
	[ 'ja', 'Japanese' ],
	[ 'ka', 'Georgian' ],
	[ 'ko', 'Korean' ],
	[ 'la', 'Latin' ],
	[ 'lt', 'Lithuanian' ],
	[ 'lv', 'Latvian' ],
	[ 'mr', 'Marathi' ],
	[ 'ms', 'Malay' ],
	[ 'ne', 'Nepali' ],
	[ 'nl', 'Dutch' ],
	[ 'no', 'Norwegian' ],
	[ 'pl', 'Polish' ],
	[ 'pt', 'Portuguese' ],
	[ 'qu', 'Quechua' ],
	[ 'Rhaeto', 'Romance' ],
	[ 'ro', 'Romanian' ],
	[ 'ru', 'Russian' ],
	[ 'sa', 'Sanskrit' ],
	[ 'sco', 'Scots' ],
	[ 'sk', 'Slovak' ],
	[ 'sl', 'Slovenian' ],
	[ 'sq', 'Albanian' ],
	[ 'sr', 'Serbian' ],
	[ 'sv', 'Swedish' ],
	[ 'sw', 'Swahili' ],
	[ 'ta', 'Tamil' ],
	[ 'th', 'Thai' ],
	[ 'tl', 'Tagalog' ],
	[ 'tr', 'Turkish' ],
	[ 'uk', 'Ukrainian' ],
	[ 'vi', 'Vietnamese' ],
	[ 'yi', 'Yiddish' ],
	[ 'zh', 'Chinese' ]
    ],

    onGetValues: function(values) {
	if (!values.languages) {
	    values['delete'] = 'languages';
	} else if (Ext.isArray(values.languages)) {
	    /*jslint confusion: true*/
	    /*languages is an array and string here*/
	    values.languages = values.languages.join(' ');
	    /*jslint confusion: false*/
	}

	return values;
    },


    initComponent : function() {
	var me = this;

	me.column1 = [];
	me.column2 = [];
	me.column3 = [];
	me.column4 = [];

	var i, len;
	for (i = 0, len = me.languages.length; i < len; i++) {
	    var config = {
		xtype: 'checkboxfield',
		inputValue: me.languages[i][0],
		boxLabel: me.languages[i][1],
		name: 'languages'
	    };
	    if ((i % 4) === 0) {
		me.column1.push(config);
	    } else if ((i % 4) === 1) {
		me.column2.push(config);
	    } else if ((i % 4) === 2) {
		me.column3.push(config);
	    } else if ((i % 4) === 3) {
		me.column4.push(config);
	    }
	}

	me.callParent();
    }
});

Ext.define('PMG.SpamDetectorLanguages', {
    extend: 'Proxmox.window.Edit',

    subject: 'Languages',

    initComponent : function() {
	var me = this;

	me.items = Ext.create('PMG.SpamDetectorLanguagesInputPanel');

	me.callParent();

	me.load({
	    success: function(response, options) {
		var value = response.result.data.languages || '';
		var languages = value.split(/[\ \,\;]+/);
		me.setValues({ languages: languages });
	    }
	});
    }
});
Ext.define('PMG.SpamDetectorOptions', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgSpamDetectorOptions'],

    monStoreErrors: true,

    initComponent : function() {
	var me = this;

	me.add_boolean_row('use_awl', gettext('Use auto-whitelists'),
			   { defaultValue: 1 });

	me.add_boolean_row('use_bayes', gettext('Use Bayesian filter'),
			   { defaultValue: 1 });
	
	me.add_boolean_row('rbl_checks', gettext('Use RBL checks'),
			   { defaultValue: 1 });

	me.add_boolean_row('use_razor', gettext('Use Razor2 checks'),
			   { defaultValue: 1 });

	me.add_integer_row('maxspamsize', gettext('Max Spam Size (bytes)'),
			   { defaultValue: 512*1024,
			     minValue: 64, deleteEmpty: true });

	me.rows.languages = {
	    required: true,
	    header: gettext('Languages'),
	    editor: 'PMG.SpamDetectorLanguages',
	    renderer: function(value) {
		return value || 'all';
	    }
	};

	me.add_integer_row('bounce_score', gettext('Backscatter Score'),
			   { defaultValue: 0,
			     minValue: 0, maxValue: 1000,
			     deleteEmpty: true });

	me.add_integer_row('clamav_heuristic_score', gettext('Heuristic Score'),
			   { defaultValue: 3,
			     minValue: 0, maxValue: 1000,
			     deleteEmpty: true });

	var baseurl = '/config/spam';

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	Ext.apply(me, {
	    tbar: [{
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { me.run_editor(); },
		selModel: me.selModel
	    }],
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
/*global Proxmox*/
Ext.define('PMG.SpamQuarantineOptions', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgSpamQuarantineOptions'],

    monStoreErrors: true,

    authmodeTextHash: {
	ticket: 'Ticket',
	ldap: 'LDAP',
	ldapticket: 'LDAP or Ticket'
    },

    reportstyleTextHash: {
	none: gettext('No Reports'),
	'short': gettext('Short'),
	verbose: gettext('Verbose'),
	outlook: gettext('Verbose') + ' (' + gettext('Outlook 2007') + ')',
	custom: gettext('Custom')
    },
    
    initComponent : function() {
	var me = this;

	me.add_integer_row('lifetime', gettext('Lifetime (days)'),
			   { minValue: 1, defaultValue: 7,
			     deleteEmpty: true });

	var render_authmode = function(value) {
	    return me.authmodeTextHash[value] || value;
	};

	/*jslint confusion: true*/
	/* defaultValue is a string and a number*/
	me.add_combobox_row('authmode', gettext('Authentication mode'), {
	    defaultValue: 'ticket',
	    renderer: render_authmode,
	    comboItems: [
		['ticket', render_authmode('ticket') ],
		['ldap', render_authmode('ldap') ],
		['ldapticket', render_authmode('ldapticket') ]]
	});

	var render_reportstyle = function(value) {
	    return me.reportstyleTextHash[value] || value;
	};

	me.add_combobox_row('reportstyle', gettext('Report Style'), {
	    defaultValue: 'verbose',
	    renderer: render_reportstyle,
	    comboItems: [
		['none', render_reportstyle('none') ],
		['short', render_reportstyle('short') ],
		['verbose', render_reportstyle('verbose') ],
		['outlook', render_reportstyle('outlook') ],
		['custom', render_reportstyle('custom') ]]
	});
	/*jslint confusion: false*/

	me.add_text_row('hostname', gettext('Quarantine Host'),
			{ deleteEmpty: true, defaultValue: Proxmox.Utils.noneText });
	me.add_text_row('mailfrom', gettext("EMail 'From:'"),
			{ deleteEmpty: true, defaultValue: Proxmox.Utils.noneText });

	me.add_boolean_row('viewimages', gettext('View images'),
			   { defaultValue: 1});
	
	me.add_boolean_row('allowhrefs', gettext('Allow HREFs'),
			   {defaultValue: 1 });

	var baseurl = '/config/spamquar';

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	Ext.apply(me, {
	    tbar: [{
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { me.run_editor(); },
		selModel: me.selModel
	    }],
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
/*global Proxmox*/
Ext.define('pmg-spamassassin-database', {
    extend: 'Ext.data.Model',
    fields: [
	'channel', 'version', 'update_version',
	{ name: 'update_avail', type: 'boolean' },
	{ name: 'last_updated', type: 'date', dateFormat: 'timestamp' }
    ],
    idProperty: 'channel'
});

Ext.define('PMG.SpamDetectorStatusGrid', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgSpamDetectorStatus',

    title: gettext('Status'),

    viewConfig: {
	trackOver: false
    },
    columns: [
	{
	    header: gettext('Channel'),
	    sortable: true,
	    flex: 1,
	    dataIndex: 'channel'
	},
	{
	    header: gettext('Last Update'),
	    sortable: true,
	    flex: 2,
	    dataIndex: 'last_updated'
	},
	{
	    header: gettext('Version'),
	    flex: 1,
	    sortable: true,
	    dataIndex: 'version'
	},
	{
	    header: gettext('Update Available'),
	    flex: 1,
	    sortable: true,
	    dataIndex: 'update_avail',
	    renderer: function(value, metaData, record) {
		if (!value) {
		    return Proxmox.Utils.noText;
		} else {
		    return Proxmox.Utils.yesText + ' (' + record.data.update_version + ')';
		}
	    }
	}
    ],

    listeners: {
	activate: function() {
	    var me = this;
	    me.store.load();
	}
    },

    tbar: [
	{
	    text: gettext('Update Now'),
	    handler: function() {
		var me = this.up('grid');
		Proxmox.Utils.API2Request({
		    url: '/nodes/' + Proxmox.NodeName + '/spamassassin/rules',
		    method: 'POST',
		    failure: function(response) {
			Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		    },
		    success: function(response) {
			var upid = response.result.data;

			var win = Ext.create('Proxmox.window.TaskViewer', {
			    upid: upid
			});
			win.show();
			me.mon(win, 'close', function() { me.store.load(); });
		    }
		});
	    }
	}
    ],

    initComponent : function() {
	var me = this;

	me.store = Ext.create('Ext.data.Store', {
	    model: 'pmg-spamassassin-database',
	    proxy: {
		type: 'proxmox',
		url: "/api2/json/nodes/" + Proxmox.NodeName + "/spamassassin/rules"
	    },
	    sorters: {
		property: 'name',
		order: 'DESC'
	    }
	});

	me.callParent();

	Proxmox.Utils.monStoreErrors(me.getView(), me.store, true);
    }
});
Ext.define('PMG.SpamDetectorConfiguration', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pmgSpamDetectorConfiguration',

    title: gettext('Configuration') + ': ' +
	gettext('Spam Detector'),

    border: false,
    defaults: { border: false },

    items: [
	{
	    title: gettext('Options'),
	    itemId: 'options',
	    xtype: 'pmgSpamDetectorOptions'
	},
	{
	    title: gettext('Quarantine'),
	    itemId: 'quarantine',
	    xtype: 'pmgSpamQuarantineOptions'
	},
	{
	    title: gettext('Status'),
	    itemId: 'status',
	    xtype: 'pmgSpamDetectorStatus'
	}
    ]
});


Ext.define('PMG.VirusDetectorOptions', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgVirusDetectorOptions'],

    monStoreErrors: true,

    initComponent : function() {
	var me = this;

	me.add_boolean_row('archiveblockencrypted',
			   gettext('Block encrypted archives'));
	
	me.add_integer_row('archivemaxrec', gettext('Max recursion'),
			   { minValue: 1, defaultValue: 5,
			     deleteEmpty: true });

	me.add_integer_row('archivemaxfiles', gettext('Max files'),
			   { minValue: 0, defaultValue: 1000,
			     deleteEmpty: true });

	me.add_integer_row('archivemaxsize', gettext('Max file size'),
			   { minValue: 1000000, defaultValue: 25000000,
			     deleteEmpty: true });

	me.add_integer_row('maxscansize', gettext('Max scan size'),
			   { minValue: 1000000, defaultValue: 100000000,
			     deleteEmpty: true });

	me.add_integer_row('maxcccount', gettext('Max credit card numbers'),
			   { minValue: 0, defaultValue: 0,
			     deleteEmpty: true });
	
	var baseurl = '/config/clamav';

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	Ext.apply(me, {
	    tbar: [{
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { me.run_editor(); },
		selModel: me.selModel
	    }],
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
Ext.define('PMG.VirusQuarantineOptions', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgVirusQuarantineOptions'],

    monStoreErrors: true,

    initComponent : function() {
	var me = this;

	me.add_integer_row('lifetime', gettext('Lifetime (days)'),
			   { minValue: 1, defaultValue: 7,
			     deleteEmpty: true });

	me.add_boolean_row('viewimages', gettext('View images'),
			   { defaultValue: 1});

	me.add_boolean_row('allowhrefs', gettext('Allow HREFs'),
			   {defaultValue: 1 });

	var baseurl = '/config/virusquar';

	me.selModel = Ext.create('Ext.selection.RowModel', {});

	Ext.apply(me, {
	    tbar: [{
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { me.run_editor(); },
		selModel: me.selModel
	    }],
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});
/*global Proxmox*/
/*jslint confusion: true*/
/*format is a string and a function*/
Ext.define('pmg-virus-list', {
    extend: 'Ext.data.Model',
    fields: [ 'id', 'envelope_sender', 'from', 'sender', 'receiver', 'subject',
	{ type: 'integer', name: 'bytes' },
	{ type: 'string', name: 'virusname' },
	{ type: 'date', dateFormat: 'timestamp', name: 'time' },
	{
	    type: 'string',
	    name: 'day',
	    convert: function(v, rec) {
		return Ext.Date.format(rec.get('time'), 'Y-m-d');
	    }, depends: ['time']
	}
    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/quarantine/virus"
    },
    idProperty: 'id'
});

Ext.define('PMG.VirusQuarantine', {
    extend: 'Ext.container.Container',
    xtype: 'pmgVirusQuarantine',

    border: false,
    layout: { type: 'border' },

    defaults: { border: false },

    controller: {

	xclass: 'Ext.app.ViewController',

	updatePreview: function(raw) {
	    var list = this.lookupReference('list');
	    var rec = list.selModel.getSelection()[0];
	    var preview = this.lookupReference('preview');

	    if (!rec || !rec.data || !rec.data.id)  {
		preview.update('');
		preview.setDisabled(true);
		return;
	    }

	    var url = '/api2/htmlmail/quarantine/content?id=' + rec.data.id + ((raw)?'&raw=1':'');
	    preview.setDisabled(false);
	    preview.update("<iframe frameborder=0 width=100% height=100% sandbox='allow-same-origin' src='" + url +"'></iframe>");
	},

	toggleRaw: function(button) {
	    var me = this;
	    me.raw = !me.raw;
	    me.updatePreview(me.raw);
	},

	btnHandler: function(button, e) {
	    var list = this.lookupReference('list');
	    var selected = list.getSelection();
	    if (!selected.length) {
		return;
	    }

	    var action = button.reference;

	    PMG.Utils.doQuarantineAction(action, selected[0].data.id, function() {
		list.getController().load();
	    });
	},

	onSelectMail: function() {
	    var me = this;
	    me.updatePreview(me.raw || false);
	},

	control: {
	    'button[reference=raw]': {
		click: 'toggleRaw'
	    },
	    'pmgQuarantineList': {
		selectionChange: 'onSelectMail'
	    }
	}

    },

    items: [
	{
	    title: gettext('Virus Quarantine'),
	    xtype: 'pmgQuarantineList',
	    emptyText: gettext('No data in database.'),
	    emailSelection: false,
	    reference: 'list',
	    region: 'west',
	    width: 500,
	    split: true,
	    collapsible: false,
	    store: {
		model: 'pmg-virus-list',
		groupField: 'day',
		groupDir: 'DESC',
		sorters: [{
		    property: 'time',
		    direction: 'DESC'
		}]
	    },

	    columns: [
		{
		    header: gettext('Sender/Subject'),
		    dataIndex: 'subject',
		    renderer: PMG.Utils.sender_renderer,
		    flex: 1
		},
		{
		    header: gettext('Virus'),
		    dataIndex: 'virusname',
		    align: 'right',
		    width: 70
		},
		{
		    header: gettext('Size') + ' (KB)',
		    renderer: function(v) { return Ext.Number.toFixed(v/1024, 0); },
		    dataIndex: 'bytes',
		    align: 'right',
		    width: 90
		},
		{
		    header: gettext('Date'),
		    dataIndex: 'day',
		    hidden: true
		},
		{
		    xtype: 'datecolumn',
		    header: gettext('Time'),
		    dataIndex: 'time',
		    format: 'H:m:s'
		}
	    ]
	},
	{
	    title: gettext('Selected Mail'),
	    border: false,
	    region: 'center',
	    split: true,
	    reference: 'preview',
	    disabled: true,
	    dockedItems: [
		{
		    xtype: 'toolbar',
		    dock: 'top',
		    items: [
			{
			    xtype: 'button',
			    reference: 'raw',
			    text: gettext('Toggle Raw'),
			    enableToggle: true,
			    iconCls: 'fa fa-file-code-o'
			},
			'->',
			{
			    reference: 'deliver',
			    text: gettext('Deliver'),
			    iconCls: 'fa fa-paper-plane-o',
			    handler: 'btnHandler'
			},
			{
			    reference: 'delete',
			    text: gettext('Delete'),
			    iconCls: 'fa fa-trash-o',
			    handler: 'btnHandler'
			}
		    ]
		}
	    ]
	}
    ]
});
/*global Proxmox*/
Ext.define('PMG.ClamAVDatabaseConfig', {
    extend: 'Proxmox.grid.ObjectGrid',
    alias: ['widget.pmgClamAVDatabaseConfig'],

    monStoreErrors: true,

    initComponent : function() {
	var me = this;

	me.add_text_row('dbmirror', gettext('Database Mirror'),
			{ deleteEmpty: true, defaultValue: 'database.clamav.net' });

	/*jslint confusion: true*/
	/*defaultValue is a string above*/
	me.add_boolean_row('safebrowsing', gettext('Google Safe Browsing'),
			   { defaultValue: 1 });
	/*jslint confusion: false*/

	var baseurl = '/config/clamav';

	Ext.apply(me, {
	    url: '/api2/json' + baseurl,
	    editorConfig: {
		url: '/api2/extjs' + baseurl
	    },
	    interval: 5000,
	    cwidth1: 200,
	    listeners: {
		itemdblclick: me.run_editor
	    }
	});

	me.callParent();

	me.on('activate', me.rstore.startUpdate);
	me.on('destroy', me.rstore.stopUpdate);
	me.on('deactivate', me.rstore.stopUpdate);
    }
});

Ext.define('pmg-clamav-database', {
    extend: 'Ext.data.Model',
    fields: [ 'name', 'type', 'build_time', 'version',
	      { name: 'nsigs', type: 'integer' }],
    idProperty: 'name'
});

Ext.define('PMG.ClamAVDatabaseStatus', {
    extend: 'Ext.grid.GridPanel',
    alias: ['widget.pmgClamAVDatabaseStatus'],

    title: gettext('Status'),

    reload: function() {
	var me = this;

        me.store.load();
    },

    initComponent : function() {
	var me = this;

	me.store = new Ext.data.Store({
	    model: 'pmg-clamav-database',
	    proxy: {
		type: 'proxmox',
		url: "/api2/json/nodes/" + Proxmox.NodeName + "/clamav/database"
	    },
	    sorters: {
		property: 'name',
		order: 'DESC'
	    }
	});

	Ext.apply(me, {
	    viewConfig: {
		trackOver: false
	    },
	    columns: [
		{
		    header: gettext('Name'),
		    sortable: true,
		    flex: 1,
		    dataIndex: 'name'
		},
		{
		    header: gettext('Build time'),
		    sortable: true,
		    flex: 2,
		    dataIndex: 'build_time'
		},
		{
		    header: gettext('Version'),
		    flex: 1,
		    sortable: true,
		    dataIndex: 'version'
		},
		{
		    header: gettext('Signatures'),
		    flex: 1,
		    sortable: true,
		    dataIndex: 'nsigs'
		}
	    ],
	    listeners: {
		activate: me.reload
	    }
	});

	me.callParent();

	/*jslint confusion: true*/
	/*monStoreErrors is a bool above*/
	Proxmox.Utils.monStoreErrors(me.getView(), me.store, true);
	/*jslint confusion: false*/
    }
});

Ext.define('PMG.ClamAVDatabase', {
    extend: 'Ext.panel.Panel',
    alias: ['widget.pmgClamAVDatabase'],

    layout: { type: 'vbox', align: 'stretch' },

    initComponent : function() {
	var me = this;

	var selModel = Ext.create('Ext.selection.RowModel', {});
	var editPanel = Ext.create('PMG.ClamAVDatabaseConfig', {
	    border: false,
	    xtype: 'pmgClamAVDatabaseConfig',
	    selModel: selModel
	});

	var statusPanel = Ext.create('PMG.ClamAVDatabaseStatus', {
	    border: false,
	    flex: 1
	});

	var update_command = function(){
	    Proxmox.Utils.API2Request({
		url: '/nodes/' + Proxmox.NodeName + '/clamav/database',
		method: 'POST',
		failure: function(response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		},
		success: function(response, opts) {
		    var upid = response.result.data;

		    var win = Ext.create('Proxmox.window.TaskViewer', {
			upid: upid
		    });
		    win.show();
		    me.mon(win, 'close', function() { statusPanel.reload(); });
		}
	    });
	};

	me.tbar = [
	    {
		text: gettext('Edit'),
		xtype: 'proxmoxButton',
		disabled: true,
		handler: function() { editPanel.run_editor(); },
		selModel: selModel
            },
	    {
		text: gettext('Update now'),
		handler: update_command
	    }
	];

	me.items = [ editPanel, statusPanel ];

	me.callParent();

	editPanel.relayEvents(me, ['activate', 'deactivate', 'destroy']);
	statusPanel.relayEvents(me, ['activate', 'deactivate', 'destroy']);

    }
});
Ext.define('PMG.VirusDetectorConfiguration', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pmgVirusDetectorConfiguration',

    title: gettext('Configuration') + ': ' +
	gettext('Virus Detector'),

    border: false,
    defaults: { border: false },

    items: [
	{
	    title: gettext('Options'),
	    itemId: 'options',
	    xtype: 'pmgVirusDetectorOptions'
	},
	{
	    title: gettext('ClamAV'),
	    itemId: 'clamav',
	    xtype: 'pmgClamAVDatabase'
	},
	{
	    title: gettext('Quarantine'),
	    itemId: 'quarantine',
	    xtype: 'pmgVirusQuarantineOptions'
	}
    ]
});


/*global Proxmox*/
/*jslint confusion: true*/
/* reload is function and string,
 * height is number and string,
 * hidden is bool and string,
 * bind is function and object,
 * callback is function and string
 */
Ext.define('pmg-ldap-config', {
    extend: 'Ext.data.Model',
    fields: [ 'profile', 'server1', 'server2', 'comment',
	      'mode', 'binddn', 'bindpw', 'basedn', 'groupbasedn',
	      'filter', 'accountattr', 'mailattr',
	      { name: 'port',  type: 'integer' },
	      { name: 'gcount',  type: 'integer' },
	      { name: 'mcount',  type: 'integer' },
	      { name: 'ucount',  type: 'integer' },
	      { name: 'disable',  type: 'boolean' }
	    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/config/ldap"
    },
    idProperty: 'profile'
});

Ext.define('PMG.LDAPInputPanel', {
    extend: 'Proxmox.panel.InputPanel',
    alias: 'widget.pmgLDAPInputPanel',

    profileId: undefined,

    onGetValues: function(values) {
	var me = this;

	values.disable = values.enable ? 0 : 1;
	delete values.enable;

	return values;
    },

    initComponent : function() {
	var me = this;

	me.column1 = [
	    {
		xtype: me.profileId ? 'displayfield' : 'textfield',
		fieldLabel: gettext('Profile Name'),
		value: me.profileId || '',
		name: 'profile',
		vtype: 'StorageId',
		allowBlank: false
	    },
	    {
		xtype: 'proxmoxKVComboBox',
		name: 'mode',
		comboItems: [
		    ['ldap', PMG.Utils.format_ldap_protocol('ldap')],
		    ['ldaps', PMG.Utils.format_ldap_protocol('ldaps')]
		],
		value: 'ldap',
		fieldLabel: gettext('Protocol')
	    },
	    {
		xtype: 'textfield',
		fieldLabel: gettext('Server'),
		allowBlank: false,
		vtype: 'IP64Address',
		name: 'server1'
	    },
	    {
		xtype: 'proxmoxtextfield',
		fieldLabel: gettext('Server'),
		allowBlank: true,
		deleteEmpty: me.isCreate ? false : true,
		vtype: 'IP64Address',
		name: 'server2'
	    },
	    {
		xtype: 'proxmoxintegerfield',
		name: 'port',
		emptyText: gettext('Default'),
		deleteEmpty: me.isCreate ? false : true,
		minValue: 1,
		maxValue: 65535,
		fieldLabel: gettext('Port')
	    },
	    {
		xtype: 'textfield',
		name: 'binddn',
		allowBlank: true,
		fieldLabel: gettext('User name')
	    },
	    {
		xtype: 'textfield',
		inputType: 'password',
		allowBlank: true,
		name: 'bindpw',
		fieldLabel: gettext('Password')
	    }
	];

	me.column2 = [
	    {
		xtype: 'proxmoxcheckbox',
		name: 'enable',
		checked: true,
		uncheckedValue: 0,
		fieldLabel: gettext('Enable')
	    },
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: true,
		deleteEmpty: me.isCreate ? false : true,
		name: 'basedn',
		fieldLabel: gettext('Base DN')
	    },
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: true,
		deleteEmpty: me.isCreate ? false : true,
		name: 'groupbasedn',
		fieldLabel: gettext('Base DN for Groups')
	    },
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: true,
		deleteEmpty: me.isCreate ? false : true,
		name: 'mailattr',
		fieldLabel: gettext('EMail attribute name(s)')
	    },
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: true,
		deleteEmpty: me.isCreate ? false : true,
		name: 'accountattr',
		fieldLabel: gettext('Account attribute name')
	    },
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: true,
		deleteEmpty: me.isCreate ? false : true,
		name: 'filter',
		fieldLabel: gettext('LDAP filter')
	    },
	    {
		xtype: 'proxmoxtextfield',
		allowBlank: true,
		deleteEmpty: me.isCreate ? false : true,
		name: 'groupclass',
		fieldLabel: gettext('Group objectclass')
	    }
	];

	me.columnB = [
	    {
		xtype: 'textfield',
		fieldLabel: gettext('Comment'),
		allowBlank: true,
		name: 'comment'
	    }
	];

	me.callParent();
    }
});

Ext.define('PMG.LDAPEdit', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pmgLDAPEdit',

    subject: 'LDAP Profile',
    isAdd: true,

    initComponent : function() {
	var me = this;

	me.isCreate = me.profileId ? false : true;

	if (me.isCreate) {
            me.url = '/api2/extjs/config/ldap';
            me.method = 'POST';
	} else {
            me.url = '/api2/extjs/config/ldap/' + me.profileId + '/config';
            me.method = 'PUT';
	}

	var ipanel = Ext.create('PMG.LDAPInputPanel', {
	    isCreate: me.isCreate,
	    profileId: me.profileId
	});

	me.items = [ ipanel ];

	me.fieldDefaults = {
	    labelWidth: 150
	};

	me.callParent();

	if (!me.isCreate) {
	    me.load({
		success:  function(response, options) {
		    var values = response.result.data;

		    values.enable = values.disable ? 0 : 1;
		    ipanel.setValues(values);
		}
	    });
	}
    }
});

Ext.define('PMG.LDAPUserGrid', {
    extend: 'Ext.grid.Panel',
    xtype: 'pmgLDAPUserGrid',

    emptyText: gettext('No data in database'),
    store: {
	autoDestroy: true,
	fields: [ 'dn', 'account', 'pmail' ],
	proxy: { type: 'proxmox' },
	sorters: [ 'dn' ]
    },
    columns: [
	{
	    text: 'DN',
	    dataIndex: 'dn',
	    flex: 1
	},
	{
	    text: gettext('Account'),
	    dataIndex: 'account',
	    flex: 1
	},
	{
	    text: gettext('Primary E-Mail'),
	    dataIndex: 'pmail',
	    flex: 1
	}
    ],

    initComponent: function() {
	var me = this;
	me.callParent();
	if (me.url) {
	    me.getStore().getProxy().setUrl(me.url);
	    me.getStore().load();
	}
    }
});

Ext.define('PMG.LDAPConfig', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgLDAPConfig',

    controller: {
	xclass: 'Ext.app.ViewController',

	openUserList: function(grid, record) {
	    var name = this.getViewModel().get('name');
	    Ext.create('Ext.window.Window', {
		title: Ext.String.format(gettext("Users of '{0}'"), record.data.dn),
		modal: true,
		width: 600,
		height: 400,
		layout: 'fit',
		items: [{
		    xtype: 'pmgLDAPUserGrid',
		    border: false,
		    url: '/api2/json/config/ldap/' + name + '/groups/' +  record.data.gid
		}]
	    }).show();
	},

	showUsers: function(button) {
	    var me = this;
	    var view = me.lookup('groupgrid');
	    var record = view.getSelection()[0];
	    me.openUserList(view, record);
	},

	openUserMails: function(grid, record) {
	    var name = this.getViewModel().get('name');
	    Ext.create('Ext.window.Window', {
		title: Ext.String.format(gettext("E-Mail addresses of '{0}'"), record.data.dn),
		modal: true,
		width: 600,
		height: 400,
		layout: 'fit',
		items: [{
		    xtype: 'grid',
		    border: false,
		    store: {
			autoLoad: true,
			field: ['email', 'primary'],
			proxy: {
			    type: 'proxmox',
			    url: '/api2/json/config/ldap/' + name + '/users/' +  record.data.pmail
			}
		    },
		    columns: [
			{ dataIndex: 'email', text: gettext('E-Mail address'), flex: 1 }
		    ]
		}]
	    }).show();
	},

	showEmails: function(button) {
	    var me = this;
	    var view = me.lookup('usergrid');
	    var record = view.getSelection()[0];
	    me.openUserMails(view, record);
	},

	reload: function(grid) {
	    var me = this;
	    var selection = grid.getSelection();
	    me.showInfo(grid, selection);
	},

	showInfo: function(grid, selected) {
	    var me = this;
	    var viewModel = me.getViewModel();
	    if (selected[0]) {
		var name = selected[0].data.profile;
		viewModel.set('selected', true);
		viewModel.set('name', name);

		// set grid stores and load them
		var gstore = me.lookup('groupgrid').getStore();
		var ustore = me.lookup('usergrid').getStore();
		gstore.getProxy().setUrl('/api2/json/config/ldap/' + name + '/groups');
		ustore.getProxy().setUrl('/api2/json/config/ldap/' + name + '/users');
		gstore.load();
		ustore.load();
	    } else {
		viewModel.set('selected', false);
	    }
	},

	init: function(view) {
	    var me = this;
	    me.lookup('grid').relayEvents(view, ['activate']);
	    var groupgrid = me.lookup('groupgrid');
	    var usergrid = me.lookup('usergrid');

	    Proxmox.Utils.monStoreErrors(groupgrid, groupgrid.getStore(), true);
	    Proxmox.Utils.monStoreErrors(usergrid, usergrid.getStore(), true);
	},

	control: {
	    'grid[reference=grid]': {
		selectionchange: 'showInfo',
		load: 'reload'
	    },
	    'grid[reference=groupgrid]': {
		itemdblclick: 'openUserList'
	    },
	    'grid[reference=usergrid]': {
		itemdblclick: 'openUserMails'
	    }
	}
    },

    viewModel: {
	data: {
	    name: '',
	    selected: false
	}
    },

    layout: 'border',

    items: [
	{
	    region: 'center',
	    reference: 'grid',
	    xtype: 'pmgLDAPConfigGrid',
	    border: false
	},
	{
	    xtype: 'tabpanel',
	    reference: 'data',
	    hidden: true,
	    height: '50%',
	    border: false,
	    split: true,
	    region: 'south',
	    bind: {
		hidden: '{!selected}'
	    },
	    items: [
		{
		    xtype: 'grid',
		    reference: 'groupgrid',
		    border: false,
		    emptyText: gettext('No data in database'),
		    tbar: [{
			xtype: 'proxmoxButton',
			text: gettext('Show Users'),
			handler: 'showUsers',
			disabled: true
		    }],
		    store: {
			fields: ['dn', 'gid'],
			proxy: { type: 'proxmox' },
			sorters: [ 'dn' ]
		    },
		    bind: {
			title: Ext.String.format(gettext("Groups of '{0}'"), '{name}')
		    },
		    columns: [
			{
			    text: 'DN',
			    dataIndex: 'dn',
			    flex: 1
			}
		    ]
		},
		{
		    xtype: 'pmgLDAPUserGrid',
		    reference: 'usergrid',
		    border: false,
		    tbar: [{
			xtype: 'proxmoxButton',
			text: gettext('Show E-Mail addresses'),
			handler: 'showEmails',
			disabled: true
		    }],
		    bind: {
			title: Ext.String.format(gettext("Users of '{0}'"), '{name}')
		    }
		}
	    ]
	}
    ]

});

Ext.define('PMG.LDAPConfigGrid', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgLDAPConfigGrid',

    controller: {
	xclass: 'Ext.app.ViewController',

	run_editor: function() {
	    var me = this;
	    var view = me.getView();
	    var rec = view.getSelection()[0];
	    if (!rec) {
		return;
	    }

	    var win = Ext.createWidget('pmgLDAPEdit', {
		profileId: rec.data.profile
	    });
	    win.on('destroy', me.reload, me);
	    win.load();
	    win.show();
	},

	newProfile: function() {
	    var me = this;
	    var win = Ext.createWidget('pmgLDAPEdit', {});
	    win.on('destroy', me.reload, me);
	    win.show();
	},


	reload: function() {
	    var me = this.getView();
	    me.getStore().load();
	    me.fireEvent('load', me);
	},

	sync: function() {
	    var me = this;
	    var view = me.getView();
	    var rec = view.getSelection()[0];
	    Proxmox.Utils.API2Request({
		url: '/config/ldap/' + rec.data.profile + '/sync',
		method: 'POST',
		waitMsgTarget: view,
		callback: function() {
		    me.reload();
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	init: function(view) {
	    var me = this;
	    Proxmox.Utils.monStoreErrors(view, view.getStore(), true);
	}
    },

    store: {
	model: 'pmg-ldap-config',
	sorters: [{
	    property: 'profile',
	    order: 'DESC'
	}]
    },

    tbar: [
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Edit'),
	    disabled: true,
	    handler: 'run_editor'
	},
	{
	    text: gettext('Create'),
	    handler: 'newProfile'
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    baseurl: '/config/ldap',
	    callback: 'reload'
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Synchronize'),
	    enableFn: function(rec) {
		return !rec.data.disable;
	    },
	    disabled: true,
	    handler: 'sync'
	}
    ],

    listeners: {
	itemdblclick: 'run_editor',
	activate: 'reload'
    },

    columns: [
	{
	    header: gettext('Profile Name'),
	    sortable: true,
	    width: 120,
	    dataIndex: 'profile'
	},
	{
	    header: gettext('Protocol'),
	    sortable: true,
	    dataIndex: 'mode',
	    renderer: PMG.Utils.format_ldap_protocol
	},
	{
	    header: gettext('Server'),
	    sortable: true,
	    dataIndex: 'server1',
	    renderer: function(value, metaData, rec) {
		if (rec.data.server2) {
		    return value + '<br>' + rec.data.server2;
		}
		return value;
	    }
	},
	{
	    header: gettext('Enabled'),
	    width: 80,
	    sortable: true,
	    dataIndex: 'disable',
	    renderer: Proxmox.Utils.format_neg_boolean
	},
	{
	    header: gettext('Comment'),
	    sortable: false,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'comment',
	    flex: 1
	},
	{
	    header: gettext('Accounts'),
	    width: 80,
	    sortable: true,
	    dataIndex: 'ucount'
	},
	{
	    header: gettext('Addresses'),
	    width: 80,
	    sortable: true,
	    dataIndex: 'mcount'
	},
	{
	    header: gettext('Groups'),
	    width: 80,
	    sortable: true,
	    dataIndex: 'gcount'
	}
    ]

});
/*global Proxmox*/
/*jslint confusion: true*/
/* submitvalue is string and bool */
Ext.define('PMG.UserEdit', {
    extend: 'Proxmox.window.Edit',
    alias: 'widget.pmgUserEdit',
    mixins: ['Proxmox.Mixin.CBind'],

    userid: undefined,

    isAdd: true,

    subject: gettext('User'),

    fieldDefaults: { labelWidth: 120 },

    cbindData: function(initialConfig) {
	var me = this;

	var userid = initialConfig.userid;
	var baseurl = '/api2/extjs/access/users';

	me.isCreate = !userid;
	me.url = userid ?  baseurl + '/' + userid : baseurl;
	me.method = userid ? 'PUT' : 'POST';
	me.autoLoad = userid ? true : false;

	return {
	    useridXType: userid ? 'displayfield' : 'textfield',
	    isSuperUser: userid === 'root@pam'
	};
    },

    items: {
	xtype: 'inputpanel',
	column1: [
	    {
		xtype: 'textfield',
		name: 'username',
		fieldLabel: gettext('User name'),
		allowBlank: false,
		cbind: {
		    submitValue: '{isCreate}',
		    xtype: '{useridXType}'
		}
	    },
	    {
		xtype: 'textfield',
		inputType: 'password',
		fieldLabel: gettext('Password'),
		minLength: 5,
		allowBlank: false,
		name: 'password',
		listeners: {
                    change: function(field){
			field.next().validate();
                    },
                    blur: function(field){
			field.next().validate();
                    }
		},
		cbind: {
		    hidden: '{!isCreate}',
		    disabled: '{!isCreate}'
		}
	    },
	    {
		xtype: 'textfield',
		inputType: 'password',
		fieldLabel: gettext('Confirm password'),
		name: 'verifypassword',
		vtype: 'password',
		initialPassField: 'password',
		allowBlank: false,
		submitValue: false,
		cbind: {
		    hidden: '{!isCreate}',
		    disabled: '{!isCreate}'
		}
	    },
	    {
		xtype: 'pmgRoleSelector',
		name: 'role',
		allowBlank: false,
		fieldLabel: gettext('Role'),
		cbind: {
		    disabled: '{isSuperUser}'
		}
	    },
	    {
                xtype: 'datefield',
                name: 'expire',
		emptyText: Proxmox.Utils.neverText,
		format: 'Y-m-d',
		submitFormat: 'U',
                fieldLabel: gettext('Expire'),
		cbind: {
		    disabled: '{isSuperUser}'
		}
            },
	    {
		xtype: 'proxmoxcheckbox',
		fieldLabel: gettext('Enabled'),
		name: 'enable',
		uncheckedValue: 0,
		defaultValue: 1,
		checked: true,
		cbind: {
		    disabled: '{isSuperUser}'
		}
	    }
	],

	column2: [
	    {
		xtype: 'proxmoxtextfield',
		name: 'firstname',
		fieldLabel: gettext('First Name'),
		cbind: {
		    deleteEmpty: '{!isCreate}'
		}
	    },
	    {
		xtype: 'proxmoxtextfield',
		name: 'lastname',
		fieldLabel: gettext('Last Name'),
		cbind: {
		    deleteEmpty: '{!isCreate}'
		}
	    },
	    {
		xtype: 'proxmoxtextfield',
		name: 'email',
		fieldLabel: gettext('E-Mail'),
		vtype: 'proxmoxMail',
		cbind: {
		    deleteEmpty: '{!isCreate}'
		}
	    }
	],

	columnB: [
	    {
		xtype: 'proxmoxtextfield',
		name: 'comment',
		fieldLabel: gettext('Comment'),
		cbind: {
		    disabled: '{isSuperUser}',
		    deleteEmpty: '{!isCreate}'
		}
	    },
	    {
		xtype: 'proxmoxtextfield',
		name: 'keys',
		fieldLabel: gettext('Key IDs'),
		cbind: {
		    deleteEmpty: '{!isCreate}'
		}
	    }
	]
    },

    getValues: function(dirtyOnly) {
	var me = this;

	var values = me.callParent(arguments);

	// hack: ExtJS datefield does not submit 0, so we need to set that
	if (!values.expire) {
	    values.expire = 0;
	}

	if (me.isCreate) {
	    values.userid = values.username + '@pmg';
	}

	delete values.username;

	if (!values.password) {
	    delete values.password;
	}

	return values;
    },

    setValues: function(values) {
	var me = this;

	if (Ext.isDefined(values.expire)) {
	    if (values.expire) {
		values.expire = new Date(values.expire * 1000);
	    } else {
		// display 'never' instead of '1970-01-01'
		values.expire = null;
	    }
	}

	me.callParent([values]);
    }
});
/*global Proxmox*/
/*jslint confusion: true*/
/*renderer is string and function*/
Ext.define('pmg-users', {
    extend: 'Ext.data.Model',
    fields: [
	'userid', 'firstname', 'lastname' , 'email', 'comment',
	'role', 'keys', 'realm',
	{ type: 'boolean', name: 'enable' },
	{ type: 'date', dateFormat: 'timestamp', name: 'expire' }
    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/access/users"
    },
    idProperty: 'userid'
});

Ext.define('PMG.UserView', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgUserView',

    store: {
	autoLoad: true,
	model: 'pmg-users',
	sorters: [
	    {
		property: 'realm',
		direction: 'ASC'
	    },
	    {
		property: 'userid',
		direction: 'ASC'
	    }
	]
    },

    controller: {

	xclass: 'Ext.app.ViewController',

	init: function(view) {
	    Proxmox.Utils.monStoreErrors(view, view.store);
	},

	renderUsername: function(userid) {
	    return Ext.htmlEncode(userid.match(/^(.+)(@[^@]+)$/)[1]);
	},

	renderFullName: function(firstname, metaData, record) {
	    var first = firstname || '';
	    var last = record.data.lastname || '';
	    return Ext.htmlEncode(first + " " + last);
	},

	onAdd: function() {
	    var view = this.getView();

            var win = Ext.create('PMG.UserEdit', {});
            win.on('destroy', function() { view.reload(); });
            win.show();
	},

	onEdit: function() {
	    var view = this.getView();

	    var rec = view.selModel.getSelection()[0];

            var win = Ext.create('PMG.UserEdit', {
		userid: rec.data.userid
            });
            win.on('destroy', function() { view.reload(); });
            win.show();
	},

	onPassword: function(btn, event, rec) {
	    var view = this.getView();

	    var win = Ext.create('Proxmox.window.PasswordEdit',{
                userid: rec.data.userid
	    });
            win.on('destroy', function() { view.reload(); });
	    win.show();
	},

	onAfterRemove: function(btn, res) {
	    var view = this.getView();
	    view.reload();
	}
    },

    listeners: {
	scope: 'controller',
	itemdblclick: 'onEdit'
    },

    tbar: [
        {
	    text: gettext('Add'),
	    reference: 'addBtn',
	    handler: 'onAdd'
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Edit'),
	    disabled: true,
	    handler: 'onEdit'
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    baseurl: '/access/users',
	    reference: 'removeBtn',
	    callback: 'onAfterRemove',
	    waitMsgTarget: true
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Password'),
	    disabled: true,
	    handler: 'onPassword'
	}
    ],

    columns: [
	{
	    header: gettext('User name'),
	    width: 200,
	    sortable: true,
	    renderer: 'renderUsername',
	    dataIndex: 'userid'
	},
	{
	    header: gettext('Realm'),
	    width: 100,
	    sortable: true,
	    dataIndex: 'realm'
	},
	{
	    header: gettext('Role'),
	    width: 150,
	    sortable: true,
	    renderer: PMG.Utils.format_user_role,
	    dataIndex: 'role'
	},
	{
	    header: gettext('Enabled'),
	    width: 80,
	    sortable: true,
	    renderer: Proxmox.Utils.format_boolean,
	    dataIndex: 'enable'
	},
	{
	    header: gettext('Expire'),
	    width: 80,
	    sortable: true,
	    renderer: Proxmox.Utils.format_expire,
	    dataIndex: 'expire'
	},
	{
	    header: gettext('Name'),
	    width: 150,
	    sortable: true,
	    renderer: 'renderFullName',
	    dataIndex: 'firstname'
	},
	{
	    header: gettext('Comment'),
	    sortable: false,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'comment',
	    flex: 1
	}
    ],

    reload: function() {
	var me = this;

	me.store.load();
    }
});
/*jslint confusion: true*/
/*value is string and number*/
Ext.define('PMG.FetchmailEdit', {
    extend: 'Proxmox.window.Edit',
    xtype: 'pmgFetchmailEdit',

    userid: undefined,

    isAdd: true,

    subject: 'Fetchmail',

    fieldDefaults: { labelWidth: 120 },

    controller: {

	xclass: 'Ext.app.ViewController',

	onProtocolChange: function() {
	    var protocol = this.lookupReference('protocol').getValue();
	    var ssl = this.lookupReference('ssl').getValue();

	    var port_field =  this.lookupReference('port');
	    if (protocol === 'pop3') {
		port_field.setValue(ssl ? 995 : 110);
	    } else if (protocol === 'imap') {
		port_field.setValue(ssl ? 993 : 143);
	    }
	}
    },

    items: {
	xtype: 'inputpanel',
	column1: [
	    {
		xtype: 'textfield',
		name: 'server',
		fieldLabel: gettext('Server'),
		allowBlank: false
	    },
	    {
		xtype: 'proxmoxKVComboBox',
		fieldLabel: gettext('Protocol'),
		name: 'protocol',
		reference: 'protocol',
		value: 'pop3',
		listeners: { change: 'onProtocolChange' },
		comboItems: [['pop3', 'pop3'], ['imap', 'imap']]
	    },
	    {
		xtype: 'proxmoxintegerfield',
		name: 'port',
		reference: 'port',
		fieldLabel: gettext('Port'),
		value: 110,
		minValue: 1,
		maxValue: 65535,
		allowBlank: false
	    },
	    {
		xtype: 'textfield',
		name: 'user',
		fieldLabel: gettext('Username'),
		allowBlank: false
	    },
	    {
		xtype: 'textfield',
		name: 'pass',
		inputType: 'password',
		fieldLabel: gettext('Passsword'),
		allowBlank: false
	    },
	    {
		xtype: 'textfield',
		name: 'target',
		fieldLabel: gettext('Deliver to'),
		allowBlank: false
	    }

	],
	column2: [
	    {
		xtype: 'proxmoxcheckbox',
		fieldLabel: gettext('Enabled'),
		name: 'enable',
		uncheckedValue: 0,
		checked: true
	    },
	    {
		xtype: 'proxmoxintegerfield',
		name: 'interval',
		fieldLabel: gettext('Interval'),
		value: 1,
		minValue: 1,
		maxValue: 24*12*7,
		allowBlank: false
	    },
	    {
		xtype: 'proxmoxcheckbox',
		fieldLabel: gettext('Use SSL'),
		listeners: { change: 'onProtocolChange' },
		name: 'ssl',
		reference: 'ssl',
		uncheckedValue: 0,
		checked: false
	    },
	    {
		xtype: 'proxmoxcheckbox',
		fieldLabel: gettext('Keep old mails'),
		name: 'keep',
		uncheckedValue: 0,
		checked: false
	    }
	]
    }
});
/*global Proxmox*/
Ext.define('pmg-fetchmail-users', {
    extend: 'Ext.data.Model',
    fields: [
	'id', 'protocol', 'port', 'server', 'user', 'pass', 'target',
	'ssl', 'keep',
	{ type: 'integer', name: 'interval' },
	{ type: 'boolean', name: 'enable' }
    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/config/fetchmail"
    },
    idProperty: 'id'
});

Ext.define('PMG.FetchmailView', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgFetchmailView',

    baseurl: '/api2/extjs/config/fetchmail',

    store: {
	autoDestroy: true,
	autoLoad: true,
	model: 'pmg-fetchmail-users'
    },

    controller: {

	xclass: 'Ext.app.ViewController',

	init: function(view) {
	    Proxmox.Utils.monStoreErrors(view, view.store, true);
	},

	onAdd: function() {
	    var view = this.getView();

            var win = Ext.create('PMG.FetchmailEdit', {
		url: view.baseurl,
		method: 'POST'
            });
            win.on('destroy', function() { view.store.load(); });
            win.show();
	},

	onEdit: function() {
	    var view = this.getView();

	    var rec = view.selModel.getSelection()[0];

            var win = Ext.create('PMG.FetchmailEdit', {
		userid: rec.data.id,
		url: view.baseurl + '/' + rec.data.id,
		method: 'PUT',
		autoLoad: true
            });
            win.on('destroy', function() { view.store.load(); });
            win.show();
	},

	onAfterRemove: function(btn, res) {
	    var view = this.getView();
	    view.store.load();
	}
    },

    tbar: [
        {
	    text: gettext('Add'),
	    reference: 'addBtn',
	    handler: 'onAdd'
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Edit'),
	    disabled: true,
	    handler: 'onEdit'
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    baseurl: '/config/fetchmail',
	    reference: 'removeBtn',
	    callback: 'onAfterRemove',
	    waitMsgTarget: true
	}
    ],

    listeners: {
	//scope: 'controller',
	itemdblclick: 'onEdit'
    },

    columns: [
	{
	    header: gettext('Server'),
	    flex: 1,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'server'
	},
	{
	    header: gettext('Protocol'),
	    dataIndex: 'protocol'
	},
	{
	    header: gettext('User name'),
	    flex: 1,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'user'
	},
	{
	    header: gettext('Deliver to'),
	    flex: 1,
	    renderer: Ext.String.htmlEncode,
	    dataIndex: 'target'
	},
	{
	    header: gettext('Enabled'),
	    sortable: true,
	    renderer: Proxmox.Utils.format_boolean,
	    dataIndex: 'enable'
	},
	{
	    header: gettext('Interval'),
	    dataIndex: 'interval'
	}
     ]
});
Ext.define('PMG.UserManagement', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pmgUserManagement',

    title: gettext('Configuration') + ': ' +
	gettext('User Management'),

    border: false,
    defaults: { border: false },

    items: [
	{
	    title: gettext('Local'),
	    itemId: 'local',
	    xtype: "pmgUserView"
	},
	{
	    title: 'LDAP',
	    itemId: 'ldap',
	    xtype: 'pmgLDAPConfig'
	},
	{
	    title: 'Fetchmail',
	    itemId: 'pop',
	    xtype: 'pmgFetchmailView'
	}
    ]
});


/*global Proxmox*/
Ext.define('PMG.ViewMailHeaders', {
    extend: 'Ext.window.Window',
    alias: 'widget.pmgViewMailHeaders',

    url: undefined,

    width: 600,

    height: 400,

    bodyPadding: 10,

    modal: true,

    layout: {
	type: 'vbox',
	align: 'stretch'
    },

    controller: {

	xclass: 'Ext.app.ViewController',

	init: function(view) {

	    var panel = view.lookupReference('contentPanel');
	    var fromField =
	    Proxmox.Utils.API2Request({
		url: view.url,
		waitMsgTarget: view,
		method: 'GET',
		success: function(response, opts) {
		    var data = response.result.data;
		    var from = data.match(/^FROM:\s*(.*\S)\s*$/mi);
		    if (from) {
			view.lookupReference('fromField').setValue(from[1]);
		    }
		    var to = data.match(/^TO:\s*(.*\S)\s*$/mi);
		    if (to) {
			view.lookupReference('toField').setValue(to[1]);
		    }
		    var subject = data.match(/^SUBJECT:\s*(.*\S)\s*$/mi);
		    if (subject) {
			view.lookupReference('subjectField').setValue(subject[1]);
		    }
		    panel.update(Ext.String.htmlEncode(data));
		},
		failure: function (response, opts) {
		    view.destroy();
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });

	}
    },

    items: [
	{
	    xtype: 'textfield',
	    fieldLabel: gettext('From'),
	    reference: 'fromField',
	    focusable: false,
	    exitable: false
	},
	{
	    xtype: 'textfield',
	    fieldLabel: gettext('To'),
	    reference: 'toField',
	    focusable: false,
	    exitable: false
	},
	{
	    xtype: 'textfield',
	    fieldLabel: gettext('Subject'),
	    reference: 'subjectField',
	    focusable: false,
	    exitable: false
	},
	{
	    xtype: 'displayfield',
	    fieldLabel: gettext('Header')
	},
	{
	    xtype: 'panel',
	    bodyPadding: 5,
	    reference: 'contentPanel',
	    flex: 1,
	    autoScroll: true,
	    bodyStyle: 'white-space:pre'
	}
    ]
});
/*global Proxmox*/
Ext.define('pmg-qshape', {
    extend: 'Ext.data.Model',
    fields: [
	'domain',
	{ type: 'integer', name: 'total'},
	{ type: 'integer', name: '5s'},
	{ type: 'integer', name: '10s'},
	{ type: 'integer', name: '20s'},
	{ type: 'integer', name: '40s'},
	{ type: 'integer', name: '80s'},
	{ type: 'integer', name: '160s'},
	{ type: 'integer', name: '320s'},
	{ type: 'integer', name: '640s'},
	{ type: 'integer', name: '1280s'},
	{ type: 'integer', name: '1280s+'}
    ],
    idProperty: 'domain'
});

Ext.define('PMG.Postfix.QShape', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgPostfixQShape',

    nodename : undefined,

    store: {
	autoLoad: true,
	model: 'pmg-qshape'
    },

    controller: {

	xclass: 'Ext.app.ViewController',

	init: function(view) {
	    if (view.nodename) {
		view.setNodename(view.nodename);
	    }
	},

	onFlush: function() {
	    var view = this.getView();

	    Proxmox.Utils.API2Request({
		url: '/api2/extjs/nodes/' + view.nodename + '/postfix/flush_queues',
		method: 'POST',
		waitMsgTarget: view,
		success: function(response, opts) {
		    view.store.load();
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	onDeleteAll: function() {
	    var view = this.getView();

	    Proxmox.Utils.API2Request({
		url: '/api2/extjs/nodes/' + view.nodename + '/postfix/queue/deferred',
		method: 'DELETE',
		waitMsgTarget: view,
		success: function(response, opts) {
		    view.store.load();
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	onDiscardVerifyDatabase: function() {
	    var view = this.getView();

	    Proxmox.Utils.API2Request({
		url: '/api2/extjs/nodes/' + view.nodename + '/postfix/discard_verify_cache',
		method: 'POST',
		waitMsgTarget: view,
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	control: {
	    '#': {
		activate: function() {
		    this.view.store.load(); // reload
		}
	    }
	}
    },

    tbar: [
        {
	    text: gettext('Flush Queue'),
	    handler: 'onFlush'
	},
	{
	    xtype: 'proxmoxButton',
	    text: gettext('Delete all Messages'),
	    dangerous: true,
	    confirmMsg: "Are you sure you want to delete all deferred mails?",
	    selModel: null,
	    handler: 'onDeleteAll'
	},
	{
	    text: gettext('Discard address verification database'),
	    handler: 'onDiscardVerifyDatabase'
	}
    ],

    columns: [
	{
	    header: gettext('Domain'),
	    width: 200,
	    dataIndex: 'domain'
	},
	{
	    header: gettext('Total'),
	    width: 80,
	    dataIndex: 'total'
	},
	{
	    header: '5s',
	    width: 80,
	    dataIndex: '5s'
	},
	{
	    header: '10s',
	    width: 80,
	    dataIndex: '10s'
	},
	{
	    header: '20s',
	    width: 80,
	    dataIndex: '20s'
	},
	{
	    header: '40s',
	    width: 80,
	    dataIndex: '40s'
	},
	{
	    header: '80s',
	    width: 80,
	    dataIndex: '80s'
	},
	{
	    header: '160s',
	    width: 80,
	    dataIndex: '160s'
	},
	{
	    header: '320s',
	    width: 80,
	    dataIndex: '320s'
	},
	{
	    header: '640s',
	    width: 80,
	    dataIndex: '640s'
	},
	{
	    header: '1280s',
	    width: 80,
	    dataIndex: '1280s'
	},
	{
	    header: '1280s+',
	    width: 80,
	    dataIndex: '1280s+'
	}
    ],

    setNodename: function(nodename) {
	var me = this;

	me.nodename = nodename;

	me.store.setProxy({
	    type: 'proxmox',
	    url: "/api2/json/nodes/" + me.nodename + "/postfix/qshape"
	});

	me.store.load();
    }

});
/*global Proxmox*/
Ext.define('pmg-mailq', {
    extend: 'Ext.data.Model',
    fields: [
	'queue_id', 'queue_name',
	{ type: 'date',  dateFormat: 'timestamp', name: 'arrival_time'},
	{ type: 'integer', name: 'message_size'},
	'sender', 'receiver', 'reason'
    ],
    idProperty: 'queue_id'
});

Ext.define('PMG.Postfix.MailQueue', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgPostfixMailQueue',

    nodename: undefined,

    filter: undefined,

    queuename: 'deferred',

    store: {
	xclass: 'Ext.data.BufferedStore',
	model: 'pmg-mailq',
	remoteFilter: true,
	pageSize: 2000
    },

    controller: {

	xclass: 'Ext.app.ViewController',

	init: function(view) {
	    if (view.nodename) {
		view.setNodename(view.nodename);
	    }

	    view.delayFilterTask = new Ext.util.DelayedTask(function() {
		var filter = view.lookupReference('filter').getValue();

		view.filter = filter;
		view.updateProxy();
	    });
	},

	onChangeFilter: function(f, v) {
	    var view = this.getView();
	    view.delayFilterTask.delay(500);
	},

	onFlush: function(button, event, rec) {
	    var view = this.getView();

	    Proxmox.Utils.API2Request({
		url: '/api2/extjs/nodes/' + view.nodename + '/postfix/queue/' +
		    view.queuename + '/' + rec.data.queue_id,
		method: 'POST',
		waitMsgTarget: view,
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });

	},

	onRemove: function(button, event, rec) {
	    var view = this.getView();

	    Proxmox.Utils.API2Request({
		url: '/api2/extjs/nodes/' + view.nodename + '/postfix/queue/' +
		    view.queuename + '/' + rec.data.queue_id,
		method: 'DELETE',
		waitMsgTarget: view,
		success: function(response, opts) {
		    view.selModel.deselectAll();
		    view.store.load();
		},
		failure: function (response, opts) {
		    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
		}
	    });
	},

	onHeaders: function(button, event, rec) {
	    var view = this.getView();

	    var url = '/api2/extjs/nodes/' + view.nodename + '/postfix/queue/' +
		view.queuename + '/' + rec.data.queue_id;

	    var win = Ext.create('PMG.ViewMailHeaders', {
		title: view.title + ' : ' + rec.data.queue_id,
		url: url
	    });
	    win.show();
	},

	control: {
	    '#': {
		activate: function() {
		    this.view.updateProxy(); // reload
		},
		itemdblclick: function(grid, rec, item, index, event) {
		    this.onHeaders(grid, event, rec);
		}
	    },
	    'field[reference=filter]': {
		change: 'onChangeFilter'
	    }
	}
    },

    tbar: [
	{
	    xtype: 'proxmoxButton',
	    disabled: true,
	    text: gettext('Headers'),
	    handler: 'onHeaders'
	},
	{
	    xtype: 'proxmoxButton',
	    disabled: true,
	    text: gettext('Flush'),
	    handler: 'onFlush'
	},
	{
	    xtype: 'proxmoxStdRemoveButton',
	    handler: 'onRemove'
	},
	{
	    xtype: 'label',
	    html: gettext('Filter') + ':'
	},
	{
	    xtype: 'textfield',
	    width: 300,
	    reference: 'filter'
	}
    ],

    columns: [
	{
	    header: gettext('Time'),
	    width: 150,
	    renderer: Ext.util.Format.dateRenderer("Y-m-d H:i:s"),
	    dataIndex: 'arrival_time'
	},
	{
	    header: 'KByte',
	    width: 80,
	    dataIndex: 'message_size'
	},
	{
	    header: gettext('Sender'),
	    width: 200,
	    dataIndex: 'sender'
	},
	{
	    header: gettext('Receiver'),
	    width: 200,
	    dataIndex: 'receiver'
	},
	{
	    header: gettext('Reason'),
	    flex: 1,
	    dataIndex: 'reason'
	}
    ],

    pendingLoad: false,

    updateProxy: function() {
	var me = this;

	if (me.pendingLoad) {
	    return;
	}

	var proxy = {
	    type: 'proxmox',
	    startParam: 'start',
	    limitParam: 'limit',
	    url: "/api2/json/nodes/" + me.nodename + "/postfix/queue/" + me.queuename
	};

	var filter = me.filter;
	var nodename = me.nodename;
	var queuename = me.queuename;

	if (filter) { proxy.extraParams = { filter: filter }; }


	me.store.setProxy(proxy);

	me.pendingLoad = true;

	me.store.load(function() {
	    me.pendingLoad = false;
	    if (me.nodename !== nodename || me.filter !== filter || me.queuename !== queuename) {
		setTimeout(function() {
		    me.updateProxy();
		}, 100);
	    }
	});
    },

    setFilter: function(filter) {
	this.lookupReference('filter').setValue(filter);
    },

    setNodename: function(nodename) {
	var me = this;

	me.nodename = nodename;

	me.updateProxy();
    },

    setQueueName: function(queuename) {
	var me = this;

	me.queuename = queuename;

	me.updateProxy();
    }

});
/*global Proxmox*/
Ext.define('PMG.QueueAdministration', {
    extend: 'Ext.tab.Panel',
    alias: 'widget.pmgQueueAdministration',

    title: gettext('Queue Administration'),

    border: false,
    defaults: { border: false },

    controller: {

	xclass: 'Ext.app.ViewController',

	onSelect: function(grid, rec) {
	    var view = this.getView();

	    var domain = rec.data.domain;

	    var mailq = this.lookupReference('mailq');

	    if (domain === "TOTAL") {
		mailq.setFilter('');
	    } else {
		mailq.setFilter('@' + rec.data.domain);
	    }

	    view.setActiveTab(mailq);
	},

	control: {
	    pmgPostfixQShape: {
		itemdblclick: 'onSelect'
	    }
	}
    },

    items: [
	{
	    title: gettext('Summary'),
	    nodename: Proxmox.NodeName,
	    itemId: 'qshape',
	    xtype: 'pmgPostfixQShape'
	},
	{
	    title: gettext('Deferred Mail'),
	    nodename: Proxmox.NodeName,
	    reference: 'mailq',
	    itemId: 'mailqueue',
	    xtype: 'pmgPostfixMailQueue'
	}
    ]
});
/*global Proxmox*/
Ext.define('pmg-cluster', {
    extend: 'Ext.data.Model',
    fields: [
	'type', 'name', 'ip', 'hostrsapubkey', 'rootrsapubkey',
	'fingerprint', { type: 'integer', name: 'cid' },
	{ type: 'boolean', name: 'insync' },
	'memory', 'loadavg', 'uptime', 'rootfs', 'conn_error', 'level',
	{ type: 'number', name: 'memory_per',
	  calculate: function(data) {
	      var mem = data.memory;
	      return Ext.isObject(mem) ? mem.used/mem.total : 0;
	  }
	},
	{ type: 'number', name: 'rootfs_per',
	  calculate: function(data) {
	      var du = data.rootfs;
	      return Ext.isObject(du) ? du.used/du.total : 0;
	  }
	}
    ],
    proxy: {
        type: 'proxmox',
	url: "/api2/json/config/cluster/status"
    },
    idProperty: 'cid'
});

Ext.define('PMG.ClusterJoinNodeWindow', {
    extend: 'Proxmox.window.Edit',
    xtype: 'pmgClusterJoinNodeWindow',

    title: gettext('Cluster Join'),

    width: 800,

    method: 'POST',

    url: '/config/cluster/join',

    items: [
	{
	    xtype: 'textfield',
	    fieldLabel: 'IP Address',
	    name: 'master_ip'
	},
	{
	    xtype: 'textfield',
	    inputType: 'password',
	    fieldLabel: gettext('Password'),
	    name: 'password'
	},
	{
	    xtype: 'textfield',
	    fieldLabel: gettext('Fingerprint'),
	    name: 'fingerprint'
	}
    ]
});

Ext.define('PMG.ClusterAddNodeWindow', {
    extend: 'Ext.window.Window',
    xtype: 'pmgClusterAddNodeWindow',
    mixins: ['Proxmox.Mixin.CBind'],

    width: 800,

    modal: true,

    title: gettext('Cluster Join') + ' : ' + gettext('Information'),

    ipAddress: undefined,

    fingerprint: undefined,

    items: [
	{
	    xtype: 'component',
	    border: false,
	    padding: '10 10 10 10',
	    html: gettext("Please use the 'Join' button on the node you want to add, using the following IP address and fingerprint.")
	},
	{
	    xtype: 'container',
	    layout: 'form',
	    border: false,
	    padding: '0 10 10 10',
	    items: [
		{
		    xtype: 'textfield',
		    fieldLabel: gettext('IP Address'),
		    cbind: { value: '{ipAddress}' },
		    editable: false
		},
		{
		    xtype: 'textfield',
		    fieldLabel: gettext('Fingerprint'),
		    cbind: { value: '{fingerprint}' },
		    editable: false
		}
	    ]
	}
    ]
});

/*jslint confusion: true*/
/* bind is a function and object */
Ext.define('PMG.ClusterAdministration', {
    extend: 'Ext.tab.Panel',
    xtype: 'pmgClusterAdministration',

    title: gettext('Cluster Administration'),

    border: false,
    defaults: { border: false },

    viewModel: {
	parent: null,
	data: {
	    nodecount: 0,
	    master: null
	}
    },

    items: [
	{
	    xtype: 'grid',
	    title: gettext('Nodes'),
	    controller: {
		xclass: 'Ext.app.ViewController',

		init: function(view) {
		    view.store.on('load', this.onLoad, this);
		    Proxmox.Utils.monStoreErrors(view, view.getStore());
		},

		onLoad: function(store, records, success) {
		    var vm = this.getViewModel();
		    if (!success || !records) {
			return;
		    }
		    vm.set('nodecount', records.length);

		    var master = null;
		    Ext.Array.each(records, function(ni) {
			if (ni.data.type === 'master') {
			    master = ni;
			}
		    });
		    vm.set('master', master);
		},

		onCreate: function() {
		    var view = this.getView();

		    Proxmox.Utils.API2Request({
			url: '/config/cluster/create',
			method: 'POST',
			waitMsgTarget: view,
			failure: function (response, opts) {
			    Ext.Msg.alert(gettext('Error'), response.htmlStatus);
			},
			success: function(response, options) {
			    var upid = response.result.data;
			    var win = Ext.create('Proxmox.window.TaskProgress', { upid: upid });
			    win.show();
			    win.on('destroy', function() { view.store.load(); });
			}
		    });
		},

		onJoin: function() {
		    var view = this.getView();
		    var win = Ext.create('PMG.ClusterJoinNodeWindow', {});
		    win.show();
		    win.on('destroy', function() {
			// fixme: logout
		    });
		},

		onAdd: function() {
		    var vm = this.getViewModel();

		    var win = Ext.create('PMG.ClusterAddNodeWindow', {
			ipAddress: vm.get('master').get('ip'),
			fingerprint: vm.get('master').get('fingerprint')
		    });

		    win.show();
		}
	    },
	    store: {
		autoLoad: true,
		model: 'pmg-cluster'
	    },
	    tbar: [
		{
		    text: gettext('Create'),
		    reference: 'createButton',
		    handler: 'onCreate',
		    bind: {
			disabled: '{nodecount}'
		    }
		},
		{
		    text: gettext('Add'),
		    reference: 'addButton',
		    handler: 'onAdd',
		    bind: {
			disabled: '{!master}'
		    }
		},
		{
		    text: gettext('Join'),
		    reference: 'joinButton',
		    handler: 'onJoin',
		    bind: {
			disabled: '{nodecount}'
		    }
		}
	    ],
	    columns: [
		{
		    header: gettext('Node'),
		    width: 150,
		    dataIndex: 'name'
		},
		{
		    header: gettext('Role'),
		    width: 100,
		    dataIndex: 'type'
		},
		{
		    header: gettext('ID'),
		    width: 80,
		    dataIndex: 'cid'
		},
		{

		    header: gettext('IP'),
		    width: 150,
		    dataIndex: 'ip'
		},
		{
		    header: gettext('State'),
		    width: 100,
		    renderer: function(value, metaData, record) {
			var d = record.data;
			var state = 'active';
			if (!d.insync) {
			    state = 'syncing';
			}
			if (d.conn_error) {
			    metaData.tdCls = 'x-form-invalid-field';
			    var html = '<p>' +  Ext.htmlEncode(d.conn_error) + '</p>';
			    html = html.replace(/\n/g, '<br>');
			    metaData.tdAttr = 'data-qwidth=600 data-qtitle="ERROR" data-qtip="' +
				html.replace(/\"/g,'&quot;') + '"';
			    state = 'error';
			}
			return state;
		    },
		    dataIndex: 'insync'
		},
		{
		    header: gettext('Subscription'),
		    width: 120,
		    renderer: Proxmox.Utils.format_subscription_level,
		    dataIndex: 'level'
		},
		{
		    header: gettext('Uptime'),
		    width: 150,
		    renderer: Proxmox.Utils.render_uptime,
		    dataIndex: 'uptime'
		},
		{
		    header: gettext('Load average'),
		    renderer: function(value) {
			if (Ext.isArray(value)) {
			    return value[0];
			}
			return value.toString();
		    },
		    dataIndex: 'loadavg'
		},
		{
		    xtype: 'widgetcolumn',
		    widget: {
			xtype: 'progressbarwidget',
			textTpl: '{value:percent}'
		    },
		    header: gettext('RAM usage'),
		    dataIndex: 'memory_per'
		},
		{
		    xtype: 'widgetcolumn',
		    widget: {
			xtype: 'progressbarwidget',
			textTpl: '{value:percent}'
		    },
		    header: gettext('HD space'),
		    dataIndex: 'rootfs_per'
		}
	    ]
	}
    ]
});
/*
 * Base class for all the multitab config panels
 *
 * How to use this:
 *
 * You create a subclass of this, and then define your wanted tabs
 * as items like this:
 *
 * items: [{
 *  title: "myTitle",
 *  xytpe: "somextype",
 *  iconCls: 'fa fa-icon',
 *  groups: ['somegroup'],
 *  expandedOnInit: true,
 *  itemId: 'someId'
 * }]
 *
 * this has to be in the declarative syntax, else we
 * cannot save them for later
 * (so no Ext.create or Ext.apply of an item in the subclass)
 *
 * the groups array expects the itemids of the items
 * which are the parents, which have to come before they
 * are used
 *
 * if you want following the tree:
 *
 * Option1
 * Option2
 *   -> SubOption1
 *	-> SubSubOption1
 *
 * the suboption1 group array has to look like this:
 * groups: ['itemid-of-option2']
 *
 * and of subsuboption1:
 * groups: ['itemid-of-option2', 'itemid-of-suboption1']
 *
 * setting the expandedOnInit determines if the item/group is expanded
 * initially (false by default)
 */
Ext.define('PMG.panel.Config', {
    extend: 'Ext.panel.Panel',
    alias: 'widget.pmgPanelConfig',

    viewFilter: undefined, // a filter to pass to that ressource grid

    dockedItems: [{
	// this is needed for the overflow handler
	xtype: 'toolbar',
	overflowHandler: 'scroller',
	dock: 'left',
	style: {
	    backgroundColor: '#f5f5f5',
	    padding: 0,
	    margin: 0
	},
	items: {
	    xtype: 'treelist',
	    itemId: 'menu',
	    ui: 'nav',
	    expanderOnly: true,
	    expanderFirst: false,
	    animation: false,
	    singleExpand: false,
	    listeners: {
		selectionchange: function(treeList, selection) {
		    var me = this.up('panel');
		    me.suspendLayout = true;
		    me.activateCard(selection.data.id);
		    me.suspendLayout = false;
		    me.updateLayout();
		},
		itemclick: function(treelist, info) {
		    var olditem = treelist.getSelection();
		    var newitem = info.node;

		    // when clicking on the expand arrow,
		    // we dont select items, but still want
		    // the original behaviour
		    if (info.select === false) {
			return;
		    }

		    // if you click on a different item which is open,
		    // leave it open
		    // else toggle the clicked item
		    if (olditem.data.id !== newitem.data.id &&
			newitem.data.expanded === true) {
			info.toggle = false;
		    } else {
			info.toggle = true;
		    }
		}
	    }
	}
    }],

    firstItem: '',
    layout: 'card',
    border: 0,

    // used for automated test
    selectById: function(cardid) {
	var me = this;

	var root = me.store.getRoot();
	var selection = root.findChild('id', cardid, true);

	if (selection) {
	    selection.expand();
	    var menu = me.down('#menu');
	    menu.setSelection(selection);
	    return cardid;
	}
    },

    activateCard: function(cardid) {
	var me = this;
	if (me.savedItems[cardid]) {
	    var curcard = me.getLayout().getActiveItem();
	    var newcard = me.add(me.savedItems[cardid]);
	    if (curcard) {
		me.setActiveItem(cardid);
		me.remove(curcard, true);

		// trigger state change

		var ncard = cardid;
		// Note: '' is alias for first tab.
		// First tab can be 'search' or something else
		if (cardid === me.firstItem) {
		    ncard = '';
		}
		if (me.hstateid) {
		   me.sp.set(me.hstateid, { value: ncard });
		}
	    }
	}
    },

    initComponent: function() {
        var me = this;

	var stateid = me.hstateid;

	me.sp = Ext.state.Manager.getProvider();

	var activeTab; // leaving this undefined means items[0] will be the default tab

	if (stateid) {
	    var state = me.sp.get(stateid);
	    if (state && state.value) {
		// if this tab does not exists, it chooses the first
		activeTab = state.value;
	    }
	}

	// get title
	var title = me.title;
	me.title = undefined;

	// create toolbar
	//var tbar = me.tbar || [];
	//me.tbar = undefined;

	//tbar.unshift('->');
	//tbar.unshift({
	//    xtype: 'tbtext',
	//    text: title,
	//    baseCls: 'x-panel-header-text'
	//});

	//me.dockedItems[1].items = tbar;

	// include search tab
	me.items = me.items || [];

	me.savedItems = {};
	/*jslint confusion:true*/
	if (me.items[0]) {
	    me.firstItem = me.items[0].itemId;
	}
	/*jslint confusion:false*/

	me.store = Ext.create('Ext.data.TreeStore', {
	    root: {
		expanded: true
	    }
	});
	var root = me.store.getRoot();
	me.items.forEach(function(item){
	    var treeitem = Ext.create('Ext.data.TreeModel',{
		id: item.itemId,
		text: item.title,
		iconCls: item.iconCls,
		leaf: true,
		expanded: item.expandedOnInit
	    });
	    item.header = false;
	    if (me.savedItems[item.itemId] !== undefined) {
		throw "itemId already exists, please use another";
	    }
	    me.savedItems[item.itemId] = item;

	    var group;
	    var curnode = root;

	    // get/create the group items
	    while (Ext.isArray(item.groups) && item.groups.length > 0) {
		group = item.groups.shift();

		var child = curnode.findChild('id', group);
		if (child === null) {
		    // did not find the group item
		    // so add it where we are
		    break;
		}
		curnode = child;
	    }

	    // insert the item

	    // lets see if it already exists
	    var node = curnode.findChild('id', item.itemId);

	    if (node === null) {
		curnode.appendChild(treeitem);
	    } else {
		// should not happen!
		throw "id already exists";
	    }
	});

	delete me.items;
	me.defaults = me.defaults || {};
	Ext.apply(me.defaults, {
	    viewFilter: me.viewFilter,
	    border: 0
	});

	me.callParent();

	var menu = me.down('#menu');
	var selection = root.findChild('id', activeTab, true) || root.firstChild;
	var node = selection;
	while (node !== root) {
	    node.expand();
	    node = node.parentNode;
	}
	menu.setStore(me.store);
	menu.setSelection(selection);

	// on a state change,
	// select the new item
	var statechange = function(sp, key, state) {
	    // it the state change is for this panel
	    if (stateid && (key === stateid) && state) {
		// get active item
		var acard = me.getLayout().getActiveItem().itemId;
		// get the itemid of the new value
		var ncard = state.value || me.firstItem;
		if (ncard && (acard != ncard)) {
		    // select the chosen item
		    menu.setSelection(root.findChild('id', ncard, true) || root.firstChild);
		}
	    }
	};

	if (stateid) {
	    me.mon(me.sp, 'statechange', statechange);
	}
    }
});
Ext.define('PMG.StatTimeSelector', {
    extend: 'Ext.container.Container',
    xtype: 'pmgStatTimeSelector',

    statics: {
	selected_year: undefined,
	selected_month: undefined,
	selected_day: undefined,

	getTimeSpan: function() {
	    var year = this.selected_year;
	    var month = this.selected_month;
	    var day = this.selected_day;

	    var starttime, endtime, span;
	
	    if (!month) {
		starttime = new Date(year, 0);
		endtime = new Date(year + 1, 0);
	    } else if (!day) {
		starttime = new Date(year, month - 1);
		endtime = new Date(year, month);
	    } else {
		starttime = new Date(year, month - 1, day);
		endtime = new Date(year, month - 1, day + 1);
	    }

	    var data = {};

	    data.starttime = (starttime.getTime() / 1000).toFixed(0);
	    data.endtime = (endtime.getTime() / 1000).toFixed(0);
	
	    return data;
	}
    },
    
    layout: {
	type: 'hbox'
    },
    
    controller: {
	xclass: 'Ext.app.ViewController',

	updateVisibility: function() {
	    var view = this.getView();
	    
	    var yearsel = this.lookupReference('yearsel');
	    var monthsel = this.lookupReference('monthsel');
	    var daysel = this.lookupReference('daysel');

	    var year = yearsel.getValue();
	    var month = monthsel.getValue();
	    daysel.setVisible(month !== 0);
	    if (!month) {
		daysel.setValue(0);
	    }
	    var day = daysel.getValue();

	    var statics = Ext.getClass(view);

	    statics.selected_year = year;
	    statics.selected_month = month;
	    statics.selected_day = day;

	    var data = statics.getTimeSpan();
	    Ext.GlobalEvents.fireEvent('pmgStatTimeSelectorUpdate', data);
	},
	
	onSelect: function() {
	    this.updateVisibility();
	},
	
	init: function(view) {
	    var statics = Ext.getClass(view);
	    
	    var yearsel = this.lookupReference('yearsel');
	    var monthsel = this.lookupReference('monthsel');
	    var daysel = this.lookupReference('daysel');
	    
	    yearsel.setValue(statics.selected_year);
	    monthsel.setValue(statics.selected_month);
	    daysel.setValue(statics.selected_month ? statics.selected_day : 0);

	    this.updateVisibility();
	}
    },
    
    items: [
	{
	    xtype: 'combobox',
	    reference: 'yearsel',
	    store: {
		fields: [ 'year' ],
		data: (function() {
		    var today = new Date();
		    var year = today.getFullYear();
		    return [{ year: year }, { year: year -1 },{ year: year -2 }];
		}())
	    },
	    listeners: { select: 'onSelect' },
	    value: (new Date()).getFullYear(),
	    queryMode: 'local',
	    displayField: 'year',
	    editable: false,
	    valueField: 'year'
	},
	{
	    xtype: 'combobox',
	    reference: 'monthsel',
	    store: {
		fields: [ 'month', 'name' ],
		data: (function() {
		    var i;
		    var data = [{ month: 0, name: gettext('Whole year') }];
		    for (i = 1; i <= 12; i++) {
			data.push({ month: i, name: Ext.Date.monthNames[i-1]});
		    }
		    return data;
		}())
	    },
	    listeners: { select: 'onSelect' },
	    queryMode: 'local',
	    displayField: 'name',
	    editable: false,
	    valueField: 'month'
	},
	{
	    xtype: 'combobox',
	    reference: 'daysel',
	    store: {
		fields: [ 'day', 'name' ],
		data: (function() {
		    var i;
		    var data = [{ day: 0, name: gettext('Whole month') }];
		    for (i = 1; i <= 31; i++) {
			data.push({ day: i, name: i});
		    }
		    return data;
		}())
	    },
	    listeners: { select: 'onSelect' },
	    queryMode: 'local',
	    displayField: 'name',
	    editable: false,
	    valueField: 'day'
	}
    ]
}, function() {

    var today = new Date();
	    
    this.selected_year = today.getFullYear();
    this.selected_month = today.getMonth() + 1;
    this.selected_day = today.getDate();

});
Ext.define('PMG.data.StatStore', {
    extend: 'Ext.data.Store',
    alias: 'store.pmgStatStore',

    autoDestroy: true,

    staturl: undefined,

    includeTimeSpan: false,

    setUrl: function(url) {
	var me = this;

	me.proxy.abort(); // abort pending requests

	me.staturl = url;
	me.proxy.extraParams = {};
	me.setData([]);
    },

    reload: function() {
	var me = this;

	me.proxy.abort(); // abort pending requests

	if (me.staturl === undefined) {
	    me.proxy.extraParams = {};
	    me.setData([]);
	    return;
	}

	var ts = PMG.StatTimeSelector.getTimeSpan();

	var last = me.proxy.extraParams;

	if (last.starttime === ts.starttime && last.endtime === ts.endtime) {
	    return; // avoid repeated loads
	}

	me.proxy.url = me.staturl;
	me.proxy.extraParams = { starttime: ts.starttime, endtime: ts.endtime };

	var timespan = 3600;
	if (me.includeTimeSpan) {
	    var period = ts.endtime - ts.starttime;
	    if (period <= 86400*7) {
		timespan = 3600;
	    } else {
		timespan = 3600*24;
	    }
	    me.proxy.extraParams.timespan = timespan;
	}

	me.load();
    },

    proxy: {
	type: 'proxmox'
    },

    autoReload: true,

    constructor: function(config) {
	var me = this;

	config = config || {};

	me.mon(Ext.GlobalEvents, 'pmgStatTimeSelectorUpdate', function() {
	    if (me.autoReload) {
		me.reload();
	    }
	}, me);

	me.callParent([config]);

	me.reload();
   }
});
Ext.define('PMG.MailStatGrid', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgMailStatGrid',
	    
    disableSelection: true,
    hideHeaders: true,

    store: {
	fields: [ 'name', 'value', 'percentage' ]
    },

    columns: [
	{
	    flex: 1,
	    dataIndex: 'name'
	},
	{
	    width: 150,
	    dataIndex: 'value'
	},
	{
	    width: 300,
	    
	    xtype: 'widgetcolumn',
	    dataIndex: 'percentage',
	    widget: {
		xtype: 'progressbarwidget',
		textTpl: ['{percent:number("0")}%' ]
	    },

	    onWidgetAttach: function (column, widget, rec) {
		if (rec.data.percentage === undefined) {
		    widget.setStyle("visibility: hidden");
		} else {
		    widget.setStyle("visibility: visible");
		}
	    }
	}
    ]
});
/*global Proxmox*/
Ext.define('PMG.VirusCharts', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgVirusCharts',

    title: gettext('Statistics') + ': ' + gettext('Virus Charts'),
    
    border: false,
    disableSelection: true,

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    emptyText: gettext('No data in database.'),
    viewConfig: {
	deferEmptyText: false
    },

    store: {
	xclass: 'PMG.data.StatStore',
	fields: [ 'name', 'count' ],
	staturl: "/api2/json/statistics/virus"
    },

    columns: [
	{
	    header: gettext('Name'),
	    flex: 1,
	    dataIndex: 'name'
	},
	{
	    header: gettext('Count'),
	    width: 150,
	    dataIndex: 'count'
	}
    ],

    initComponent: function() {
	var me = this;
	
	me.callParent();
	
	Proxmox.Utils.monStoreErrors(me, me.store);
    }
});
Ext.define('PMG.SpamScoreDistribution', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgSpamScoreDistribution',

    disableSelection: true,
    border: false,

    title: gettext('Statistics') + ': ' + gettext('Spam Scores'),

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    store: {
	xclass: 'PMG.data.StatStore',
	staturl: "/api2/json/statistics/spamscores",
	fields: [
	    'count', 'ratio',
	    {
		type: 'string',
		name: 'label',
		convert: function(v, rec) {
		    if (rec.data.level >= 10) {
			return PMG.Utils.scoreText + ' >= 10';
		    } else {
			return PMG.Utils.scoreText + ' ' + rec.data.level.toString();
		    }
		}
	    }
	]
    },

    columns: [
	{
	    header: PMG.Utils.scoreText,
	    flex: 1,
	    dataIndex: 'label'
	},
	{
	    header: gettext("Count"),
	    width: 150,
	    dataIndex: 'count'
	},
	{
	    header: gettext("Percentage"),
	    width: 300,

	    xtype: 'widgetcolumn',
	    dataIndex: 'ratio',
	    widget: {
		xtype: 'progressbarwidget',
		textTpl: ['{percent:number("0")}%' ]
	    }
	}
    ]
});
Ext.define('PMG.GeneralMailStatistics', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgGeneralMailStatistics',

    scrollable: true,
    
    bodyPadding: '10 0 0 0',
    border: false,
    defaults: {
	width: 700,
	padding: '0 0 10 10'
    },

    layout: 'column',

    title: gettext('Statistics') + ': ' + gettext('Mail'),

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    getInData: function(data) {
	var res = [];

	res.push({
	    name: gettext("Incoming Mails"),
	    value: data.count_in,
	    percentage: 1
	});
	    
	res.push({
	    name: gettext("Junk Mails"),
	    value: data.junk_in,
	    percentage: data.junk_in/data.count_in
	});
	    
	res.push({
	    name: gettext("Greylisted Mails"),
	    value: data.glcount,
	    percentage: data.glcount/data.count_in
	});

	res.push({
	    name: gettext("Spam Mails"),
	    value: data.spamcount_in,
	    percentage: data.spamcount_in/data.count_in
	});
	
	res.push({
	    name: gettext("SPF rejects"),
	    value: data.spfcount,
	    percentage: data.spfcount/data.count_in
	});

	res.push({
	    name: gettext("Bounces"),
	    value: data.bounces_in,
	    percentage: data.bounces_in/data.count_in
	});

	res.push({
	    name: gettext("Virus Mails"),
	    value: data.viruscount_in,
	    percentage: data.viruscount_in/data.count_in
	});

	return res;
    },
	    
   getOutData: function(data) {
	var res = [];

	res.push({
	    name: gettext("Outgoing Mails"),
	    value: data.count_out,
	    percentage: 1
	});
	    
	res.push({
	    name: gettext("Bounces"),
	    value: data.bounces_out,
	    percentage: data.bounces_out/data.count_out
	});

	res.push({
	    name: gettext("Virus Mails"),
	    value: data.viruscount_out,
	    percentage: data.viruscount_out/data.count_out
	});

	return res;
    },

    getGeneralData: function(data) {
	var res = [];
	
	res.push({
	    name: gettext("Total Mails"),
	    value: data.count,
	    percentage: 1
	});
	
	res.push({
	    name: gettext("Incoming Mails"),
	    value: data.count_in,
	    percentage: data.count_in/data.count
	});

	res.push({
	    name: gettext("Outgoing Mails"),
	    value: data.count_out,
	    percentage: data.count_out/data.count
	});
	    
	res.push({
	    name: gettext("Virus Outbreaks"),
	    value: data.viruscount_out
	});
	    
	res.push({
	    name: gettext("Avg. Mail Processing Time"),
	    value: Ext.String.format(gettext("{0} seconds"),
				     Ext.Number.toFixed(data.avptime, 2))
	});

	res.push({
	    name: gettext("Incoming Mail Traffic"),
	    value: Ext.Number.toFixed(data.bytes_in/(1024*1024), 2) + ' MiB'
	});

	res.push({
	    name: gettext("Outgoing Mail Traffic"),
	    value: Ext.Number.toFixed(data.bytes_out/(1024*1024), 2) + ' MiB'
	});
	return res;
    },

    initComponent: function() {
	var me = this;

	var countstore = Ext.create('PMG.data.StatStore', {
	    includeTimeSpan: true,
	    staturl: '/api2/json/statistics/mailcount',
	    fields: [
		{ type: 'integer', name: 'count' },
		{ type: 'integer', name: 'count_in' },
		{ type: 'integer', name: 'count_out' },
		{ type: 'integer', name: 'spamcount_in' },
		{ type: 'integer', name: 'spamcount_out' },
		{ type: 'integer', name: 'viruscount_in' },
		{ type: 'integer', name: 'viruscount_out' },
		{ type: 'integer', name: 'bounces_in' },
		{ type: 'integer', name: 'bounces_out' },
		{ type: 'date', dateFormat: 'timestamp', name: 'time' }
	    ]
	});
	
	var totalgrid = Ext.createWidget('pmgMailStatGrid', {
	    dockedItems: [
		{
		    dock: 'top',
		    title: gettext("Total Mail Count"),
		    xtype: 'proxmoxRRDChart',
		    fields: ['count', 'count_in', 'count_out'],
		    fieldTitles: [
			gettext('Total Mail Count'),
			gettext('Incoming Mails'), gettext('Outgoing Mails')],
		    store: countstore
		}
	    ]
	});
	
	var ingrid = Ext.createWidget('pmgMailStatGrid', {
	    dockedItems: [
		{
		    dock: 'top',
		    title: gettext("Incoming Mails"),
		    xtype: 'proxmoxRRDChart',
		    fields: ['count_in', 'spamcount_in', 'viruscount_in', 'bounces_in'],
		    fieldTitles: [
			gettext('Incoming Mails'), gettext('Junk Mails'),
			gettext('Virus Mails'), gettext('Bounces')],
		    store: countstore
		}
	    ]
	});
	
	var outgrid = Ext.createWidget('pmgMailStatGrid', {
	    dockedItems: [
		{
		    dock: 'top',
		    title: gettext("Outgoing Mails"),
		    xtype: 'proxmoxRRDChart',
		    fields: ['count_out', 'viruscount_out', 'bounces_out'],
		    fieldTitles: [
			gettext('Outgoing Mails'),
			gettext('Virus Mails'), gettext('Bounces')],
		    store: countstore
		}
	    ]
	});

	var infostore = Ext.create('PMG.data.StatStore', {
	    staturl: "/api2/json/statistics/mail",
	    fields: [ 'name', 'value', 'percentage' ],
	    listeners: {
		load: function(store, records) {
		    var data = me.getGeneralData(records[0].data);
		    totalgrid.store.setData(data);
		    data = me.getInData(records[0].data);
		    ingrid.store.setData(data);
		    data = me.getOutData(records[0].data);
		    outgrid.store.setData(data);
		}
	    }
	});

	me.items = [ totalgrid, ingrid, outgrid ];
	
	me.callParent();

	me.on('destroy', infostore.destroy, infostore);
	me.on('destroy', countstore.destroy, countstore);
    }
});
Ext.define('PMG.RBLStatistics', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgRBLStatistics',

    scrollable: true,
    border: false,

    bodyPadding: '10 0 10 10',

    title: gettext('Statistics') + ': ' + gettext('Postscreen'),

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    items: [
	{
	    title: gettext('Rejects'),
	    xtype: 'proxmoxRRDChart',
	    fields: [ 'rbl_rejects', 'pregreet_rejects'],
	    fieldTitles: ['RBL', 'PREGREET'],
	    store: {
		type: 'pmgStatStore',
		includeTimeSpan: true,
		staturl: "/api2/json/statistics/rejectcount",
		fields: [
		    { type: 'integer', name: 'rbl_rejects' },
		    { type: 'integer', name: 'pregreet_rejects' },
		    { type: 'date', dateFormat: 'timestamp', name: 'time' }
		]
	    }
	}
    ]
});
/*global Proxmox*/
Ext.define('PMG.DomainStatistics', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgDomainStatistics',

    title: gettext('Statistics') + ': ' + gettext('Domain'),

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    layout: 'fit',
    border: false,

    initComponent: function() {
	var me = this;

	var fields = [
	    'domain',
	    { type: 'integer', name: 'count_in' },
	    { type: 'integer', name: 'count_out' },
	    { type: 'integer', name: 'spamcount_in' },
	    { type: 'integer', name: 'spamcount_out' },
	    { type: 'integer', name: 'viruscount_in' },
	    { type: 'integer', name: 'viruscount_out' },
	    { type: 'number', name: 'bytes_in' },
	    { type: 'number', name: 'bytes_out' }
	];

	var store = Ext.create('PMG.data.StatStore', {
	    staturl: '/api2/json/statistics/domains',
	    fields: fields
	});

	var store_in = Ext.create('Ext.data.ArrayStore', {
	    fields: fields,
	    filters: [ function(item) { return item.data.count_in > 0; } ]
	});

	var store_out = Ext.create('Ext.data.ArrayStore', {
	    fields: fields,
	    filters: [ function(item) { return item.data.count_out > 0; } ]
	});

	store.on('load', function(s, records, successful) {
	    if (!successful) {
		store_in.setData([]);
		store_out.setData([]);
	    } else {
		store_in.setData(records);
		store_out.setData(records);
	    }
	});

	var render_domain = function(v) {
	    return v === '' ? '--- EMPTY ADDRESS ---' : Ext.htmlEncode(v);
	};

	me.items = [{
	    xtype: 'tabpanel',
	    border: false,
	    items: [
		{
		    xtype: 'grid',
		    title: gettext('Incoming'),
		    border: false,
		    disableSelection: true,
		    store: store_in,
		    emptyText: gettext('No data in database.'),
		    viewConfig: {
			deferEmptyText: false
		    },
		    columns: [
			{
			    text: gettext('Domain')+ ' (' +
				gettext('Receiver') + ')',
			    flex: 1,
			    renderer: render_domain,
			    dataIndex: 'domain'
			},
			{
			    text: gettext('Traffic') + ' (MB)',
			    dataIndex: 'bytes_in',
			    renderer: function(v) {
				return Ext.Number.toFixed(v/(1024*1024), 2);
			    }
			},
			{
			    text: gettext('Count'),
			    columns: [
				{
				    text: gettext('Mail'),
				    dataIndex: 'count_in'
				},
				{
				    header: gettext('Virus'),
				    dataIndex: 'viruscount_in'
				},
				{
				    header: gettext('Spam'),
				    dataIndex: 'spamcount_in'
				}
			    ]
			}
		    ]
		},
		{
		    xtype: 'grid',
		    title: gettext('Outgoing'),
		    border: false,
		    disableSelection: true,
		    store: store_out,
		    emptyText: gettext('No data in database.'),
		    viewConfig: {
			deferEmptyText: false
		    },
		    columns: [
			{
			    text: gettext('Domain')+ ' (' +
				gettext('Receiver') + ')',
			    flex: 1,
			    renderer: render_domain,
			    dataIndex: 'domain'
			},
			{
			    text: gettext('Traffic') + ' (MB)',
			    dataIndex: 'bytes_out',
			    renderer: function(v) {
				return Ext.Number.toFixed(v/(1024*1024), 2);
			    }
			},
			{
			    text: gettext('Count'),
			    columns: [
				{
				    text: gettext('Mail'),
				    dataIndex: 'count_out'
				},
				{
				    header: gettext('Virus'),
				    dataIndex: 'viruscount_out'
				}
			    ]
			}
		    ]
		}
	    ]
	}];

	me.callParent();

	Proxmox.Utils.monStoreErrors(me, store);

	me.on('destroy', store.destroy, store);
    }
});
/*global Proxmox*/
/*jslint confusion: true*/
/* filter is a function and object */
Ext.define('PMG.SenderDetails', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgSenderDetails',

    dockedItems: [
	{
	    dock: 'top',
	    xtype: 'panel',
	    itemId: 'info',
	    bodyPadding: 10,
	    html: gettext('Please select a sender.')
	}
    ],

    disableSelection: true,

    plugins: 'gridfilters',

    setUrl: function(url, title) {
	var me = this;

	me.store.setUrl(url);
	me.store.setRemoteFilter(url !== undefined);
	Proxmox.Utils.setErrorMask(me, false);
	me.store.reload();

	var infopanel = me.getComponent('info');
	if (title) {
	    infopanel.update(title);
	} else {
	    infopanel.update(gettext('Please select a sender.'));
	}
    },

    store: {
	type: 'pmgStatStore',
	autoReload: false,
	remoteSort: true,
	remoteFilter: false, // enabled dynamically
	fields: [
	    'receiver', 'virusinfo',
	    { type: 'integer', name: 'bytes' },
	    { type: 'boolean', name: 'blocked' },
	    { type: 'integer', name: 'spamlevel' },
            { type: 'date', dateFormat: 'timestamp', name: 'time' }
	],
	proxy: {
	    type: 'pmgfilterproxy',
	    filterId: 'x-gridfilter-receiver',
	    sortParam: 'orderby'
	},
	sorters: [
	    {
		property: 'time',
		direction: 'ASC'
	    }
	]
    },

    columns: [
	{
	    text: gettext('Receiver'),
	    renderer: Ext.htmlEncode,
	    flex: 1,
	    filter: { type: 'string' },
	    dataIndex: 'receiver'
	},
	{
	    header: gettext('Size') + ' (KB)',
	    renderer: function(v) { return Ext.Number.toFixed(v/1024, 0); },
	    dataIndex: 'bytes'
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Date'),
	    format: 'Y-m-d',
	    dataIndex: 'time'
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Time'),
	    format: 'H:i:s',
	    dataIndex: 'time'
	}
    ],

    initComponent: function() {
	var me = this;
	me.callParent();

	Proxmox.Utils.monStoreErrors(me, me.store, true);
    }
});

Ext.define('PMG.SenderList', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgSenderList',

    title: gettext('Statistics') + ': ' + gettext('Sender') +
	' (' + gettext('Outgoing') +')',

    multiColumnSort: true,
    plugins: 'gridfilters',

    emptyText: gettext('No data in database.'),
    viewConfig: {
	deferEmptyText: false
    },

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    store: {
	type: 'pmgStatStore',
	staturl: '/api2/json/statistics/sender',
	remoteSort: true,
	remoteFilter: true,
	fields: [
	    'sender',
	    { type: 'integer', name: 'count' },
	    { type: 'integer', name: 'bytes' },
	    { type: 'integer', name: 'viruscount' }
	],
	proxy: {
	    type: 'pmgfilterproxy',
	    sortParam: 'orderby',
	    filterId: 'x-gridfilter-sender'
	},
	sorters: [
	    {
		property: 'count',
		direction: 'DESC'
	    },
	    {
		property: 'bytes',
		direction: 'DESC'
	    },
	    {
		property: 'sender',
		direction: 'ASC'
	    }
	]
    },

    columns: [
	{
	    text: gettext('Sender'),
	    flex: 1,
	    renderer: Ext.htmlEncode,
	    dataIndex: 'sender',
	    filter: {
		type: 'string',
		itemDefaults: {
		    // any Ext.form.field.Text configs accepted
		}
	    }
	},
	{
	    text: gettext('Count'),
	    columns: [
		{
		    text: gettext('Mail'),
		    dataIndex: 'count'
		},
		{
		    header: gettext('Virus'),
		    dataIndex: 'viruscount'
		}
	    ]
	},
	{
	    text: gettext('Size') + ' (KB)',
	    dataIndex: 'bytes',
	    renderer: function(v) {
		return Ext.Number.toFixed(v/1024, 0);
	    }
	}
    ],

    initComponent: function() {
	var me = this;
	me.callParent();

	Proxmox.Utils.monStoreErrors(me, me.store, true);
    }
});

Ext.define('PMG.SenderStatistics', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgSenderStatistics',

    layout: 'border',
    border: false,
    defaults: {
	border: false
    },

    controller: {
	xclass: 'Ext.app.ViewController',

	selectionChange: function(grid, selected, eOpts) {
	    var details =  this.lookupReference('details');
	    if (selected.length > 0) {
		var sender = selected[0].data.sender;
		var url = "/api2/json/statistics/sender/" +
		    encodeURIComponent(sender);
		details.setUrl(url, '<b>' + gettext('Sender') + ':</b> ' + Ext.htmlEncode(sender));
	    } else {
		details.setUrl();
	    }
	}
    },

    items: [
	{
	    xtype: 'pmgSenderList',
	    multiColumnSort: true,
	    region: 'center',
	    layout: 'fit',
	    flex: 1,

	    listeners: { selectionchange: 'selectionChange' }
	},
	{
	    xtype: 'pmgSenderDetails',
	    region: 'east',
	    reference: 'details',
	    split: true,
	    flex: 1
	}
    ]
});
/*global Proxmox*/
/*jslint confusion: true*/
/* filter is a function and object */
Ext.define('PMG.ReceiverDetails', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgReceiverDetails',

    dockedItems: [
	{
	    dock: 'top',
	    xtype: 'panel',
	    itemId: 'info',
	    bodyPadding: 10,
	    html: gettext('Please select a receiver.')
	}
    ],

    disableSelection: true,

    plugins: 'gridfilters',

    setUrl: function(url, title) {
	var me = this;

	me.store.setUrl(url);
	me.store.setRemoteFilter(url !== undefined);
	Proxmox.Utils.setErrorMask(me, false);
	me.store.reload();

	var infopanel = me.getComponent('info');
	if (title) {
	    infopanel.update(title);
	} else {
	    infopanel.update(gettext('Please select a sender.'));
	}
    },

    store: {
	type: 'pmgStatStore',
	autoReload: false,
	remoteSort: true,
	remoteFilter: false, // enabled dynamically
	fields: [
	    'sender', 'virusinfo',
	    { type: 'integer', name: 'bytes' },
	    { type: 'boolean', name: 'blocked' },
	    { type: 'integer', name: 'spamlevel' },
            { type: 'date', dateFormat: 'timestamp', name: 'time' }
	],
	proxy: {
	    type: 'pmgfilterproxy',
	    filterId: 'x-gridfilter-sender',
	    sortParam: 'orderby'
	},
	sorters: [
	    {
		property: 'time',
		direction: 'ASC'
	    }
	]
    },

    columns: [
	{
	    text: gettext('Sender'),
	    renderer: Ext.htmlEncode,
	    flex: 1,
	    filter: { type: 'string' },
	    dataIndex: 'sender'
	},
	{
	    header: gettext('Size') + ' (KB)',
	    renderer: function(v) { return Ext.Number.toFixed(v/1024, 0); },
	    dataIndex: 'bytes'
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Date'),
	    format: 'Y-m-d',
	    dataIndex: 'time'
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Time'),
	    format: 'H:i:s',
	    dataIndex: 'time'
	},
	{
	    header: gettext('Virus info'),
	    dataIndex: 'virusinfo'
	},
	{
	    header: gettext('Score'),
	    dataIndex: 'spamlevel'
	}
    ],

    initComponent: function() {
	var me = this;
	me.callParent();

	Proxmox.Utils.monStoreErrors(me, me.store, true);
    }
});

Ext.define('PMG.ReceiverList', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgReceiverList',

    title: gettext('Statistics') + ': ' + gettext('Receiver') +
	' (' + gettext('Incoming') +')',

    multiColumnSort: true,
    plugins: 'gridfilters',

    emptyText: gettext('No data in database.'),
    viewConfig: {
	deferEmptyText: false
    },

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    store: {
	type: 'pmgStatStore',
	staturl: '/api2/json/statistics/receiver',
	remoteSort: true,
	remoteFilter: true,
	fields: [
	    'receiver',
	    { type: 'integer', name: 'count' },
	    { type: 'integer', name: 'bytes' },
	    { type: 'integer', name: 'viruscount' }
	],
	proxy: {
	    type: 'pmgfilterproxy',
	    sortParam: 'orderby',
	    filterId: 'x-gridfilter-receiver'
	},
	sorters: [
	    {
		property: 'count',
		direction: 'DESC'
	    },
	    {
		property: 'bytes',
		direction: 'DESC'
	    },
	    {
		property: 'receiver',
		direction: 'ASC'
	    }
	]
    },

    columns: [
	{
	    text: gettext('Receiver'),
	    flex: 1,
	    renderer: Ext.htmlEncode,
	    dataIndex: 'receiver',
	    filter: {
		type: 'string',
		itemDefaults: {
		    // any Ext.form.field.Text configs accepted
		}
	    }
	},
	{
	    text: gettext('Count'),
	    columns: [
		{
		    text: gettext('Mail'),
		    dataIndex: 'count'
		},
		{
		    header: gettext('Virus'),
		    dataIndex: 'viruscount'
		},
		{
		    header: gettext('Spam'),
		    dataIndex: 'spamcount'
		}
	    ]
	},
	{
	    text: gettext('Size') + ' (KB)',
	    dataIndex: 'bytes',
	    renderer: function(v) {
		return Ext.Number.toFixed(v/1024, 0);
	    }
	}
   ],

    initComponent: function() {
	var me = this;
	me.callParent();

	Proxmox.Utils.monStoreErrors(me, me.store, true);
    }
});

Ext.define('PMG.ReceiverStatistics', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgReceiverStatistics',

    layout: 'border',
    border: false,
    defaults: {
	border: false
    },

    controller: {
	xclass: 'Ext.app.ViewController',

	selectionChange: function(grid, selected, eOpts) {
	    var details =  this.lookupReference('details');
	    if (selected.length > 0) {
		var receiver = selected[0].data.receiver;
		var url = "/api2/json/statistics/receiver/" +
		    encodeURIComponent(receiver);
		details.setUrl(url, '<b>' + gettext('Receiver') + ':</b> ' + Ext.htmlEncode(receiver));
	    } else {
		details.setUrl();
	    }
	}
    },

    items: [
	{
	    xtype: 'pmgReceiverList',
	    multiColumnSort: true,
	    region: 'center',
	    layout: 'fit',
	    flex: 1,

	    listeners: { selectionchange: 'selectionChange' }
	},
	{
	    xtype: 'pmgReceiverDetails',
	    region: 'east',
	    reference: 'details',
	    split: true,
	    flex: 1
	}
    ]
});
/*global Proxmox*/
/*jslint confusion: true*/
/* filter is a function and object */
Ext.define('PMG.ContactDetails', {
    extend: 'Ext.grid.GridPanel',
    xtype: 'pmgContactDetails',

    dockedItems: [
	{
	    dock: 'top',
	    xtype: 'panel',
	    itemId: 'info',
	    bodyPadding: 10,
	    html: gettext('Please select a contact.')
	}
    ],

    disableSelection: true,

    plugins: 'gridfilters',

    setUrl: function(url, title) {
	var me = this;

	me.store.setUrl(url);
	me.store.setRemoteFilter(url !== undefined);
	Proxmox.Utils.setErrorMask(me, false);
	me.store.reload();

	var infopanel = me.getComponent('info');
	if (title) {
	    infopanel.update(title);
	} else {
	    infopanel.update(gettext('Please select a contact'));
	}
    },

    store: {
	type: 'pmgStatStore',
	autoReload: false,
	remoteSort: true,
	remoteFilter: false, // enabled dynamically
	fields: [
	    'sender', 'virusinfo',
	    { type: 'integer', name: 'bytes' },
	    { type: 'boolean', name: 'blocked' },
	    { type: 'integer', name: 'spamlevel' },
            { type: 'date', dateFormat: 'timestamp', name: 'time' }
	],
	proxy: {
	    type: 'pmgfilterproxy',
	    filterId: 'x-gridfilter-sender',
	    sortParam: 'orderby'
	},
	sorters: [
	    {
		property: 'time',
		direction: 'ASC'
	    }
	]
    },

    columns: [
	{
	    text: gettext('Sender'),
	    renderer: Ext.htmlEncode,
	    flex: 1,
	    filter: { type: 'string' },
	    dataIndex: 'sender'
	},
	{
	    header: gettext('Size') + ' (KB)',
	    renderer: function(v) { return Ext.Number.toFixed(v/1024, 0); },
	    dataIndex: 'bytes'
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Date'),
	    format: 'Y-m-d',
	    dataIndex: 'time'
	},
	{
	    xtype: 'datecolumn',
	    header: gettext('Time'),
	    format: 'H:i:s',
	    dataIndex: 'time'
	}
    ],

    initComponent: function() {
	var me = this;
	me.callParent();

	Proxmox.Utils.monStoreErrors(me, me.store, true);
    }
});

Ext.define('PMG.ContactList', {
    extend: 'Ext.grid.GridPanel',
    alias: 'widget.pmgContactList',

    title: gettext('Statistics') + ': ' + gettext('Contact') +
	' (' + gettext('Receiver') + ', ' + gettext('Outgoing') +')',

    multiColumnSort: true,
    plugins: 'gridfilters',

    emptyText: gettext('No data in database.'),
    viewConfig: {
	deferEmptyText: false
    },

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    store: {
	type: 'pmgStatStore',
	staturl: '/api2/json/statistics/contact',
	remoteSort: true,
	remoteFilter: true,
	fields: [
	    'contact',
	    { type: 'integer', name: 'count' },
	    { type: 'integer', name: 'viruscount' },
	    { type: 'integer', name: 'bytes' }
	],
	proxy: {
	    type: 'pmgfilterproxy',
	    sortParam: 'orderby',
	    filterId: 'x-gridfilter-contact'
	},
	sorters: [
	    {
		property: 'count',
		direction: 'DESC'
	    },
	    {
		property: 'bytes',
		direction: 'DESC'
	    },
	    {
		property: 'contact',
		direction: 'ASC'
	    }
	]
    },

    columns: [
	{
	    text: gettext('Contact'),
	    flex: 1,
	    renderer: Ext.htmlEncode,
	    dataIndex: 'contact',
	    filter: {
		type: 'string',
		itemDefaults: {
		    // any Ext.form.field.Text configs accepted
		}
	    }
	},
	{
	    text: gettext('Count'),
	    columns: [
		{
		    text: gettext('Mail'),
		    dataIndex: 'count'
		},
		{
		    header: gettext('Virus'),
		    dataIndex: 'viruscount'
		}
	    ]
	},
	{
	    text: gettext('Size') + ' (KB)',
	    dataIndex: 'bytes',
	    renderer: function(v) {
		return Ext.Number.toFixed(v/1024, 0);
	    }
	}
    ],

    initComponent: function() {
	var me = this;
	me.callParent();

	Proxmox.Utils.monStoreErrors(me, me.store, true);
    }
});

Ext.define('PMG.ContactStatistics', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgContactStatistics',

    layout: 'border',
    border: false,
    defaults: {
	border: false
    },

    controller: {
	xclass: 'Ext.app.ViewController',

	selectionChange: function(grid, selected, eOpts) {
	    var details =  this.lookupReference('details');
	    if (selected.length > 0) {
		var contact = selected[0].data.contact;
		var url = "/api2/json/statistics/contact/" +
		    encodeURIComponent(contact);
		details.setUrl(url, '<b>' + gettext('Contact') + ':</b> ' + Ext.htmlEncode(contact));
	    } else {
		details.setUrl();
	    }
	}
    },

    items: [
	{
	    xtype: 'pmgContactList',
	    multiColumnSort: true,
	    region: 'center',
	    layout: 'fit',
	    flex: 1,

	    listeners: { selectionchange: 'selectionChange' }
	},
	{
	    xtype: 'pmgContactDetails',
	    region: 'east',
	    reference: 'details',
	    split: true,
	    flex: 1
	}
    ]
});
Ext.define('PMG.MailDistChart', {
    extend: 'Ext.chart.CartesianChart',
    xtype: 'pmgMailDistChart',

    width: 770,
    height: 300,
    animation: false,

    axes: [{
	type: 'numeric',
	title: gettext('Count'),
	position: 'left',
	grid: true,
	minimum: 0
    }, {
	type: 'category',
	position: 'bottom',
	fields: ['index']
    }],

    initComponent: function() {
	var me = this;


	if (!me.store) {
	    throw "cannot work without store";
	}

	if (!me.field) {
	    throw "cannot work without a field";
	}

	me.callParent();

	me.addSeries({
	    type: 'bar',
	    xField: 'index',
	    yField: me.field,
	    tooltip: {
		trackMouse: true,
		renderer: function(tooltip, record, item) {
		    var start = record.get('index');
		    var end = start+1;
		    tooltip.setHtml('Time: ' + start.toString() +
				    ' - ' + end.toString() + '<br>' +
				    'Count: ' + record.get(item.field));
		}
	    }
	});
    }
});

Ext.define('PMG.HourlyMailDistribution', {
    extend: 'Ext.panel.Panel',
    xtype: 'pmgHourlyMailDistribution',

    scrollable: true,
    border: false,

    bodyPadding: '10 0 0 0',
    defaults: {
	width: 700,
	padding: '0 0 10 10'
    },

    layout: 'column',

    title: gettext('Statistics') + ': ' + gettext('Hourly Distribution'),

    tbar: [ { xtype: 'pmgStatTimeSelector' } ],

    initComponent: function() {
	var me = this;

	var store = Ext.create('PMG.data.StatStore', {
	    staturl: '/api2/json/statistics/maildistribution',
	    fields: [
		{ type: 'integer', name: 'index' },
		{ type: 'integer', name: 'count' },
		{ type: 'integer', name: 'count_in' },
		{ type: 'integer', name: 'count_out' },
		{ type: 'integer', name: 'spamcount_in' },
		{ type: 'integer', name: 'spamcount_out' },
		{ type: 'integer', name: 'viruscount_in' },
		{ type: 'integer', name: 'viruscount_ou' },
		{ type: 'integer', name: 'bounces_in' },
		{ type: 'integer', name: 'bounces_out' }
	    ]
	});

	me.items = [
	    {
		xtype: 'pmgMailDistChart',
		title: gettext('Incoming Mails'),
		field: 'count_in',
		store: store
	    },
	    {
		xtype: 'pmgMailDistChart',
		title: gettext('Outgoing Mails'),
		field: 'count_out',
		store: store
	    }
	];

	me.callParent();

	me.on('destroy', store.destroy, store);
    }
});
/*global Proxmox*/
Ext.define('PMG.Application', {
    extend: 'Ext.app.Application',

    name: 'PMG',
    appProperty: 'app',

    stores: [
	'NavigationStore'
    ],

    layout: 'fit',

    realignWindows: function() {
	var modalwindows = Ext.ComponentQuery.query('window[modal]');
	Ext.Array.forEach(modalwindows, function(item) {
	    item.center();
	});
    },

    logout: function() {
	var me = this;
	Proxmox.Utils.authClear();
	me.changeView('loginview', true);
    },

    changeView: function(view, skipCheck) {
	var me = this;
	PMG.view = view;
	me.view = view;
	me.currentView.destroy();
	me.currentView = Ext.create({
	    xtype: view,
	    targetview: me.targetview
	});
	if (skipCheck !== true) {
	    Proxmox.Utils.checked_command(function() {}); // display subscription status
	}
    },

    view: 'loginview',
    targetview: 'mainview',

    launch: function() {
	var me = this;
	Ext.on('resize', me.realignWindows);

	var provider = new Ext.state.LocalStorageProvider({ prefix: 'ext-pmg-' });
	Ext.state.Manager.setProvider(provider);

	// show login window if not loggedin
	var loggedin = Proxmox.Utils.authOK();
	var cookie = Ext.util.Cookies.get(Proxmox.Setup.auth_cookie_name);
	var qs = Ext.Object.fromQueryString(location.search);

	var pathname = location.pathname.replace(/\/+$/, '');

	if (pathname === "/quarantine") {
	    me.targetview = 'quarantineview';

	    if (qs.ticket == undefined && loggedin) {
		me.view = 'quarantineview';
	    }
	} else if (loggedin && cookie.substr(0, 7) !== 'PMGQUAR') {
	    me.view = 'mainview';
	}

	PMG.view = me.view;
	me.currentView = Ext.create({
	    xtype: me.view,
	    targetview: me.targetview
	});
    }
});

Ext.application('PMG.Application');
